package com.starbuds.server.api.app.provider;

import com.starbuds.server.api.app.config.GameDevConfig;
import com.starbuds.server.api.app.pojo.GameDevResult;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.GameCoinOrderApiPlatformEnum;
import com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.finance.GameService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.game.sdkdev.GameDevResponse;
import com.starbuds.server.game.sdkdev.param.TransferParam;
import com.starbuds.server.game.sdkdev.request.TransferRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import static com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum.IN;

/**
 * 游戏Dev
 *
 * @author aoe
 * @date 2020/11/16
 */
@Service
@Slf4j
public class GameDevProvider {

    @Resource
    private GameDevConfig gameDevConfig;

    @Reference(version = "1.0.0")
    private GameService gameService;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    /**
     * 添加游戏币订单(划转资产，转入/转出)
     *
     * @param operateEnum 操作（10:转入;20:转出）
     * @param coin        游戏币数量
     * @return null:成功
     */
    public GameDevResult addGameCoinOrder(GameCoinOrderOperateEnum operateEnum, Long coin, Long userId) {
        GameDevResult orderResult = new GameDevResult();
        orderResult.setIsSuccess(false);

        if (IN == operateEnum) {
            Long balance = walletService.getBalance(userId, WalletTypeEnum.GAME_COIN);
            // 转入在途总金额
            Long inOnWayTotal = gameService.getInOnWayTotal(userId);
            if (coin > balance - inOnWayTotal) {
                orderResult.setErrMsg("余额不足");
                return orderResult;
            }
        }

        // 保存订单
        Long orderId = gameService.addGameCoinOrder(userId, "dev", operateEnum,
                GameCoinOrderApiPlatformEnum.DEV, coin);
        if (orderId == null) {
            orderResult.setErrMsg("保存订单失败");
            return orderResult;
        }

        // 转出时，调用第三方接口游戏币转RMB
        TransferParam param = TransferParam.builder()
                .userid(userId + "")
                // 类型 1转入 2转出
                .type(IN == operateEnum ? "1" : "2")
                // 金额，整数，最小1
                .amount(coin + "")
                .orderNo(orderId + "")
                .build();
        TransferRequest request = new TransferRequest();
        request.setParam(param);
        Result result = getResult(gameDevConfig.client().execute(request));
        if (!result.isSuccess()) {
            // 更新订单
            gameService.updateGameCoinOrder(orderId, false, result.getMsg());
            orderResult.setErrMsg(result.getMsg());
            return orderResult;
        }

        // 更新订单
        boolean flag = gameService.updateGameCoinOrder(orderId, true, "第三方接口返回成功");
        if (!flag) {
            orderResult.setErrMsg("更新订单失败");
            return orderResult;
        }

        orderResult.setIsSuccess(true);
        orderResult.setOrderId(orderId);
        return orderResult;
    }

    public Result getResult(GameDevResponse res) {
        if (isSuccess(res.getCode())) {
            Result result = new Result(res.getData());
            result.setMsg(res.getMessage());
            return result;
        }
        return new Result(GlobalApiError.OperationFailed, res.getMessage());
    }

    /**
     * 请求是否成功
     *
     * @param code
     * @return true 成功
     */
    private boolean isSuccess(String code) {
        return "0000".equals(code);
    }
}
