package com.starbuds.server.api.app.provider;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.user.SexEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.NearUserDTO;
import com.starbuds.server.common.pojo.data.NearbyUser;
import com.starbuds.server.common.pojo.db.UserSwitchDO;
import com.starbuds.server.common.pojo.user.PeopleNearbyVO;
import com.starbuds.server.common.pojo.util.DistanceUtil;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.feed.FeedService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 附近的人
 *
 * @author aoe
 * @date 2020/12/18
 */
@Service
@Slf4j
public class PeopleNearbyProvider {

    @Reference(version = "1.0.0")
    private UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    private FeedService feedService;

    /**
     * 获取附近的人分页数据
     *
     * @param userDTO
     * @return
     */
    public PageList<PeopleNearbyVO> getPeopleNearbyPageList(NearUserDTO userDTO) {
        userDTO.setDistance(userService.getPeopleNearbyDistance());

        String myLocationCity = userService.getBasicInfo(userDTO.getMyselfUserId()).getLocationCity();

        PageList<NearbyUser> pageList = userSearchService.getNearUserPageList(userDTO);
        PageList<PeopleNearbyVO> result = new PageList();
        if (CollectionUtil.isEmpty(pageList.getList())) {
            return result;
        }

        List<NearbyUser> userList = pageList.getList().stream()
                .filter(user -> user != null)
                .filter(user -> user.getUserId() != null)
                .collect(Collectors.toList());
        Set<Long> userIds = userList.stream().map(NearbyUser::getUserId).collect(Collectors.toSet());
        Map<Long, PeopleNearbyVO> userCardVOMap = userService.getUserMapByByIds(userIds);
        // 照片墙
        Map<Long, List<String>> photoWallMap = feedService.getPhotoWallByUserIds(userIds);
        // 账号定位
        Map<Long, UserSwitchDO> switchMap = userService.getSwitchMap(userIds, SwitchTypeEnum.ACCOUNT_LOCATION);
        String[] mySplit = null;
        if (StringUtils.isNotEmpty(myLocationCity)) {
            mySplit = myLocationCity.split("-");
        }

        List<PeopleNearbyVO> list = new ArrayList<>();
        BeanUtils.copyProperties(pageList, result);

        long curTime = System.currentTimeMillis();

        String[] finalMySplit = mySplit;
        pageList.getList().stream().forEach(user -> {
            Long userId = user.getUserId();
            if (userDTO.getMyselfUserId() != null && userDTO.getMyselfUserId().equals(userId)) {
                // 不显示自己
                return;
            }
            PeopleNearbyVO vo = userCardVOMap.get(userId);
            if (vo == null) {
                return;
            }

            UserSwitchDO userSwitchDO = CollectionUtil.isEmpty(switchMap) ? null : switchMap.get(userId);
            if (userSwitchDO != null && !Optional.ofNullable(userSwitchDO.getSwitchStatus()).map(e -> e == BoolIntVal.True).orElse(true)) {
                return;
            }

            vo.setDistance(DistanceUtil.getDistancePretty(user.getDistance()));
            vo.setPhotoWalls(Optional.ofNullable(photoWallMap.get(userId)).orElse(Collections.EMPTY_LIST));

            // 用户定位城市
            String userLocationCity = user.getLocationCity();
            if (StringUtils.isNotEmpty(userLocationCity)) {
                String[] split = userLocationCity.split("-");
                // 默认显示城市
                if (split.length > 2) {
                    vo.setLocationCity(split[2]);
                } else {
                    vo.setLocationCity(userLocationCity);
                }
                // 若同一个城市，则显示到区
                if (split.length == 4 && StringUtils.isNotEmpty(myLocationCity) && finalMySplit.length == 4 && split[2].equals(finalMySplit[2])) {
                    vo.setLocationCity(split[3]);
                }
            }

            // 没有数据时设置默认值
            if (StringUtils.isBlank(vo.getLocationCity())) {
                vo.setLocationCity("未知");
            }
            if (vo.getUserSex() == null || SexEnum.UNKNOWN.getCode() - vo.getUserSex() == 0) {
                vo.setUserSex(SexEnum.FEMALE.getCode());
            }
            if (vo.getUserAge() == null) {
                vo.setUserAge(18);
            }

            // 用户在线状态
            Long userUpdateTime = vo.getUserUpdateTime();
            Integer onLine = Optional.ofNullable(vo.getIsOnline()).orElse(BoolIntVal.False);

            if (BoolIntVal.True == onLine) {
                vo.setOnLineStr("在线");
            } else if (userUpdateTime != null && (curTime - userUpdateTime) < 24 * 3600 * 1000L) {
                vo.setOnLineStr("刚刚");
            } else {
                vo.setOnLineStr("最近");
            }

            list.add(vo);
        });
        result.setList(rtcRoomService.setRtcRoomInfo(list));

        return result;
    }

}
