package com.starbuds.server.api.app.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import com.starbuds.server.api.app.pojo.gift.SendGiftForm;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.finance.gift.GiftReceiver;
import com.starbuds.server.common.pojo.finance.gift.SendGiftByFinanceAbstract;
import com.starbuds.server.common.pojo.finance.gift.SendGiftOrder;
import com.starbuds.server.common.pojo.media.GiftPayItem;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.SendGiftService;
import com.starbuds.server.common.service.finance.SendGiftServiceFromBagService;
import com.starbuds.server.common.service.finance.SendGiftServicePayDiamondService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;
import static com.starbuds.server.common.utils.CheckFormUtil.checkCreate;

/**
 * 赠送礼物
 *
 * @author aoe
 * @date 2021/11/16
 */
@Component
public class SendGiftProvider {

    @Reference(version = "1.0.0")
    private SendGiftServiceFromBagService sendGiftServiceFromBag;

    @Reference(version = "1.0.0")
    private SendGiftServicePayDiamondService sendGiftServicePayDiamond;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private BagService bagService;

    /**
     * 发送礼物
     *
     * @param form           语音房赠送礼物表单
     * @param sendGift       发送礼物业务对象
     * @param customizeCheck 自定义检查规则 💪封装变化
     * @return
     */
    public Result sendGift2Result(SendGiftForm form, SendGiftByFinanceAbstract sendGift, Supplier<Result> customizeCheck,
                                  Long senderUserId, Long channelId) {
        // 检查赠送礼物各项参数
        Long giftId = form.getGiftId();
        GiftPayItem giftPayItem = giftId != null ? giftService.getPayItem(giftId) : null;
        Result check = check(form, sendGift, customizeCheck, senderUserId, giftPayItem);
        if (check != null) {
            return check;
        }
        SendGiftOrder order = sendGift(form, sendGift, senderUserId, giftPayItem, channelId);

        if (order == null) {
            if (!form.getIsFromBag()) {
                return needRecharge(giftPayItem.getGiftPriceType());
            }
            return Result.ofOperationFailed("礼物赠送失败");
        }

        MutableResult mResult = new MutableResult();
        if (form.getIsFromBag()) {
            // 从背包赠送
            mResult.put("quantity", order.getQuantity());
        } else {
            // 消耗星钻赠送
            mResult.put("balance", order.getBalance());
            mResult.put("walletType", order.getWalletType().getValue());
        }
        if (isDev()) {
            mResult.put("orderId", order.getOrderId());
        }
        return mResult.build();
    }

    /**
     * 需要充值
     *
     * @param walletType
     * @return
     */
    public Result needRecharge(Integer walletType) {
        MutableResult mResult = new MutableResult();
        // 消耗星钻赠送，余额不足
        mResult.put("balance", -1);
        mResult.put("walletType", walletType);
        return mResult.build();
    }

    /**
     * 检查送礼物数据
     *
     * @param form
     * @param sendGift
     * @param customizeCheck
     * @param senderUserId
     * @param giftPayItem
     * @return
     */
    private Result check(SendGiftForm form, SendGiftByFinanceAbstract sendGift, Supplier<Result> customizeCheck,
                         Long senderUserId, GiftPayItem giftPayItem) {
        // 检查赠送礼物各项参数
        Long giftId = form.getGiftId();
        if (giftId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的礼物参数");
        }
        // 检查礼物基础信息
        Result check = checkGiftBase(form, giftPayItem, senderUserId);
        if (check != null) {
            return check;
        }

        // 检查自定义规则
        check = customizeCheck.get();
        if (check != null) {
            return check;
        }
        // 检查从背包或者消耗钻石赠送礼物的条件
        return checkFromBagOrDiamond(form, senderUserId, giftPayItem);
    }

    private SendGiftOrder sendGift(SendGiftForm form, SendGiftByFinanceAbstract sendGift, Long senderUserId,
                                   GiftPayItem giftPayItem, Long channelId) {
        // 建造发送礼物业务对象
        sendGift = buildSendGift(form, sendGift, giftPayItem, senderUserId, channelId);
        sendGift.setIsFromBag(form.getIsFromBag());
        if (form.getIsFromBag()) {
            // 从背包赠送
            return sendGiftServiceFromBag.sendGift(sendGift);
        } else {
            // 支付礼物
            return sendGiftServicePayDiamond.sendGift(sendGift);
        }
    }

    /**
     * 建造发送礼物业务对象
     *
     * @param form
     * @param sendGift
     * @param giftPayItem
     * @return
     */
    private SendGiftByFinanceAbstract buildSendGift(SendGiftForm form, SendGiftByFinanceAbstract sendGift,
                                                    GiftPayItem giftPayItem, Long senderUserId, Long channelId) {
        sendGift.setSenderUserId(senderUserId);
        UserCardVO sender = userService.getCard(senderUserId);
        sendGift.setSenderUserName(sender.getUserName());
        sendGift.setSenderUserAvatar(sender.getUserAvatar());

        // 获取接收用户信息
        Set<Long> receiverUserIds = Arrays.stream(form.getUserIds().split(","))
                .map(Long::parseLong)
                .collect(Collectors.toSet());
        List<UserCardVO> receiverUserVOs = userService.getNameAndAvatarByIds(receiverUserIds);
        Set<GiftReceiver> receiverUsers = new HashSet<>();
        receiverUserVOs.stream()
                .forEach(vo -> {
                    GiftReceiver receiver = new GiftReceiver();
                    receiver.setUserId(vo.getUserId());
                    receiver.setUserName(vo.getUserName());
                    receiver.setAvatar(vo.getUserAvatar());
                    receiverUsers.add(receiver);
                });
        sendGift.setReceiverUsers(receiverUsers);

        sendGift.setGiftId(form.getGiftId());
        sendGift.setGiftType(giftPayItem.getGiftType());
        sendGift.setGiftName(giftPayItem.getGiftName());
        sendGift.setGiftPriceType(giftPayItem.getGiftPriceType());
        sendGift.setGiftPrice(giftPayItem.getGiftPrice());
        sendGift.setQuantity(form.getQuantity());
        sendGift.setChannelId(channelId);
        return sendGift;
    }

    /**
     * 检查赠送礼物各项参数
     *
     * @param form
     * @return null:参数正确
     */
    private Result checkGiftBase(SendGiftForm form, GiftPayItem giftPayItem, Long senderUserId) {
        String check = checkCreate(form);
        if (check != null) {
            return new Result(GlobalApiError.BadRequest, check);
        }

        // 判断礼物数量
        Integer quantity = form.getQuantity();
        if (quantity < 1) {
            return new Result(GlobalApiError.BadRequest, "赠送数量必须是大于0");
        }
        if (quantity > 9999) {
            return new Result(GlobalApiError.BadRequest, "单次最多赠送9999个哦");
        }

        // 验证接收礼物的用户
        String userIds = form.getUserIds();
        List<String> userIdList = !userIds.contains(",") ? Arrays.asList(userIds) :
                Arrays.stream(userIds.split(","))
                        .filter(id -> id == null)
                        .collect(Collectors.toList());

        if (userIdList.stream().anyMatch(id -> !NumberUtil.isLong(id))) {
            return new Result(GlobalApiError.BadRequest, "不能包含无效的用户id");
        }

        Set<Long> userIdSet = Arrays.stream(userIds.split(",")).map(Convert::toLong).collect(Collectors.toSet());
        if (CollectionUtil.isEmpty(userIdSet)) {
            return new Result(GlobalApiError.OperationFailed, "请选择要赠送的用户");
        }

        if (userIdSet.contains(senderUserId)) {
            return new Result(GlobalApiError.BadRequest, "不能给自己赠送礼物哦");
        }

        // 判断礼物是否有效
        if (giftPayItem == null) {
            return new Result(GlobalApiError.BadRequest, "无效的礼物或礼物已过期");
        }
        // 判断礼物类型
        GiftTypeEnum giftTypeEnum = GiftTypeEnum.toEnum(giftPayItem.getGiftType());
        if (GiftTypeEnum.COMMON != giftTypeEnum && GiftTypeEnum.LOTTERY != giftTypeEnum) {
            return new Result(GlobalApiError.BadRequest, "暂不支持该礼物类型");
        }
        // 目前仅支持普通星钻礼物
        if (WalletTypeEnum.STAR_DIAMOND.getValue() != giftPayItem.getGiftPriceType()) {
            return new Result(GlobalApiError.BadRequest, "目前不支持该礼物");
        }
        return null;
    }

    /**
     * 检查从背包或者消耗钻石赠送礼物的条件
     *
     * @param form
     * @param senderUserId
     * @param giftPayItem
     * @return
     */
    private Result checkFromBagOrDiamond(SendGiftForm form, Long senderUserId, GiftPayItem giftPayItem) {
        Long giftId = form.getGiftId();
        // 礼物信息 getHeader().getChannelId()
        Integer quantity = form.getQuantity();
        int receiverUserCount = (int) Arrays.stream(form.getUserIds().split(","))
                .filter(id -> id != null)
                .count();
        Integer giftTotalQuantity = quantity * receiverUserCount;

        if (form.getIsFromBag()) {
            // 从背包赠送
            // 判断背包中是否有该礼物
            if (bagService.getItemQuantity(senderUserId, BagItemTypeEnum.GIFT, giftId) < giftTotalQuantity) {
                return new Result(GlobalApiError.BadRequest, "数量不足");
            }
        } else {
            // 消耗星钻赠送
            GiftTypeEnum giftType = GiftTypeEnum.toEnum(giftPayItem.getGiftType());
            if (GiftTypeEnum.COMMON != giftType && GiftTypeEnum.LOTTERY != giftType) {
                return new Result(GlobalApiError.BadRequest, "不支持的礼物类型");
            }

            // 礼物限制判断
            String conditionCheckResult = giftService.checkGiftConditions(giftId, null, senderUserId);
            if (StringUtils.isNotBlank(conditionCheckResult)) {
                return new Result(GlobalApiError.BadRequest, conditionCheckResult);
            }
        }
        return null;
    }
}
