package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.app.pojo.activity.ActivityBarrageVo;
import com.starbuds.server.api.app.v1.facade.ActivityBarrageFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.ActivityBarrageDO;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.activity.ActivityBarrageService;
import com.starbuds.server.common.service.activity.ActivityGiftsRankService;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.*;

/**
 * 活动弹幕接口实现
 *
 * @author lujiao
 * @date 2021/8/9 15:12
 */
@Path("/v1/activity")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ActivityBarrageFacadeImpl extends BaseFacade implements ActivityBarrageFacade {


    @Reference(version = "1.0.0")
    private WordFilterService wordFilterService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private ActivityBarrageService activityBarrageService;

    @Reference(version = "1.0.0")
    private ActivityGiftsRankService activityGiftsRankService;

    @Reference(version = "1.0.0")
    private ActivityService activityService;

    /**
     * 最大弹幕长度
     */
    private static final Integer MAX_CONTEXT_LENGTH = 30;

    private static final Long RANK_VALUE_JUDGE = 300000L;

    /**
     * 发送弹幕
     *
     * @param context
     * @param activityId
     * @return
     */
    @POST
    @Path("sendBarrage")
    @Override
    public Result sendBarrage(@FormParam("context") String context, @FormParam("activityId") Long activityId) {
        Long userId = this.getToken().getUserId();
        if (StringUtils.isBlank(context)) {
            return new Result(GlobalApiError.OperationFailed, "弹幕内容不能为空");
        }
        if (context.length() > MAX_CONTEXT_LENGTH) {
            return new Result(GlobalApiError.OperationFailed, "弹幕内容过长");
        }
        ActivityDO activity = activityService.getActivity(activityId);
        if (activity == null || activity.getActivityEndTime() < System.currentTimeMillis()) {
            return new Result(GlobalApiError.OperationFailed, "活动已结束");
        }

        long rankValue = activityGiftsRankService.getTotalValueByUser(activityId, userId);
        if (rankValue <= RANK_VALUE_JUDGE) {
            return new Result(GlobalApiError.OperationFailed, "甜蜜值不足300000，无法发送表白弹幕");
        }
        ActivityBarrageDO lastOne = activityBarrageService.getLastBarrage(userId, activityId);
        if (lastOne != null && (System.currentTimeMillis() - lastOne.getCreateTime()) < DateUtil.DAY_OF_MILLION) {
            return new Result(GlobalApiError.OperationFailed, "24小时内只能发送一条弹幕哦");
        }
        if (wordFilterService.isMatchedPhrases(WordFilterTypeEnum.COMMON, context)) {
            return new Result(GlobalApiError.OperationFailed, "弹幕包含敏感词，无法发送");
        }
        activityBarrageService.sendBarrage(userId, context, activityId);
        return new Result();
    }

    /**
     * 获取弹幕
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getBarrage")
    @Override
    public Result getBarrage(@QueryParam("activityId") Long activityId) {
        Long userId = this.getToken().getUserId();
        List<ActivityBarrageDO> barrageDOList = activityBarrageService.getBarrageList(userId, activityId);
        if (CollectionUtil.isNotEmpty(barrageDOList)) {
            List<ActivityBarrageVo> res = new ArrayList<>();
            Set<Long> userIds = new HashSet<>();
            barrageDOList.forEach(i -> userIds.add(i.getUserId()));
            Map<Long, UserBasicInfo> userMap = userService.getBasicInfoMap(userIds);
            barrageDOList.forEach(i -> {
                UserBasicInfo userBasicInfo = userMap.get(i.getUserId());
                if (userBasicInfo != null) {
                    ActivityBarrageVo tmp = new ActivityBarrageVo();
                    tmp.setContext(i.getContext());
                    tmp.setAvatar(userBasicInfo.getUserAvatar());
                    tmp.setUserId(i.getUserId());
                    if (i.getUserId().equals(userId)) {
                        tmp.setIsSelf(BoolIntVal.True);
                    } else {
                        tmp.setIsSelf(BoolIntVal.False);
                    }
                    res.add(tmp);
                }
            });
            return new Result(res);
        }
        return new Result(new ArrayList<>());
    }

    /**
     * 创建弹幕
     *
     * @param sendId
     * @param receiveId
     * @param giftRank
     * @return
     */
    @POST
    @Path("createBradCast")
    @Override
    public Result createBradCast(@FormParam("sendId") Long sendId, @FormParam("receiveId") Long receiveId, @FormParam("giftRank") Long giftRank) {
        activityBarrageService.sendActivityRankBroadcastTest(sendId, receiveId, giftRank);
        return new Result();
    }
}
