package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ReUtil;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.v1.facade.AnchorFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.user.*;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.DataForbiddenWordOptionVO;
import com.starbuds.server.common.pojo.db.AnchorDO;
import com.starbuds.server.common.pojo.user.LiveUserProfile;
import com.starbuds.server.common.pojo.user.RoomVO;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.user.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.math.BigDecimal;
import java.util.Optional;

/**
 * 主播接口实现
 *
 * @author jiangxiaowei
 * @date 2019/12/23 14:54
 */
@Path("/v1/anchor")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class AnchorFacadeImpl extends BaseFacade implements AnchorFacade {


    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    AgentService agentService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    WordFilterService wordFilterService;

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    /**
     * 发起主播认证申请
     *
     * @param idCardHoldImg
     * @param idCardFaceImg
     * @param idCardNationImg
     * @return
     */
    @POST
    @Path("addApply")
    @Override
    public Result addApply(@FormParam("idCardHoldImg") String idCardHoldImg,
                           @FormParam("idCardFaceImg") String idCardFaceImg,
                           @FormParam("idCardNationImg") String idCardNationImg) {
        Long userId = this.getToken().getUserId();

        // 判断是否主播
        if (anchorService.isAnchor(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "您已经是认证主播了，快去开播吧~");
        }

        // 判断参数
        if (StringUtils.isBlank(idCardHoldImg) || StringUtils.isBlank(idCardFaceImg) || StringUtils.isBlank(idCardNationImg)) {
            return new Result(GlobalApiError.OperationFailed, "您还有照片没有上传哦");
        }

        // 判断是否绑定手机号
        if (StringUtils.isEmpty(userService.getPhone(userId))) {
            return new Result(GlobalApiError.ServerError, "认证前请先绑定手机号");
        }

        // 判断是否有进行中的审核记录
        Integer lastApplyStatus = anchorService.getLastApplyStatus(userId);
        if (lastApplyStatus != null) {
            AuditStatusEnum auditStatus = AuditStatusEnum.toEnum(lastApplyStatus);
            if (AuditStatusEnum.WAIT == auditStatus) {
                return new Result(GlobalApiError.OperationFailed, "您有正在审核中的认证申请，请等待审核");
            }
            if (AuditStatusEnum.PASS == auditStatus) {
                return new Result(GlobalApiError.OperationFailed, "您的认证申请已经通过了，快去开播吧~");
            }
        }

        if (anchorService.addApply(this.getToken().getUserId(), idCardHoldImg, idCardFaceImg, idCardNationImg)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取最近一次认证记录
     *
     * @return
     */
    @GET
    @Path("getLastApplyRecord")
    @Override
    public Result getLastApplyRecord() {
        return new Result(anchorService.getLastApplyRecord(this.getToken().getUserId()));
    }

    /**
     * 更新主播信息  直播分类、直播标题、颜值封面
     *
     * @param categoryId
     * @param liveTitle
     * @param liveCover
     * @return
     */
    @POST
    @Path("editAnchorInfo")
    @Override
    public Result editAnchorInfo(@FormParam("categoryId") Long categoryId,
                                 @FormParam("liveTitle") String liveTitle,
                                 @FormParam("liveCover") String liveCover) {
        if (!anchorService.isAnchor(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "请先认证主播");
        }

        AnchorDO anchor = new AnchorDO();
        anchor.setCategoryId(categoryId);
        anchor.setLiveCover(liveCover);
        anchor.setLiveTitle(liveTitle);
        anchor.setLatitude(Convert.toDouble(this.getHeader().getLatitude()));
        anchor.setLongitude(Convert.toDouble(this.getHeader().getLongitude()));
        anchor.setUserId(this.getToken().getUserId());
        if (anchorService.editAnchor(anchor)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置粉丝团名称
     *
     * @param fansName
     * @return
     */
    @POST
    @Path("setFansName")
    @Override
    public Result setFansName(@FormParam("fansName") String fansName) {
        Long userId = this.getToken().getUserId();

        // 判断是否主播
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "请先认证主播");
        }

        // 判断是否已设置粉丝团名称
        if (anchorService.getFansName(userId) != null) {
            return new Result(GlobalApiError.OperationFailed, "您已设置过粉丝团名称");
        }

        // 判断是否名称是否合规
        if (!ReUtil.isMatch("^[\\u4e00-\\u9fa5a-zA-Z0-9]{2,4}$", fansName)) {
            return new Result(GlobalApiError.OperationFailed, "粉丝团名称不符合规范，必须是2到4个中英文数字字符，请重新设置");
        }

        // 敏感词过滤
        if (wordFilterService.isMatchedPhrases(WordFilterTypeEnum.FANS_NAME, fansName)) {
            return new Result(GlobalApiError.OperationFailed, "粉丝团名称中含有敏感词汇，请重新设置");
        }

        // 判断粉丝团名称是否已被占用
        if (anchorService.isFansNameExists(fansName)) {
            return new Result(GlobalApiError.OperationFailed, "您来晚了，该名称已被其他主播使用");
        }

        if (anchorService.setFansName(userId, fansName)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 添加房管
     *
     * @param userId
     * @return
     */
    @POST
    @Path("addMaster")
    @Override
    public Result addMaster(@FormParam("userId") Long userId) {
        Long anchorUserId = this.getToken().getUserId();

        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "请先认证主播");
        }

        // 判断是否自己
        if (userId.equals(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "不能设置自己为房管");
        }

        // 判断用户是否存在
        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.OperationFailed, "用户不存在");
        }

        // 判断是否已是房管
        if (anchorService.isMaster(anchorUserId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户已是您直播间的房管");
        }

        // 房管上限
        if (anchorService.getMasterCount(anchorUserId) > 4) {
            return new Result(GlobalApiError.OperationFailed, "房管设置已达上限");
        }

        if (anchorService.setMaster(anchorUserId, userId, BoolIntVal.True)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 取消房管
     *
     * @param userId
     * @return
     */
    @POST
    @Path("removeMaster")
    @Override
    public Result removeMaster(@FormParam("userId") Long userId) {
        Long anchorUserId = this.getToken().getUserId();

        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "请先认证主播");
        }

        // 判断是否房管
        if (!anchorService.isMaster(anchorUserId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户已非您直播间的房管");
        }

        if (anchorService.setMaster(anchorUserId, userId, BoolIntVal.False)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取房管列表
     *
     * @return
     */
    @GET
    @Path("getMasters")
    @Override
    public Result getMasters() {
        Long anchorUserId = this.getToken().getUserId();

        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "请先认证主播");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", anchorService.getMasterList(anchorUserId));
        return mResult.build();
    }

    /**
     * 禁言用户
     *
     * @param anchorUserId
     * @param targetUserId
     * @param remarks
     * @param optionId
     * @return
     */
    @POST
    @Path("forbiddenWords")
    @Override
    public Result forbiddenWords(@FormParam("anchorUserId") Long anchorUserId,
                                 @FormParam("targetUserId") Long targetUserId,
                                 @FormParam("remarks") String remarks,
                                 @FormParam("optionId") Long optionId) {
        // 不能对自己操作
        Long myUserId = this.getToken().getUserId();
        if (myUserId.equals(targetUserId)) {
            return new Result(GlobalApiError.OperationFailed, "不能对自己操作哦");
        }

        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到直播间");
        }

        // 主播无法被禁言
        if (anchorUserId.equals(targetUserId)) {
            return new Result(GlobalApiError.OperationFailed, "不能禁言主播");
        }

        // 超管无法被禁言
        LiveUserProfile targetProfile = userService.getLiveUserProfile(targetUserId, anchorUserId);
        if (LiveUserRole.SUPER == targetProfile.getUserRole()) {
            return new Result(GlobalApiError.OperationFailed, "超管大大无法被禁言哦~");
        }

        DataForbiddenWordOptionVO dataForbiddenWordOptionVO = dataService.getForbiddenWordOptionById(optionId);
        if (dataForbiddenWordOptionVO == null) {
            return new Result(GlobalApiError.OperationFailed, "禁言选项无效");
        }

        // 判断自己是否超管
        LiveUserProfile myProfile = userService.getLiveUserProfile(myUserId, anchorUserId);
        // 不是超管
        if (LiveUserRole.SUPER != myProfile.getUserRole()) {
            Integer myNobleLevel = nobleService.getUserNobleLevel(myUserId);
            // 判断权限
            if (!myUserId.equals(anchorUserId) && LiveUserRole.MASTER != myProfile.getUserRole() && myNobleLevel < NobleLevelEnum.GONGJUE.getCode()) {
                return new Result(GlobalApiError.OperationFailed, "您没有禁言权限哦");
            }

            // 判断目标用户是否贵族
            Integer targetNobleLevel = nobleService.getUserNobleLevel(targetUserId);
            if (targetNobleLevel > 0) {
                // 如果是贵族 判断自己的贵族等级是否高于对方
                if (myNobleLevel <= targetNobleLevel) {
                    return new Result(GlobalApiError.OperationFailed, "对方有防禁言特权，您当前的贵族等级无法对其禁言");
                }
            }

            if (LiveUserRole.MASTER == targetProfile.getUserRole() && !myUserId.equals(anchorUserId)) {
                return new Result(GlobalApiError.OperationFailed, "只有主播可以禁言房管哦~");
            }
        }

        if (anchorService.setUserForbiddenWords(anchorUserId, targetUserId, myUserId, remarks, dataForbiddenWordOptionVO.getOptionDuration())) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 解除禁言
     *
     * @param anchorUserId
     * @param targetUserId
     * @param remarks
     * @return
     */
    @POST
    @Path("enableWords")
    @Override
    public Result enableWords(@FormParam("anchorUserId") Long anchorUserId,
                              @FormParam("targetUserId") Long targetUserId,
                              @FormParam("remarks") String remarks) {
        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "无效的直播间");
        }

        Long masterUserId = this.getToken().getUserId();
        if (!anchorUserId.equals(masterUserId) && !anchorService.isMaster(anchorUserId, masterUserId)) {
            return new Result(GlobalApiError.OperationFailed, "您没有解禁权限");
        }

        // 判断是否被禁言
        if (!anchorService.isUserForbidden(anchorUserId, targetUserId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户未被禁言");
        }

        if (anchorService.setUserForbiddenWords(anchorUserId, targetUserId, masterUserId, remarks, 0)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取禁言用户列表
     *
     * @return
     */
    @GET
    @Path("getForbiddenUsers")
    @Override
    public Result getForbiddenUsers() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", anchorService.getForbiddenUsers(this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 申请签约公会
     *
     * @param agentId  公会id
     * @param applyMsg 申请时备注
     * @return result对象
     */
    @POST
    @Path("applyAgent")
    @Override
    public Result applyAgent(@FormParam("agentId") Long agentId,
                             @FormParam("applyMsg") String applyMsg) {
        if (!agentService.isAgentExist(agentId)) {
            return new Result(GlobalApiError.OperationFailed, "公会不存在");
        }
        if (anchorService.isSign(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "您只可签约一个公会");
        }
        if (agentService.applySign(agentId, this.getToken().getUserId(), applyMsg)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 申请解约公会
     *
     * @param agentId  公会id
     * @param applyMsg 申请时备注
     * @return result对象
     */
    @POST
    @Path("cancelAgent")
    @Override
    public Result cancelAgent(@FormParam("agentId") Long agentId,
                              @FormParam("applyMsg") String applyMsg) {
        if (!agentService.isAgentExist(agentId)) {
            return new Result(GlobalApiError.OperationFailed, "公会不存在");
        }
        if (!agentService.isSign(agentId, this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "您还没签约该公会");
        }
        if (agentService.cancelSign(this.getToken().getUserId(), agentId, applyMsg)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取公会列表
     *
     * @param content  搜索内容
     * @param pageNum  当前页码
     * @param pageSize 分页大小
     * @return result对象
     */
    @GET
    @Path("getAgentPageList")
    @Override
    public Result getAgentPageList(@QueryParam("content") String content,
                                   @QueryParam("pageNum") int pageNum,
                                   @QueryParam("pageNum") int pageSize) {
        return new Result(agentService.getAgentPageList(content, pageNum, pageSize));
    }


    /**
     * 获取公会详情
     *
     * @param agentId 公会id
     * @return 公会详情
     */
    @GET
    @Path("getAgentDetail")
    @Override
    public Result getAgentDetail(@QueryParam("agentId") Long agentId) {
        if (!agentService.isAgentExist(agentId)) {
            return new Result(GlobalApiError.OperationFailed, "公会不存在");
        }
        return new Result(agentService.getAgentDetail(agentId));
    }


    /**
     * 获取用户最近一次申请信息
     *
     * @return result对象
     */
    @GET
    @Path("getLastApplyInfo")
    @Override
    public Result getLastApplyInfo() {
        return new Result(agentService.getLastApplyInfo(this.getToken().getUserId(), ApplyTypeEnum.JOIN.getCode()));
    }

    /**
     * 获取信息
     *
     * @return result对象
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo(@QueryParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();
        boolean isSelf = userId == null || myUserId.equals(userId);
        if (userId == null) {
            userId = this.getToken().getUserId();
        }

        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户不是主播哦");
        }

        MutableResult mResult = new MutableResult();
        if (isSelf) {
            mResult.put("subscribe", socialRelationService.getFollowCount(this.getToken().getUserId()));
            mResult.put("gift", 0);
            mResult.put("brokerage", BigDecimal.ZERO);
            mResult.put("sellEnable", BoolIntVal.fromBool(anchorService.isSellEnabled(userId)));
        }
        mResult.put("anchorInfo", anchorService.getAnchorInfo(this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取粉丝团成员列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getFansList")
    @Override
    public Result getFansList(@QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }

        return new Result(anchorService.getFansList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId()));
    }

    /**
     * 拉黑
     *
     * @param userId
     * @param anchorUserId
     * @param remarks
     * @return
     */
    @POST
    @Path("addBlack")
    @Override
    public Result addBlack(@FormParam("userId") Long userId,
                           @FormParam("anchorUserId") Long anchorUserId,
                           @FormParam("remarks") String remarks) {
        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到直播间");
        }

        // 判断是否踢主播
        if (anchorUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "调皮了~ 主播不能被关入小黑屋哦");
        }

        // 判断是否踢超管
        LiveUserProfile targetProfile = userService.getLiveUserProfile(userId, anchorUserId);
        if (targetProfile.getUserRole().equals(LiveUserRole.SUPER)) {
            return new Result(GlobalApiError.OperationFailed, "超管大大无法关小黑屋哦");
        }

        // 超管
        Long myUserId = this.getToken().getUserId();
        LiveUserProfile myProfile = userService.getLiveUserProfile(myUserId, anchorUserId);
        boolean isSuper = myProfile.getUserRole().equals(LiveUserRole.SUPER);

        // 不是超管
        Integer myNobleLevel = nobleService.getUserNobleLevel(myUserId);
        if (!isSuper) {
            if (!myUserId.equals(anchorUserId) && myNobleLevel != NobleLevelEnum.DIHUANG.getCode()) {
                return new Result(GlobalApiError.OperationFailed, "您没有关小黑屋的权限哦！");
            }

            // 如果是房管
            if (LiveUserRole.MASTER == targetProfile.getUserRole() && !myUserId.equals(anchorUserId)) {
                return new Result(GlobalApiError.OperationFailed, "该用户是房管，只有主播可以操作哦~");
            }

            // 判断目标用户是否有防踢
            Integer targetNobleLevel = nobleService.getUserNobleLevel(userId);
            if (targetNobleLevel > NobleLevelEnum.NANJUE.getCode()) {
                // 是贵族有防踢 非帝皇无法处理
                if (targetNobleLevel == NobleLevelEnum.DIHUANG.getCode() || myNobleLevel != NobleLevelEnum.DIHUANG.getCode()) {
                    return new Result(GlobalApiError.OperationFailed, "对方有防踢权限，您无法将其关入小黑屋");
                }
            } else {
                // 如果踢的是普通用户
                if (!myUserId.equals(anchorUserId) && myNobleLevel < NobleLevelEnum.ZIJUE.getCode()) {
                    return new Result(GlobalApiError.OperationFailed, "您没有关小黑屋的权限~");
                }
            }
        }

        if (anchorService.addBlack(anchorUserId, userId, remarks)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "拉黑失败，请稍后再试");
    }

    /**
     * 解除拉黑
     *
     * @param userId
     * @return
     */
    @POST
    @Path("removeBlack")
    @Override
    public Result removeBlack(@FormParam("userId") Long userId) {
        Long anchorUserId = this.getToken().getUserId();
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "只有主播可以拉黑用户哦");
        }
        if (anchorService.removeBlack(anchorUserId, userId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed, "解除失败，请稍后再试");
    }

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getBlackList")
    @Override
    public Result getBlackList(@QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(anchorService.getBlackList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId()));
    }

    /**
     * 封禁主播
     *
     * @param anchorUserId
     * @param duration
     * @param remarks
     * @param reportId
     * @return
     */
    @POST
    @Path("disableAnchor")
    @Override
    public Result disableAnchor(@FormParam("anchorUserId") Long anchorUserId,
                                @FormParam("duration") Long duration,
                                @FormParam("remarks") String remarks,
                                @FormParam("reportId") Long reportId) {
        Long userId = this.getToken().getUserId();
        if (!userService.isSuper(userId)) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户不是主播");
        }

        if (anchorService.disableAnchor(anchorUserId, userId, System.currentTimeMillis() + duration, remarks, reportId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取pk主播列表
     *
     * @param type
     * @return
     */
    @GET
    @Path("getAnchorPkList")
    @Override
    public Result getAnchorPkList(@QueryParam("type") int type,
                                  @QueryParam("pageIndex") int pageIndex) {
        AnchorPkSearchType searchType = AnchorPkSearchType.toEnum(type);
        if (searchType == null) {
            searchType = AnchorPkSearchType.ALL;
        }
        return new Result(anchorService.getAnchorPkList(searchType, this.getToken().getUserId(), pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取主播列表
     *
     * @param content
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getAnchorList")
    @Override
    public Result getAnchorList(@QueryParam("content") String content,
                                @QueryParam("pageIndex") int pageIndex) {
        return new Result(anchorService.getAnchorList(content, pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取房间
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getRoom")
    @Override
    public Result getRoom(@QueryParam("userId") Long userId) {

        // 如果userId为空，则查看自己的房间
        userId = Optional.ofNullable(userId).orElse(this.getToken().getUserId());

        // 如果主播且直播中
        if (anchorService.isAnchor(userId) && LiveStatusEnum.Living == anchorService.getLiveStatus(userId)) {
            RoomVO roomVO = new RoomVO();
            roomVO.setUserId(userId);
            roomVO.setIsAnchor(BoolIntVal.True);
            roomVO.setLiveStatus(LiveStatusEnum.Living.getValue());
            roomVO.setCharmLevel(userService.getCharmLevelByUserId(userId));
            roomVO.setLivePopularityMax(liveService.getLivePopularityMax(userId));
            roomVO.setFansMemberCount(anchorService.getFansMemberCount(userId));
            return new Result(roomVO);
        }

        return new Result();
    }
}
