package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import com.starbuds.server.common.pojo.user.TokenBO;
import org.apache.commons.lang3.StringUtils;
import org.jboss.resteasy.spi.ResteasyProviderFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * 接口基类
 *
 * @author Lay
 * @date 2019-02-18
 */
public class BaseFacade {
    protected HttpServletRequest getRequest() {
        return ResteasyProviderFactory.getInstance().getContextData(HttpServletRequest.class);
    }

    protected HttpServletResponse getResponse() {
        return ResteasyProviderFactory.getInstance().getContextData(HttpServletResponse.class);
    }

    protected String getParamString(String name) {
        return this.getRequest().getParameter(name);
    }

    protected int getParamInt(String name) {
        return Convert.toInt(this.getParamString(name));
    }

    protected float getParamFloat(String name) {
        return Convert.toFloat(this.getParamString(name));
    }

    protected double getParamDouble(String name) {
        return Convert.toDouble(this.getParamString(name));
    }

    protected long getParamLong(String name) {
        return Convert.toLong(this.getParamString(name));
    }

    protected String getHeaderString(String name) {
        String str = this.getRequest().getHeader(name);
        return StringUtils.isBlank(str) ? "" : str;
    }

    protected int getHeaderInt(String name) {
        return Convert.toInt(this.getHeaderString(name));
    }

    protected BigInteger getHeaderBigInteger(String name) {
        String str = this.getHeaderString(name);
        return str == "" ? BigInteger.ZERO : new BigInteger(str);
    }

    protected float getHeaderFloat(String name) {
        return Convert.toFloat(this.getHeaderString(name));
    }

    protected double getHeaderDouble(String name) {
        return Convert.toDouble(this.getHeaderString(name));
    }

    protected long getHeaderLong(String name) {
        return Convert.toLong(this.getHeaderString(name));
    }

    protected HeaderPackage getHeader() {
        Object headerPackage = this.getRequest().getAttribute("header");
        if (headerPackage != null) {
            return (HeaderPackage) headerPackage;
        }
        return null;
    }

    protected TokenBO getToken() {
        Object token = this.getRequest().getAttribute("token");
        if (token != null) {
            return (TokenBO) token;
        }
        return null;
    }

    protected Long getUserId() {
        TokenBO token = getToken();
        if (token == null) {
            return null;
        }
        return token.getUserId();
    }

    /**
     * 获取用户真实IP地址，不使用request.getRemoteAddr();的原因是有可能用户使用了代理软件方式避免真实IP地址,
     * 参考文章： http://developer.51cto.com/art/201111/305181.htm
     * <p>
     * 可是，如果通过了多级反向代理的话，X-Forwarded-For的值并不止一个，而是一串IP值，究竟哪个才是真正的用户端的真实IP呢？
     * 答案是取X-Forwarded-For中第一个非unknown的有效IP字符串。
     * <p>
     * 如：X-Forwarded-For：192.168.1.110, 192.168.1.120, 192.168.1.130,
     * 192.168.1.100
     * <p>
     * 用户真实IP为： 192.168.1.110
     *
     * @return
     */
    public String getIpAddress() {
        HttpServletRequest request = this.getRequest();
        String ipAddress = request.getHeader("x-forwarded-for");
        // K8S 中使用 x-original-forwarded-for
        if (StringUtils.isBlank(ipAddress)) {
            ipAddress = request.getHeader("x-original-forwarded-for");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getRemoteAddr();
            if (ipAddress.equals("127.0.0.1") || ipAddress.equals("0:0:0:0:0:0:0:1")) {
                // 根据网卡取本机配置的IP
                InetAddress address = null;
                try {
                    address = InetAddress.getLocalHost();
                } catch (UnknownHostException e) {
                    e.printStackTrace();
                }
                ipAddress = address.getHostAddress();
            }
        }
        // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        if (ipAddress != null && ipAddress.length() > 15) {
            // "***.***.***.***".length() = 15
            if (ipAddress.indexOf(",") > 0) {
                ipAddress = ipAddress.substring(0, ipAddress.indexOf(","));
            }
        }
        return ipAddress;
    }

    /**
     * 获得语言类型
     *
     * @return
     */
    protected String lang() {
        return getRequest().getHeader("lang");
    }
}
