package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.v1.facade.BattleFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.media.BattleStatusEnum;
import com.starbuds.server.common.constant.media.BattlingStatus;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.LiveBattleDO;
import com.starbuds.server.common.pojo.media.BattleInviteDTO;
import com.starbuds.server.common.pojo.user.AnchorInfoVO;
import com.starbuds.server.common.service.media.BattleService;
import com.starbuds.server.common.service.user.AnchorService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import java.util.Optional;

/**
 * PK 接口实现
 *
 * @author Lay
 * @date 2020/4/20
 */
@Path("/v1/battle")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class BattleFacadeImpl extends BaseFacade implements BattleFacade {

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    BattleService battleService;

    /**
     * 邀请 pk
     *
     * @param userId
     * @return
     */
    @POST
    @Path("invite")
    @Override
    public Result invite(@FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();

        // 判断是否自己
        if (myUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "你好调皮哦，不能向自己发起PK哦");
        }

        AnchorInfoVO myAnchorInfo = anchorService.getInfo(myUserId);

        // 判断自己是否主播
        if (myAnchorInfo == null) {
            return new Result(GlobalApiError.OperationFailed, "只有主播可以发起PK哦");
        }

        // 判断自己的直播状态
        if (LiveStatusEnum.Living != LiveStatusEnum.toEnum(myAnchorInfo.getLiveStatus())) {
            return new Result(GlobalApiError.OperationFailed, "直播状态下才可发起PK哦");
        }

        // 判断自己的 PK 开关
        if (BoolIntVal.True != Optional.ofNullable(myAnchorInfo.getBattleEnable()).orElse(BoolIntVal.False)) {
            return new Result(GlobalApiError.OperationFailed, "请先打开 PK 开关");
        }

        // 判断自己的 PK 状态
        if (BoolIntVal.False != Optional.ofNullable(myAnchorInfo.getBattling()).orElse(BoolIntVal.False)) {
            return new Result(GlobalApiError.OperationFailed, "您当前正在 PK 中");
        }

        AnchorInfoVO destAnchorInfo = anchorService.getInfo(userId);

        // 判断对方是否主播
        if (destAnchorInfo == null) {
            return new Result(GlobalApiError.OperationFailed, "只能向主播发起PK邀请哦");
        }

        // 判断对方的直播状态
        if (LiveStatusEnum.Living != LiveStatusEnum.toEnum(destAnchorInfo.getLiveStatus())) {
            return new Result(GlobalApiError.OperationFailed, "对方未在直播，换个主播试试吧");
        }

        // 判断对方的 PK 开关
        if (BoolIntVal.True != Optional.ofNullable(destAnchorInfo.getBattleEnable()).orElse(BoolIntVal.False)) {
            return new Result(GlobalApiError.OperationFailed, "对方关闭了 PK 开关");
        }

        // 判断对方的 PK 状态
        if (BoolIntVal.False != Optional.ofNullable(destAnchorInfo.getBattling()).orElse(BoolIntVal.False)) {
            return new Result(GlobalApiError.OperationFailed, "对方当前正在 PK 中");
        }

        String battleId = battleService.invite(BattleInviteDTO.builder().userId(myUserId).destUserId(userId).channelId(this.getHeader().getChannelId()).build());
        if (StringUtils.isNotBlank(battleId)) {
            MutableResult mResult = new MutableResult();
            mResult.put("battleId", battleId);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed, "邀请失败，请稍后再试");
    }

    /**
     * 取消邀请
     *
     * @param battleId
     * @return
     */
    @POST
    @Path("inviteCancel")
    @Override
    public Result inviteCancel(@FormParam("battleId") String battleId) {
        Long userId = this.getToken().getUserId();

        // 判断状态
        LiveBattleDO liveBattle = battleService.getBattle(battleId);
        if (liveBattle == null || !userId.equals(liveBattle.getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "未找到 PK 记录");
        }

        // 判断是否进行中
        if (BattleStatusEnum.Battling == BattleStatusEnum.toEnum(liveBattle.getBattleStatus())) {
            return new Result(GlobalApiError.OperationFailed, "当前状态无法取消");
        }

        if (battleService.inviteCancel(battleId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "邀请取消失败");
    }

    /**
     * 决策邀请
     *
     * @param battleId
     * @param decision
     * @return
     */
    @POST
    @Path("inviteDecide")
    @Override
    public Result inviteDecide(@FormParam("battleId") String battleId, @FormParam("decision") int decision) {
        Long userId = this.getToken().getUserId();

        if (BoolIntVal.True != decision) {
            decision = BoolIntVal.False;
        }

        // 判断状态
        LiveBattleDO liveBattle = battleService.getBattle(battleId);
        if (liveBattle == null || !userId.equals(liveBattle.getDestUserId())) {
            return new Result(GlobalApiError.OperationFailed, "未找到 PK 记录");
        }

        // 判断是否呼叫中
        if (BattleStatusEnum.Ringing != BattleStatusEnum.toEnum(liveBattle.getBattleStatus())) {
            return new Result(GlobalApiError.OperationFailed, "当前状态无法接通");
        }

        if (battleService.inviteDecide(battleId, decision)) {
            if (BoolIntVal.True == decision) {
                MutableResult mResult = new MutableResult();
                mResult.put("roomToken", battleService.getRoomToken(battleId, userId));
                return mResult.build();
            }
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "PK 已取消");
    }

    /**
     * 加入 pk 房间
     *
     * @param battleId
     * @return
     */
    @POST
    @Path("joinBattleRoom")
    @Override
    public Result joinBattleRoom(@FormParam("battleId") String battleId) {
        Long userId = this.getToken().getUserId();

        // 判断状态
        LiveBattleDO liveBattle = battleService.getBattle(battleId);
        if (liveBattle == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到 PK 记录");
        }

        if (!userId.equals(liveBattle.getUserId()) && !userId.equals(liveBattle.getDestUserId())) {
            return new Result(GlobalApiError.OperationFailed, "您无权加入房间");
        }

        // 判断是否呼叫中
        if (BattlingStatus.Wait != battleService.getBattlingStatus(liveBattle.getBattleId())) {
            return new Result(GlobalApiError.OperationFailed, "当前无法加入房间");
        }
        battleService.joinBattleRoom(battleId, userId);
        return new Result();
    }

    /**
     * 结束 pk
     *
     * @param battleId
     * @return
     */
    @POST
    @Path("closeBattle")
    @Override
    public Result closeBattle(@FormParam("battleId") String battleId) {
        Long userId = this.getToken().getUserId();

        // 判断状态
        LiveBattleDO liveBattle = battleService.getBattle(battleId);
        if (liveBattle == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到 PK 记录");
        }

        if (!userId.equals(liveBattle.getUserId()) && !userId.equals(liveBattle.getDestUserId())) {
            return new Result(GlobalApiError.OperationFailed, "您无权加入房间");
        }

        // 判断是否在惩罚阶段
        Integer battlingStatus = battleService.getBattlingStatus(battleId);
        if (battlingStatus == null) {
            return new Result();
        }

        if (BattlingStatus.Battling == battlingStatus) {
            return new Result(GlobalApiError.OperationFailed, "PK还在进行中哦，请等待PK结束或者主动认输后再试");
        }

        if (BattlingStatus.Punish == battlingStatus || BattlingStatus.Wait == battlingStatus) {
            if (battleService.closeBattle(battleId)) {
                return new Result();
            }
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 认输
     *
     * @param battleId
     * @return
     */
    @POST
    @Path("surrenderBattle")
    @Override
    public Result surrenderBattle(@FormParam("battleId") String battleId) {
        Long userId = this.getToken().getUserId();

        // 判断状态
        LiveBattleDO liveBattle = battleService.getBattle(battleId);
        if (liveBattle == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到 PK 记录");
        }

        // 判断是否在 PK 阶段
        Integer battlingStatus = Optional.ofNullable(battleService.getBattlingStatus(battleId)).orElse(BattlingStatus.Over);
        if (BattlingStatus.Battling == battlingStatus) {
            battleService.surrenderBattle(battleId, userId);
            return new Result();
        }
        return new Result();
    }
}
