package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.v1.facade.ChainFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.UserVirtualCoinAccountDO;
import com.starbuds.server.common.service.finance.ChainService;
import com.starbuds.server.common.service.finance.WalletService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.List;
import java.util.Optional;

/**
 * 链支付接口
 *
 * @author Lay
 * @date 2020/3/13
 */
@Path("/v1/chain")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChainFacadeImpl extends BaseFacade implements ChainFacade {

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    ChainService chainService;

    /**
     * 获取账户列表
     *
     * @return
     */
    @GET
    @Path("getAccounts")
    @Override
    public Result getAccounts() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", chainService.getAccounts(this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 创建账户
     *
     * @param coinNet
     * @return
     */
    @POST
    @Path("createAccount")
    @Override
    public Result createAccount(@FormParam("coinNet") String coinNet) {
        if (!chainService.isCoinNetExists(coinNet)) {
            return new Result(GlobalApiError.OperationFailed, "不支持该币种开户");
        }

        // 判断是否已开户
        Long userId = this.getToken().getUserId();
        UserVirtualCoinAccountDO userVirtualCoinAccount = chainService.getAccountByCoinNet(userId, coinNet);
        if (userVirtualCoinAccount != null) {
            return new Result(userVirtualCoinAccount.getAccountAddress());
        }

        String account = chainService.createAccount(userId, coinNet);
        if (account == null) {
            return new Result(GlobalApiError.OperationFailed, "开户失败");
        }
        return new Result(account);
    }

    /**
     * 获取币种列表
     *
     * @return
     */
    @GET
    @Path("getVirtualCoinList")
    @Override
    public Result getVirtualCoinList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", chainService.getVirtualCoinList());
        return mResult.build();
    }

    /**
     * 获取账户记录列表
     *
     * @param pageIndex
     * @param coinId
     * @param transType
     * @return
     */
    @GET
    @Path("getAccountHisList")
    @Override
    public Result getAccountHisList(@QueryParam("pageIndex") int pageIndex,
                                    @QueryParam("coinId") Integer coinId,
                                    @QueryParam("transType") Integer transType) {
        return new Result(chainService.getAccountHisList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId(), coinId, transType));
    }

    /**
     * 申请提币
     *
     * @param coinId
     * @param amount
     * @param toAddress
     * @return
     */
    @POST
    @Path("applyWithdraw")
    @Override
    public Result applyWithdraw(@FormParam("coinId") Integer coinId,
                                @FormParam("amount") Long amount,
                                @FormParam("toAddress") String toAddress) {
        if (Optional.ofNullable(amount).orElse(0L) < 1) {
            return new Result(GlobalApiError.OperationFailed, "提币金额错误");
        }

        // 判断币种支持
        if (!chainService.isVirtualCoinExists(coinId)) {
            return new Result(GlobalApiError.OperationFailed, "不支持该币种");
        }

        // 判断地址是否自己的地址
        Long userId = this.getToken().getUserId();
        List<UserVirtualCoinAccountDO> accounts = chainService.getAccounts(userId);
        for (UserVirtualCoinAccountDO account : accounts) {
            if (account.getAccountAddress().equals(toAddress)) {
                return new Result(GlobalApiError.OperationFailed, "不能提至当前账户");
            }
        }

        // 判断余额充足
        if (walletService.getBalance(userId, coinId) < amount) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        // 申请提币
        if (chainService.applyWithdraw(userId, coinId, amount, toAddress)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "提币申请失败");
    }
}
