package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.v1.facade.CoinShopFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.CoinGoodsDO;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.data.CoinGoodsService;
import com.starbuds.server.common.service.finance.CoinGoodsOrderService;
import com.starbuds.server.common.service.i18n.I18nService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;

/**
 * 积分商城接口实现
 *
 * @author Lay
 * @date 2020/3/30
 */
@Path("/v1/coinShop")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class CoinShopFacadeImpl extends BaseFacade implements CoinShopFacade {

    @Reference(version = "1.0.0")
    CoinGoodsService coinGoodsService;

    @Reference(version = "1.0.0")
    CoinGoodsOrderService coinGoodsOrderService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    private I18nService i18nService;

    /**
     * 获取商品列表
     *
     * @param pageIndex
     * @param categoryId
     * @return
     */
    @GET
    @Path("getGoodsList")
    @Override
    public Result getGoodsList(@QueryParam("pageIndex") int pageIndex,
                               @QueryParam("categoryId") Long categoryId) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(i18nService.doI18n(coinGoodsService.getCoinGoodsList(pageIndex, GlobalConfig.PAGE_SIZE, null, categoryId, this.getHeader().getChannelId()), lang(), 5));
    }

    /**
     * 获取商品类目
     *
     * @return
     */
    @GET
    @Path("getGoodsCategories")
    @Override
    public Result getGoodsCategories() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(coinGoodsService.getCategories(), lang()));
        return mResult.build();
    }

    /**
     * 创建商品订单
     *
     * @param goodsId
     * @param quantity
     * @param userNo
     * @return
     */
    @POST
    @Path("addGoodsOrder")
    @Override
    public Result addGoodsOrder(@FormParam("goodsId") Long goodsId,
                                @FormParam("quantity") Integer quantity,
                                @FormParam("userNo") Long userNo) {

        if (quantity == null || quantity < 1) {
            quantity = 1;
        }

        // 判断商品是否存在
        CoinGoodsDO scoreGoods = coinGoodsService.getCoinGoods(goodsId);
        if (scoreGoods == null) {
            return new Result(GlobalApiError.OperationFailed, "商品不存在或已下架");
        }

        // 判断商品库存是否充足
        if (scoreGoods.getGoodsStock() < quantity) {
            return new Result(GlobalApiError.OperationFailed, "库存不足");
        }
        Long curUserId = this.getToken().getUserId();
        Long userIdByNo = null;
        if (userNo == null) {
            userIdByNo = curUserId;
        } else {
            // 判断目标用户是否存在
            UserBasicInfo userByUserNoOrNiceNo = userService.getUserByUserNoOrNiceNo(userNo);
            if (userByUserNoOrNiceNo == null) {
                return new Result(GlobalApiError.OperationFailed, "输入用户ID/靓号有误");
            }
            userIdByNo = userByUserNoOrNiceNo.getUserId();
        }

        // 下单
        Long balance = coinGoodsOrderService.addCoinGoodsOrder(curUserId, goodsId, quantity, userIdByNo);
        if (balance != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("balance", balance);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取订单列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getGoodsOrderList")
    @Override
    public Result getGoodsOrderList(@QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(coinGoodsOrderService.getCoinGoodsOrderList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId()));
    }
}
