package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.FamilyFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.PayStatusEnum;
import com.starbuds.server.common.constant.finance.SkillOrderStatusEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.FamilySkillVO;
import com.starbuds.server.common.pojo.db.FamilyAuditDO;
import com.starbuds.server.common.pojo.db.SkillOrderDO;
import com.starbuds.server.common.pojo.user.FamilyAuditVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.Calendar;
import java.util.Date;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 家族
 *
 * @author jiangxiaowei
 * @date 2020/10/14 16:52
 */
@Path("/v1/family")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FamilyFacadeImpl extends BaseFacade implements FamilyFacade {

    /**
     * 连击间隔时间(毫秒)
     */
    private static final long DOUBLE_CLICK_INTERVAL_MILLS = 1500;

    /**
     * 连击提示信息
     */
    private static final String DOUBLE_CLICK_MSG = "请勿重复提交";

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    PayService payService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 获取家族列表
     *
     * @param content   搜索内容
     * @param pageIndex 当前页码
     * @return result对象
     */
    @GET
    @Path("getFamilyList")
    @Override
    public Result getFamilyList(@QueryParam("content") String content,
                                @QueryParam("pageIndex") int pageIndex) {
        return new Result(familyService.getFamilyList(content, pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取家族详情
     *
     * @param familyId
     * @return
     */
    @GET
    @Path("getFamilyDetail")
    @Override
    public Result getFamilyDetail(@QueryParam("familyId") Long familyId) {
        return new Result(familyService.getFamilyDetail(familyId, this.getToken().getUserId()));
    }

    /**
     * 添加技能认证
     *
     * @param skillId
     * @param skillTagId
     * @param skillIntro
     * @param images
     * @param voice
     * @param familyId
     * @param duration
     * @return
     */
    @POST
    @Path("addSkillAudit")
    @Override
    public Result addSkillAudit(@FormParam("skillId") Long skillId,
                                @FormParam("skillTagId") Long skillTagId,
                                @FormParam("skillIntro") String skillIntro,
                                @FormParam("images") String images,
                                @FormParam("voice") String voice,
                                @FormParam("familyId") Long familyId,
                                @FormParam("duration") Long duration) {
        Long userId = this.getToken().getUserId();
        if (!userService.isAuthExist(userId)) {
            return new Result(GlobalApiError.ServerError, "请先实名认证");
        }
        // 判断技能和标签是否存在
        if (!skillService.isSkillExist(skillId)) {
            return new Result(GlobalApiError.ServerError, "技能不存在");
        }
        if (!skillService.isSkillTagExist(skillTagId)) {
            return new Result(GlobalApiError.ServerError, "技能标签不存在");
        }
        Long userFamilyId = familyService.getFamilyId(userId);
        // 初次签约
        if (userFamilyId == null) {
            if (familyId == null) {
                return new Result(GlobalApiError.ServerError, "请选择签约的家族");
            }
            // 如果未提交过
            if (!familyService.isFamilyAuditExist(familyId, userId, ApplyTypeEnum.JOIN.getCode(), AuditStatusEnum.WAIT.getCode())) {
                // 提交签约家族申请
                FamilyAuditDO auditDO = new FamilyAuditDO();
                auditDO.setAuditId(identityService.getId());
                auditDO.setFamilyId(familyId);
                auditDO.setUserId(userId);
                auditDO.setApplyMsg(skillIntro);
                auditDO.setApplyTime(System.currentTimeMillis());
                auditDO.setAuditStatus(AuditStatusEnum.WAIT.getCode());
                auditDO.setApplyType(ApplyTypeEnum.JOIN.getCode());
                if (!familyService.addFamilyAudit(auditDO)) {
                    return new Result(GlobalApiError.ServerError, "签约家族失败");
                }
            }
        } else {
            if (familyId != null && !userFamilyId.equals(familyId)) {
                return new Result(GlobalApiError.ServerError, "只能签约一个家族");
            }
        }

        // 判断技能认证是否存在
        if (familyService.isSkillAuditExist(familyId, userId, skillId, AuditStatusEnum.WAIT.getCode())) {
            return new Result(GlobalApiError.ServerError, "您有申请正在审核中,请耐心等待...");
        }

        // 提交认证技能申请
        if (!familyService.addSkillAuth(familyId, userId, skillId, skillTagId, skillIntro, images, voice, duration)) {
            return new Result(GlobalApiError.ServerError, "技能认证失败");
        }
        return new Result();
    }

    /**
     * 提交家族签约/解约流程
     *
     * @param familyId
     * @param applyType
     * @param applyMsg
     * @return
     */
    @POST
    @Path("addFamilyAudit")
    @Override
    public Result addFamilyAudit(@FormParam("familyId") Long familyId, @FormParam("applyType") Integer applyType, @FormParam("applyMsg") String applyMsg) {
        if (applyType == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        ApplyTypeEnum applyTypeEnum = ApplyTypeEnum.toEnum(applyType);
        if (applyTypeEnum == null) {
            return new Result(GlobalApiError.ServerError, "传入类型错误");
        }
        Long userId = this.getToken().getUserId();

        if (familyService.isFamilyPresident(userId)) {
            return new Result(GlobalApiError.ServerError, "家族长不能进行解约/签约操作");
        }

        if (familyService.isFamilyAuditExist(familyId, userId, applyType, AuditStatusEnum.WAIT.getCode())) {
            return new Result(GlobalApiError.ServerError, "您有流程正在审核中,请耐心等待");
        }

        switch (applyTypeEnum) {
            case JOIN:
                if (familyId == null) {
                    return new Result(GlobalApiError.ServerError, "缺少家族id");
                }
                if (familyService.getFamilyId(userId) != null) {
                    return new Result(GlobalApiError.ServerError, "您已签约家族");
                }
                break;
            case LEAVE:
                familyId = familyService.getFamilyId(userId);
                if (familyId == null) {
                    return new Result(GlobalApiError.ServerError, "您没有签约家族");
                }
                break;
            default:
                break;
        }
        FamilyAuditDO auditDO = new FamilyAuditDO();
        auditDO.setAuditId(identityService.getId());
        auditDO.setFamilyId(familyId);
        auditDO.setUserId(userId);
        auditDO.setApplyMsg(applyMsg);
        auditDO.setApplyTime(System.currentTimeMillis());
        auditDO.setAuditStatus(AuditStatusEnum.WAIT.getCode());
        auditDO.setApplyType(applyTypeEnum.getCode());
        auditDO.setIsForce(BoolIntVal.False);
        if (familyService.addFamilyAudit(auditDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取已经认证的技能
     *
     * @return
     */
    @GET
    @Path("getSkillsAuth")
    @PermitGuest
    @Override
    public Result getSkillsAuth(@QueryParam("userId") Long userId) {
        if (userId == null && getToken() != null) {
            userId = getToken().getUserId();
        }
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的参数");
        }
        MutableResult mResult = new MutableResult();
        mResult.put("list", familyService.getSkillsAuth(userId));
        return mResult.build();
    }

    /**
     * 获取已经认证的技能 不包括语音速配
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getSkillsAuthIgnoreVoiceMatch")
    @Override
    public Result getSkillsAuthIgnoreVoiceMatch(@QueryParam("userId") Long userId) {
        if (userId == null && getToken() != null) {
            userId = getToken().getUserId();
        }
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的参数");
        }
        MutableResult mResult = new MutableResult();
        mResult.put("list", familyService.getSkillsAuthIgnoreVoiceMatch(userId));
        return mResult.build();
    }

    /**
     * 获取已提交的技能列表
     *
     * @return
     */
    @GET
    @Path("getSkillsSubmit")
    @Override
    public Result getSkillsSubmit() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", familyService.getSkillsSubmit(this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 技能设置
     *
     * @param skillId
     * @param orderPrice
     * @param orderTime
     * @param orderStatus
     * @return
     */
    @POST
    @Path("setSkill")
    @Override
    public Result setSkill(@FormParam("skillId") Long skillId,
                           @FormParam("orderPrice") Long orderPrice,
                           @FormParam("orderTime") String orderTime,
                           @FormParam("orderStatus") Integer orderStatus) {
        Long userId = this.getToken().getUserId();
        if (!familyService.isSkillAuth(skillId, userId)) {
            return new Result(GlobalApiError.ServerError, "请先认证技能");
        }
        if (familyService.setSkill(familyService.getFamilyId(userId), userId, skillId, orderPrice, orderTime, orderStatus)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取技能设置
     *
     * @param skillId
     * @return
     */
    @GET
    @Path("getSkill")
    @Override
    public Result getSkill(@QueryParam("skillId") Long skillId) {
        Long userId = this.getToken().getUserId();
        if (!familyService.isSkillAuth(skillId, userId)) {
            return new Result(GlobalApiError.ServerError, "需要先认证技能");
        }
        return new Result(familyService.getSkill(familyService.getFamilyId(userId), userId, skillId));
    }

    /**
     * 下单
     *
     * @param receiveUserId
     * @param skillId
     * @param quantity
     * @param remark
     * @param requireTime
     * @return
     */
    @POST
    @Path("addOrder")
    @Override
    public Result addOrder(@FormParam("receiveUserId") Long receiveUserId,
                           @FormParam("skillId") Long skillId,
                           @FormParam("quantity") Integer quantity,
                           @FormParam("remark") String remark,
                           @FormParam("requireTime") Long requireTime) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:addOrder:%d:%d:%d", userId, receiveUserId, skillId, quantity);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        Long receiveFamilyId = familyService.getFamilyId(receiveUserId);
        if (receiveFamilyId == null) {
            return new Result(GlobalApiError.OperationFailed, "对方未认证此功能");
        }
        FamilySkillVO skillVO = familyService.getSkill(receiveFamilyId, receiveUserId, skillId);
        if (skillVO == null || skillVO.getSkillStatus() < 1) {
            return new Result(GlobalApiError.ServerError, "对方该功能已下线");
        }
        // 判断技能是否可用
        if (!skillService.isSkillEnable(receiveFamilyId, receiveUserId, skillId)) {
            return new Result(GlobalApiError.OperationFailed, "技能已下线，暂不支持该操作");
        }
        if (skillVO.getOrderStatus() != BoolIntVal.True) {
            return new Result(GlobalApiError.ServerError, "对方已下线");
        }
        if (Optional.ofNullable(skillVO.getVoiceMatchEnable()).map(e -> e == BoolIntVal.True).orElse(false)) {
            return new Result(GlobalApiError.OperationFailed, "不可以对语音速配下单");
        }
        Long requireTimeFormat = Convert.toLong(DateUtil.format(new Date(requireTime), "HHmm"));

        // 接单时间
        String orderTime = skillVO.getOrderTime();
        String[] timeArray = orderTime.split("-");

        Long left = Convert.toLong(timeArray[0].replace(":", ""));
        Long right = Convert.toLong(timeArray[1].replace(":", ""));

        // 06:00-22:00
        if (left < right) {
            if (requireTimeFormat < left || requireTimeFormat > right) {
                return new Result(GlobalApiError.ServerError, "用户该时间段不接单，请换个时间段再来试试吧");
            }
        } else {
            // 22:00-06:00
            if (requireTimeFormat < left && requireTimeFormat > right) {
                return new Result(GlobalApiError.ServerError, "用户该时间段不接单，请换个时间段再来试试吧");
            }
        }

        long curTime = System.currentTimeMillis();
        Long orderId = identityService.getId();
        Long payAmount = skillVO.getOrderPrice() * quantity;

        if (payAmount == null || payAmount <= 0) {
            return new Result(GlobalApiError.ServerError, "错误的订单金额");
        }

        SkillOrderDO orderDO = new SkillOrderDO();
        orderDO.setOrderId(orderId);
        orderDO.setUserId(userId);
        orderDO.setReceiveUserId(receiveUserId);
        orderDO.setReceiveFamilyId(receiveFamilyId);
        orderDO.setSkillId(skillId);
        orderDO.setSkillTagId(skillVO.getSkillTagId());
        orderDO.setPayAmount(payAmount);
        orderDO.setOrderPrice(skillVO.getOrderPrice());
        orderDO.setQuantity(quantity);
        orderDO.setPayStatus(PayStatusEnum.WAITTING.getValue());
        orderDO.setOrderStatus(SkillOrderStatusEnum.ORDER_CREATE.getValue());
        orderDO.setCreateTime(curTime);
        orderDO.setRequireTime(requireTime);
        orderDO.setUpdateTime(curTime);
        orderDO.setRemark(remark);
        orderDO.setChannelId(this.getHeader().getChannelId());
        if (payService.addSkillOrder(orderDO)) {
            MutableResult mResult = new MutableResult();
            mResult.put("orderId", orderId);
            mResult.put("paySuccess", payService.payForSkill(orderId, this.getHeader().getChannelId()));
            return mResult.build();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 支付订单
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("payOrder")
    @Override
    public Result payOrder(@FormParam("orderId") Long orderId) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:payOrder:%d", userId, orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        SkillOrderDO skillOrder = payService.getSkillOrder(orderId);
        if (skillOrder == null) {
            return new Result(GlobalApiError.ServerError, "订单不存在");
        }
        if (!userId.equals(skillOrder.getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "仅下单人可操作");
        }
        // 判断技能是否可用
        if (!skillService.isSkillEnable(skillOrder.getReceiveFamilyId(), skillOrder.getReceiveUserId(), skillOrder.getSkillId())) {
            return new Result(GlobalApiError.OperationFailed, "技能已下线，暂不支持该操作");
        }
        PayStatusEnum payStatusEnum = PayStatusEnum.toEnum(skillOrder.getPayStatus());
        if (payStatusEnum != PayStatusEnum.WAITTING) {
            return new Result(GlobalApiError.ServerError, "订单不是待支付状态");
        }
        Long payAmount = skillOrder.getPayAmount();
        // 判断金额是否足够
        if (walletService.getBalance(userId, WalletTypeEnum.STAR_DIAMOND) < payAmount) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        if (payService.payForSkill(orderId, this.getHeader().getChannelId())) {
            long curTime = System.currentTimeMillis();
            if (skillOrder.getRequireTime() <= curTime) {
                payService.startService(orderId);
            }
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 取消支付
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("cancelPay")
    @Override
    public Result cancelPay(@FormParam("orderId") Long orderId) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:cancelPay:%d", userId, orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        SkillOrderDO skillOrder = payService.getSkillOrder(orderId);
        if (skillOrder == null) {
            return new Result(GlobalApiError.ServerError, "订单不存在");
        }
        if (!userId.equals(skillOrder.getUserId())) {
            return new Result(GlobalApiError.ServerError, "仅下单人可操作");
        }
        // 判断技能是否可用
        if (!skillService.isSkillEnable(skillOrder.getReceiveFamilyId(), skillOrder.getReceiveUserId(), skillOrder.getSkillId())) {
            return new Result(GlobalApiError.OperationFailed, "技能已下线，暂不支持该操作");
        }
        if (!payService.isWaitPay(orderId)) {
            return new Result(GlobalApiError.ServerError, "当前状态不支持取消");
        }
        if (payService.cancelPay(orderId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取订单详情
     *
     * @param orderId
     * @return
     */
    @GET
    @Path("getOrderDetail")
    @Override
    public Result getOrderDetail(@QueryParam("orderId") Long orderId) {
        return new Result(payService.getOrderDetail(orderId, this.getToken().getUserId()));
    }

    /**
     * 获取接单记录
     *
     * @param orderStatus
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getOrderReceiveList")
    @Override
    public Result getOrderReceiveList(@QueryParam("pageIndex") int pageIndex, @QueryParam("orderStatus") Integer orderStatus) {
        return new Result(payService.getOrderReceiveList(pageIndex, GlobalConfig.PAGE_SIZE, orderStatus, this.getToken().getUserId()));
    }

    /**
     * 获取点单记录
     *
     * @param pageIndex
     * @param orderStatus
     * @param payStatus
     * @return
     */
    @GET
    @Path("getOrderPayList")
    @Override
    public Result getOrderPayList(@QueryParam("pageIndex") int pageIndex, @QueryParam("orderStatus") Integer orderStatus, @QueryParam("payStatus") Integer payStatus) {
        return new Result(payService.getOrderPayList(pageIndex, GlobalConfig.PAGE_SIZE, orderStatus, payStatus, this.getToken().getUserId()));
    }

    /**
     * 接收订单
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("receiveOrder")
    @Override
    public Result receiveOrder(@FormParam("orderId") Long orderId) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:receiveOrder:%d", userId, orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        SkillOrderDO skillOrder = payService.getSkillOrder(orderId);
        if (skillOrder == null) {
            return new Result(GlobalApiError.ServerError, "订单不存在");
        }
        if (!userId.equals(skillOrder.getReceiveUserId())) {
            return new Result(GlobalApiError.ServerError, "您不是接单人");
        }
        // 判断技能是否可用
        if (!skillService.isSkillEnable(skillOrder.getReceiveFamilyId(), skillOrder.getReceiveUserId(), skillOrder.getSkillId())) {
            return new Result(GlobalApiError.OperationFailed, "技能已下线，暂不支持该操作");
        }
        if (skillOrder.getOrderStatus() != SkillOrderStatusEnum.WAIT_RECEIVE.getValue()) {
            return new Result(GlobalApiError.ServerError, "该订单已处理");
        }

        if (payService.receiveOrder(orderId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 拒绝订单
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("rejectOrder")
    @Override
    public Result rejectOrder(@FormParam("orderId") Long orderId) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:rejectOrder:%d", userId, orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        SkillOrderDO skillOrder = payService.getSkillOrder(orderId);
        if (skillOrder == null) {
            return new Result(GlobalApiError.ServerError, "订单不存在");
        }
        if (!userId.equals(skillOrder.getReceiveUserId())) {
            return new Result(GlobalApiError.ServerError, "您不是接单人");
        }
        // 判断技能是否可用
        if (!skillService.isSkillEnable(skillOrder.getReceiveFamilyId(), skillOrder.getReceiveUserId(), skillOrder.getSkillId())) {
            return new Result(GlobalApiError.OperationFailed, "技能已下线，暂不支持该操作");
        }
        if (skillOrder.getOrderStatus() != SkillOrderStatusEnum.WAIT_RECEIVE.getValue()) {
            return new Result(GlobalApiError.ServerError, "该订单已处理");
        }
        if (payService.rejectOrder(orderId, "接单人拒单")) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 开始服务
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("startService")
    @Override
    public Result startService(@FormParam("orderId") Long orderId) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:startService:%d", userId, orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        SkillOrderDO skillOrder = payService.getSkillOrder(orderId);
        if (skillOrder == null) {
            return new Result(GlobalApiError.ServerError, "订单不存在");
        }
        if (!userId.equals(skillOrder.getReceiveUserId())) {
            return new Result(GlobalApiError.ServerError, "您不是接单人");
        }
        // 判断技能是否可用
        if (!skillService.isSkillEnable(skillOrder.getReceiveFamilyId(), skillOrder.getReceiveUserId(), skillOrder.getSkillId())) {
            return new Result(GlobalApiError.OperationFailed, "技能已下线，暂不支持该操作");
        }
        if (payService.startService(orderId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 确认完成
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("confirmService")
    @Override
    public Result confirmService(@FormParam("orderId") Long orderId) {
        Long userId = this.getToken().getUserId();
        String key = String.format("%s:confirm:%d", userId, orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }
        SkillOrderDO skillOrder = payService.getSkillOrder(orderId);
        if (skillOrder == null) {
            return new Result(GlobalApiError.ServerError, "订单不存在");
        }
        if (!userId.equals(skillOrder.getUserId())) {
            return new Result(GlobalApiError.ServerError, "仅下单人可操作");
        }
        Result result = payService.confirmService(orderId);
        if (result.isSuccess()) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError, result.getMsg());
    }

    /**
     * 获取技能用户列表
     *
     * @param pageIndex
     * @param skillId
     * @return
     */
    @GET
    @Path("getSkillUserList")
    @PermitGuest
    @Override
    public Result getSkillUserList(@QueryParam("pageIndex") int pageIndex, @QueryParam("skillId") Long skillId) {
        return new Result(familyService.getSkillUserList(pageIndex, GlobalConfig.PAGE_SIZE, skillId));
    }

    /**
     * 获取推荐技能用户列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getRecommendSkillUserList")
    @PermitGuest
    @Override
    public Result getRecommendSkillUserList(@QueryParam("pageIndex") int pageIndex) {
        return new Result(familyService.getRecommendSkillUserList(pageIndex, 30));
    }

    /**
     * 获取陪玩头条
     *
     * @return
     */
    @GET
    @Path("getPlayTop")
    @PermitGuest
    @Override
    public Result getPlayTop() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", dataService.getPlayTop(50));
        return mResult.build();
    }

    /**
     * 获取用户技能详情
     *
     * @param skillId
     * @param userId
     * @return
     */
    @GET
    @Path("getSkillUserDetail")
    @PermitGuest
    @Override
    public Result getSkillUserDetail(@QueryParam("skillId") Long skillId, @QueryParam("userId") Long userId) {
        if (skillId == null || userId == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long curUserId = getToken() == null ? null : getToken().getUserId();
        Long familyId = familyService.getFamilyId(userId);
        return new Result(familyService.getSkillUserDetail(familyId, userId, skillId, curUserId));
    }

    /**
     * 获取我的家族以及签约状态
     *
     * @return
     */
    @GET
    @Path("getFamily")
    @Override
    public Result getFamily() {
        Long userId = this.getToken().getUserId();
        Long familyIdIgnoreStatus = familyService.getFamilyIdIgnoreStatus(userId);
        FamilyAuditVO lastApplyInfo = null;

        // 家族长
        if (familyService.isFamilyPresident(userId)) {
            // 模拟提交
            lastApplyInfo = new FamilyAuditVO();
            lastApplyInfo.setApplyType(ApplyTypeEnum.JOIN.getCode());
            lastApplyInfo.setAuditStatus(AuditStatusEnum.PASS.getCode());
        } else {
            lastApplyInfo = familyService.getLastApplyInfo(userId, familyIdIgnoreStatus, null);
        }

        MutableResult mResult = new MutableResult();

        if (lastApplyInfo == null || (lastApplyInfo.getAuditStatus() == AuditStatusEnum.PASS.getCode() && lastApplyInfo.getApplyType() == ApplyTypeEnum.LEAVE.getCode())) {
            // 未提交或退会成功
            mResult.put("familyId", null);
            mResult.put("familyName", "");
            mResult.put("isSkillSubmit", false);
        } else {
            mResult.put("familyId", familyIdIgnoreStatus);
            mResult.put("familyName", familyIdIgnoreStatus != null ? familyService.getFamilyName(familyIdIgnoreStatus) : "");
            mResult.put("isSkillSubmit", familyIdIgnoreStatus != null ? familyService.isSkillSubmit(userId, familyIdIgnoreStatus) : false);
        }

        mResult.put("lastApplyType", Optional.ofNullable(lastApplyInfo).map(FamilyAuditVO::getApplyType).orElse(null));
        mResult.put("lastAuditStatus", Optional.ofNullable(lastApplyInfo).map(FamilyAuditVO::getAuditStatus).orElse(null));
        return mResult.build();
    }

    /**
     * 强制解约申请
     *
     * @param applyMsg
     * @return
     */
    @POST
    @Path("forceLeave")
    @Override
    public Result forceLeave(@FormParam("applyMsg") String applyMsg) {
        Long curUserId = this.getToken().getUserId();

        if (familyService.isFamilyPresident(curUserId)) {
            return new Result(GlobalApiError.ServerError, "家族长不能进行解约/签约操作");
        }

        Long familyId = familyService.getFamilyId(curUserId);
        if (familyId == null) {
            return new Result(GlobalApiError.ServerError, "您没有签约家族");
        }

        long curTime = System.currentTimeMillis();

        // 最近一次申请
        FamilyAuditDO lastAudit = familyService.getFamilyAudit(familyId, curUserId, ApplyTypeEnum.LEAVE.getCode(), AuditStatusEnum.WAIT.getCode());
        if (lastAudit != null) {
            if (Optional.ofNullable(lastAudit.getIsForce()).orElse(BoolIntVal.False) == BoolIntVal.False) {
                // 自动拒绝
                familyService.setFamilyAuditStatus(lastAudit.getAuditId(), AuditStatusEnum.DENY.getCode(), "拒绝当前的申请，提交强制解约", curTime, 0L);
            } else {
                return new Result(GlobalApiError.ServerError, String.format("已提交强制解约，于%d生效", lastAudit.getForceEffectDate()));
            }
        }

        FamilyAuditDO auditDO = new FamilyAuditDO();
        auditDO.setAuditId(identityService.getId());
        auditDO.setFamilyId(familyId);
        auditDO.setUserId(curUserId);
        auditDO.setApplyMsg(applyMsg);
        auditDO.setApplyTime(curTime);
        auditDO.setAuditStatus(AuditStatusEnum.WAIT.getCode());
        auditDO.setApplyType(ApplyTypeEnum.LEAVE.getCode());
        auditDO.setIsForce(BoolIntVal.True);
        Calendar calendar = DateUtil.calendar();
        calendar.add(Calendar.DAY_OF_MONTH, 14);
        auditDO.setForceEffectDate(Convert.toLong(DateUtil.format(calendar.getTime(), "yyyyMMdd")));
        if (familyService.addFamilyAudit(auditDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }
}
