package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.CommentAddReq;
import com.starbuds.server.api.app.v1.facade.FeedFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.feed.FeedActionEnum;
import com.starbuds.server.common.constant.feed.FeedActionTargetEnum;
import com.starbuds.server.common.constant.feed.FeedStatusEnum;
import com.starbuds.server.common.constant.feed.FeedTypeEnum;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.ShareParam;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.feed.*;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RtcRoomCategoryService;
import com.starbuds.server.common.service.feed.FeedActivityInfoService;
import com.starbuds.server.common.service.feed.FeedService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.assertj.core.internal.bytebuddy.implementation.bytecode.Throw;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.*;
import java.util.*;

/**
 * 动态
 *
 * @author aoe
 * @date 2020/5/27
 */
@Path("/v1/feed")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FeedFacadeImpl extends BaseFacade implements FeedFacade {

    @Reference(version = "1.0.0")
    private FeedService feedService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Value("${share.url}")
    private String shareUrl;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    RtcRoomCategoryService rtcRoomCategoryService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    FeedActivityInfoService feedActivityInfoService;

    /**
     * 创建内容
     *
     * @param feedType      媒体类型
     * @param url           内容链接地址
     * @param feedTitle     标题（一句话心情）
     * @param feedCover     （视频）封面链接地址
     * @param feedContent   内容
     * @param feedLongitude 经度
     * @param feedLatitude  纬度
     * @param feedPosName   位置名称
     * @return
     */
    @POST
    @Path("createFeed")
    @Override
    public Result createFeed(@FormParam("feedType") Integer feedType,
                             @FormParam("url") String url,
                             @FormParam("feedTitle") String feedTitle,
                             @FormParam("feedCover") String feedCover,
                             @FormParam("feedContent") String feedContent,
                             @FormParam("feedLongitude") Double feedLongitude,
                             @FormParam("feedLatitude") Double feedLatitude,
                             @FormParam("feedPosName") String feedPosName,
                             @FormParam("photoWall") Integer photoWall) {
        if (feedType == null || StringUtils.isBlank(url)) {
            return new Result(GlobalApiError.BadRequest, "必须传递 feedType、url");
        }
        FeedTypeEnum feedTypeEnum = FeedTypeEnum.toEnum(feedType);
        if (feedTypeEnum == null) {
            return new Result(GlobalApiError.BadRequest, "feedType 不正确");
        }
        long photoCount = Arrays.stream(url.split(",")).count();
        if (FeedTypeEnum.PHOTO == feedTypeEnum
                && photoCount > 9) {
            return new Result(GlobalApiError.BadRequest, "最多上传9张");
        }
        Long userId = this.getToken().getUserId();

        FeedDO feed = new FeedDO();
        feed.setUserId(userId);
        feed.setFeedType(feedTypeEnum.value());
        feed.setUrl(url);
        feed.setFeedTitle(feedTitle);
        feed.setFeedCover(feedCover);
        feed.setFeedContent(feedContent);
        feed.setFeedLongitude(feedLongitude);
        feed.setFeedLatitude(feedLatitude);
        feed.setFeedPosName(feedPosName);
        feed.setVersion(1L);
        feed.setViewCount(0);
        feed.setLikeCount(0);
        feed.setCollectCount(0);
        feed.setCommentCount(0);
        feed.setShareCount(0);
        feed.setSort(0);
        feed.setAdminId(0L);
        feed.setFeedStatus(feedTypeEnum == FeedTypeEnum.PHOTO ? FeedStatusEnum.AUDIT_ROBOT.value()
                : FeedStatusEnum.REVIEW_ROBOT.value());
        feed.setPhotoWall(photoWall);
        if (photoWall != null && photoWall == 1) {
            // 数量判断
            if (feedService.getPhotoWallCount(userId) + photoCount > 16) {
                return new Result(GlobalApiError.ServerError, "16张照片墙已满，请取消部分照片墙图片，才能再同步新的美照到照片墙。");
            }
        }

        // 存储并进行资源内容百度风控绿色检查
        Long feedId = feedService.createFeed(feed, this.getHeader().getChannelId());

        if (feedId == null) {
            log.error("创建内容失败 feed={}", feed);
            return new Result(GlobalApiError.OperationFailed, "创建失败");
        }

        return new Result(feedId);
    }

    /**
     * 添加评论
     *
     * @param req
     * @return
     */
    @POST
    @Path("addComment")
    @Override
    public Result addComment(@BeanParam CommentAddReq req) {

        if (req.getReplyUserId() == null) {
            req.setReplyUserId(0L);
        }

        if (req.getReplyId() == null) {
            req.setReplyId(0L);
        }

        if (req.getOrigReplyId() == null) {
            req.setOrigReplyId(0L);
        }

        if (StringUtils.isEmpty(req.getContent())) {
            return new Result(GlobalApiError.ServerError, "请输入评论内容");
        }

        if (!feedService.isFeedExist(req.getFeedId())) {
            return new Result(GlobalApiError.ServerError, "内容不存在");
        }

        // 更新内容评论数量
        if (!feedService.changeCommentCount(req.getFeedId(), 1)) {
            log.info("更新内容评论数量失败");
            return new Result(GlobalApiError.OperationFailed, "评论失败");
        }

        FeedCommentDO commentDO = new FeedCommentDO();
        BeanUtils.copyProperties(req, commentDO);

        if (req.getReplyId() > 0) {
            // 答复评论的时候
            if (!feedService.isCommentExist(req.getReplyId())) {
                return new Result(GlobalApiError.ServerError, "答复的评论不存在");
            }
            if (!feedService.changeFeedCommentCount(req.getReplyId(), 1)) {
                log.info("更新当前评论数量失败");
                return new Result(GlobalApiError.OperationFailed, "评论失败");
            }
            // 顶层回复
            Long oriReplyId = feedService.getOriReplyId(req.getReplyId());
            // 回复的上级
            Long replyId = feedService.getReplyId(req.getReplyId());
            if (oriReplyId > 0) {
                commentDO.setOrigReplyId(oriReplyId);
                if (!feedService.changeFeedCommentCount(oriReplyId, 1)) {
                    log.info("更新顶层评论数量失败");
                    return new Result(GlobalApiError.OperationFailed, "评论失败");
                }
            }
            if (replyId > 0) {
                if (oriReplyId == 0) {
                    commentDO.setOrigReplyId(replyId);
                }
                feedService.changeFeedCommentCount(replyId, 1);
            }
        }

        commentDO.setCommentId(identityService.getId());
        commentDO.setStatus(CommonStatus.Normal);
        commentDO.setCreateTime(System.currentTimeMillis());
        commentDO.setUserId(this.getToken().getUserId());
        commentDO.setIsHot(BoolIntVal.False);
        commentDO.setLikeCount(0);
        commentDO.setVersion(1L);
        commentDO.setCommentCount(0);

        if (feedService.addComment(commentDO)) {
            return new Result(commentDO.getCommentId());
        }

        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 添加动作
     *
     * @param targetId
     * @param actionType
     * @param targetType
     * @return
     */
    @POST
    @Path("addAction")
    @Override
    public Result addAction(@FormParam("targetId") Long targetId,
                            @FormParam("actionType") Integer actionType,
                            @FormParam("targetType") Integer targetType) {

        if (targetId == null || actionType == null || targetType == null) {
            return new Result(GlobalApiError.OperationFailed, "缺少必要参数");
        }

        FeedActionTargetEnum targetEnum = FeedActionTargetEnum.toEnum(targetType);
        if (targetEnum == null) {
            return new Result(GlobalApiError.ServerError, "暂不支持操作此目标");
        }

        if (targetEnum == FeedActionTargetEnum.COMMENT) {
            if (!feedService.isCommentExist(targetId)) {
                return new Result(GlobalApiError.ServerError, "评论不存在");
            }
        } else {
            if (!feedService.isFeedExist(targetId)) {
                return new Result(GlobalApiError.ServerError, "内容不存在");
            }
        }

        FeedActionEnum actionEnum = FeedActionEnum.toEnum(actionType);
        if (actionEnum == null) {
            return new Result(GlobalApiError.ServerError, "暂不支持此动作类型");
        }

        // 是否取消
        boolean isCancel = false;
        Integer actionCode = actionEnum.getCode();

        switch (actionEnum) {
            case LIKE:
                if (targetEnum == FeedActionTargetEnum.COMMENT) {
                    feedService.changeCommentLikeCount(targetId, 1);
                } else {
                    feedService.changeLikeCount(targetId, 1);
                }
                break;
            case LIKE_CANCEL:
                isCancel = true;
                actionCode = FeedActionEnum.LIKE.getCode();
                if (targetEnum == FeedActionTargetEnum.COMMENT) {
                    feedService.changeCommentLikeCount(targetId, -1);
                } else {
                    feedService.changeLikeCount(targetId, -1);
                }
                break;
            case SHARE:
                feedService.changeShareCount(targetId, 1);
                break;
            case COLLECT:
                feedService.changeCollectCount(targetId, 1);
                break;
            case COLLECT_CANCEL:
                isCancel = true;
                actionCode = FeedActionEnum.COLLECT.getCode();
                feedService.changeCollectCount(targetId, -1);
                break;
            default:
                break;
        }

        Long userId = this.getToken().getUserId();
        long curTime = System.currentTimeMillis();

        FeedActionDO actionDO = new FeedActionDO();
        actionDO.setUserId(userId);
        actionDO.setTargetId(targetId);
        actionDO.setTargetType(targetType);
        actionDO.setActionType(actionCode);
        actionDO.setStatus(isCancel ? CommonStatus.Disabled : CommonStatus.Normal);
        actionDO.setCreateTime(curTime);
        actionDO.setVersion(1L);

        if (feedService.addAction(actionDO)) {
            return new Result();
        }

        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取相册
     *
     * @param lastId
     * @param userId
     * @return
     */
    @GET
    @Path("getPhotoList")
    @PermitGuest
    @Override
    public Result getPhotoList(@QueryParam("lastId") Long lastId, @QueryParam("userId") Long userId) {
        if (userId == null && getToken() != null) {
            userId = getToken().getUserId();
        }
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的参数");
        }
        return new Result(feedService.getFeedList(lastId, GlobalConfig.PAGE_SIZE, FeedTypeEnum.PHOTO.value(), userId, isSelf(userId)));
    }

    /**
     * 获取视频
     *
     * @param lastId
     * @param userId
     * @return
     */
    @GET
    @Path("getVideoList")
    @Override
    public Result getVideoList(@QueryParam("lastId") Long lastId, @QueryParam("userId") Long userId) {
        if (userId == null) {
            userId = this.getToken().getUserId();
        }
        return new Result(feedService.getFeedList(lastId, GlobalConfig.PAGE_SIZE, FeedTypeEnum.VIDEO.value(), userId, isSelf(userId)));
    }

    /**
     * 获取评论
     *
     * @param feedId
     * @param lastId
     * @return
     */
    @GET
    @Path("getCommentList")
    @Override
    public Result getCommentList(@QueryParam("feedId") Long feedId,
                                 @QueryParam("lastId") Long lastId) {
        if (!feedService.isFeedExist(feedId)) {
            return new Result(GlobalApiError.ServerError, "内容不存在");
        }
        return new Result(feedService.getCommentList(feedId, this.getToken().getUserId(), lastId, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 删除内容
     *
     * @return
     */
    @POST
    @Path("delFeed")
    @Override
    public Result delFeed(@FormParam("feedId") Long feedId) {
        if (!feedService.isFeedExist(feedId)) {
            return new Result(GlobalApiError.ServerError, "内容不存在");
        }
        if (!this.getToken().getUserId().equals(feedService.getUserId(feedId))) {
            return new Result(GlobalApiError.ServerError, "仅作者才可删除");
        }
        if (feedService.delFeed(feedId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取视频
     *
     * @param lastId
     * @return
     */
    @GET
    @Path("getVideoStreamList")
    @Override
    public Result getVideoStreamList(@QueryParam("lastId") Long lastId) {
        return new Result(feedService.getFeedList(lastId, GlobalConfig.PAGE_SIZE, FeedTypeEnum.VIDEO.value(), null, BoolIntVal.False));
    }

    /**
     * 添加查看
     *
     * @return
     */
    @POST
    @Path("addViewCount")
    @Override
    public Result addViewCount(@FormParam("feedId") Long feedId) {

        if (!feedService.isFeedExist(feedId)) {
            return new Result(GlobalApiError.ServerError, "内容不存在");
        }

        if (feedService.addViewCount(feedId, this.getHeader().getChannelId())) {
            return new Result();
        }

        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 删除评论
     *
     * @param commentId
     * @return
     */
    @POST
    @Path("delComment")
    @Override
    public Result delComment(@FormParam("commentId") Long commentId) {

        if (!feedService.isCommentExist(commentId)) {
            return new Result(GlobalApiError.ServerError, "评论不存在");
        }

        Long commentUserId = feedService.getCommentUserId(commentId);

        if (!this.getToken().getUserId().equals(commentUserId)) {
            return new Result(GlobalApiError.ServerError, "仅发布人才可删除");
        }

        // 上级id 评论数减1
        Long replyId = feedService.getReplyId(commentId);

        Long feedId = feedService.getFeedId(commentId);

        // 更新内容的评论数量
        if (!feedService.changeCommentCount(feedId, -1)) {
            log.info("修改内容评论数量失败");
            return new Result(GlobalApiError.ServerError, "删除评论失败");
        }

        // 更新评论的评论数量
        if (replyId != null && replyId > 0) {
            feedService.changeFeedCommentCount(replyId, -1);
            Long parentId = feedService.getReplyId(replyId);
            // 更新顶层评论的数量
            if (parentId != null && parentId > 0) {
                feedService.changeFeedCommentCount(parentId, -1);
            }
        }

        if (feedService.delComment(commentId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取子评论
     *
     * @param lastId
     * @param commentId
     * @return
     */
    @GET
    @Path("getChildCommentList")
    @Override
    public Result getChildCommentList(@QueryParam("lastId") Long lastId,
                                      @QueryParam("commentId") Long commentId) {
        if (!feedService.isCommentExist(commentId)) {
            return new Result(GlobalApiError.OperationFailed, "主评论不存在");
        }
        return new Result(feedService.getChildCommentList(lastId, this.getToken().getUserId(), commentId, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取视频（游客）
     *
     * @param lastId
     * @param userId
     * @return
     */
    @GET
    @Path("getVideoListGuest")
    @PermitGuest
    @Override
    public Result getVideoListGuest(@QueryParam("lastId") Long lastId, @QueryParam("userId") Long userId) {
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "必须传 userId");
        }

        return new Result(feedService.getFeedList(lastId, GlobalConfig.PAGE_SIZE, FeedTypeEnum.VIDEO.value(), userId, BoolIntVal.False));
    }

    /**
     * 获取内容详情（游客）
     *
     * @param feedId 内容Id
     * @return
     */
    @GET
    @Path("getFeedDetailGuest")
    @PermitGuest
    @Override
    public Result getFeedDetailGuest(@QueryParam("feedId") Long feedId) {
        if (feedId == null) {
            return new Result(GlobalApiError.BadRequest, "必须传 feedId");
        }
        FeedVO feedVO = feedService.getFeedDetail(feedId, FeedStatusEnum.PASSED.value(), BoolIntVal.False);
        if (feedVO == null) {
            return new Result(GlobalApiError.BadRequest, "feed 不存在");
        }
        return new Result(feedVO);
    }

    /**
     * 获取内容详情
     *
     * @param feedId 内容Id
     * @return
     */
    @GET
    @Path("getFeedDetail")
    @Override
    public Result getFeedDetail(@QueryParam("feedId") Long feedId) {
        if (feedId == null) {
            return new Result(GlobalApiError.BadRequest, "缺少参数");
        }
        Long userId = this.getToken().getUserId();
        FeedSquareVO vo = feedService.getFeedDetail(feedId, userId);
        if (vo == null) {
            return new Result(GlobalApiError.BadRequest, "动态迷路了");
        }
        return Result.ofSuccess(vo);
    }

    /**
     * 照片墙
     *
     * @param feedId
     * @param photoWall
     * @return
     */
    @POST
    @Path("setPhotoWall")
    @Override
    public Result setPhotoWall(@FormParam("feedId") Long feedId,
                               @FormParam("photoWall") Integer photoWall) {
        if (photoWall == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        FeedDO feed = feedService.getFeed(feedId);
        if (feed == null) {
            return new Result(GlobalApiError.ServerError, "未找到该内容");
        }
        Long userId = this.getToken().getUserId();
        if (!userId.equals(feed.getUserId())) {
            return new Result(GlobalApiError.ServerError, "仅发布者才可操作");
        }
        if (feed.getFeedType() != FeedTypeEnum.PHOTO.value()) {
            return new Result(GlobalApiError.ServerError, "仅照片才可操作");
        }
        // 数量判断
        if (photoWall == 1 && feedService.getPhotoWallCount(userId) >= 16) {
            return new Result(GlobalApiError.ServerError, "16张照片墙已满，请取消部分照片墙图片，才能再同步新的美照到照片墙。");
        }
        if (feedService.setPhotoWall(feedId, photoWall)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取照片墙
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getPhotoWall")
    @PermitGuest
    @Override
    public Result getPhotoWall(@QueryParam("userId") Long userId) {
        if (userId == null && getToken() != null) {
            userId = getToken().getUserId();
        }
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的参数");
        }
        return new Result(feedService.getPhotoWall(userId));
    }

    /**
     * 获取内容按天分组分页列表
     *
     * @param userId
     * @param pageNum  当前页码
     * @param pageSize 分页大小
     * @return 分页对象
     */
    @GET
    @Path("getFeedByDatePageList")
    @Override
    public Result getFeedByDatePageList(@QueryParam("userId") Long userId,
                                        @QueryParam("pageNum") int pageNum,
                                        @QueryParam("pageSize") int pageSize) {
        if (userId == null || userId < 1) {
            return new Result(GlobalApiError.BadRequest, "无效的用户id");
        }
        if (pageNum < 1 || pageSize < 1) {
            return new Result(GlobalApiError.BadRequest, "无效的分页参数");
        }

        Integer feedStatus = userId - getToken().getUserId() == 0 ? null : FeedStatusEnum.PASSED.value();
        Long myselfUserId = getToken().getUserId();
        return new Result(feedService.getFeedByDatePageList(userId, myselfUserId, feedStatus, pageNum, pageSize));
    }

    /**
     * 获取广场的动态列表
     *
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    @GET
    @Path("getSquareFeedList")
    public Result getSquareFeedList(@QueryParam("pageNum") Integer pageNum,
                                    @QueryParam("pageSize") Integer pageSize) {
        Long userId = this.getToken().getUserId();
        PageList<FeedSquareVO> res = feedService.getSquareFeedList(pageNum, pageSize, userId);
        return Result.ofSuccess(res);
    }

    /**
     * 点赞以及取消点赞
     *
     * @param feedId
     * @return
     */
    @Override
    @GET
    @Path("thumbsUp")
    public Result thumbsUp(@QueryParam("feedId") Long feedId) {
        Long userId = this.getToken().getUserId();
        FeedThumbsUpResVO res = feedService.thumbsUp(feedId, userId);
        return Result.ofSuccess(res);
    }


    /**
     * 是否自己
     *
     * @param userId
     * @return
     */
    private Integer isSelf(Long userId) {
        if (getToken() == null) {
            return BoolIntVal.False;
        }
        return userId.equals(this.getToken().getUserId()) ? BoolIntVal.True : BoolIntVal.False;
    }

    /**
     * 获取咨询内容
     * @param infoId
     * @return
     */
    @POST
    @Path("getFeedActivityInfo")
    @Override
    @PermitAll
    public Result getFeedActivityInfo(@FormParam("infoId") Long infoId) {

        Map<String, Object> result = new HashMap<>();

        //资讯标题附加信息
        FeedActivityInfoVO mod = feedActivityInfoService.getFeedActivityInfo(infoId);
        if (mod != null) {
            result.put("type", mod.getType());
            result.put("createTime", mod.getCreateTime());
            result.put("showStyle", mod.getShowStyle());
            result.put("title", mod.getTitle());
            result.put("infoTitle", mod.getInfoTitle());
            result.put("sort", mod.getSort());
            result.put("source", mod.getSource());
            result.put("status", mod.getStatus());
        } else {
            return new Result(result);
        }

        //资讯内容
        List<FeedActivityInfoItemVO> ites = feedActivityInfoService.getFeedActivityInfoItesm(infoId);
        List<Map> content = new ArrayList<>();
        for (FeedActivityInfoItemVO m : ites) {
            Map<String, Object> map = new HashMap<>();
            int type = m.getType();
            String value = m.getValue();
            map.put("type", type);


            //文本或者图片
            if (type == 0 || type == 1) {
                map.put("value", value);
            }

            //房间
            if (type == 2) {
                //根据房间ID获取房间信息 value为房间id
                RtcRoom room = rtcRoomService.getRoom(Long.parseLong(value));
                if (room != null) {
                    map.put("icon", room.getRoomCover());
                    map.put("roomType", RoomTypeEnum.toEnum(room.getRoomType()).getDesc());
                    map.put("id", room.getRoomId());
                    map.put("title", room.getRoomName());
                    map.put("subTitle", room.getRoomNo());
                    map.put("value", room.getRoomId());

                    //类型背景
                    long[] ids = {room.getCategoryId()};
                    List<RtcRoomCategoryDO> ctList = rtcRoomCategoryService.findByIds(ids);
                    RtcRoomCategoryDO category = ctList.get(0);
                    if (category != null) {
                        if (StringUtils.isNotEmpty(category.getBackgroundImage())) {
                            map.put("roomTypeColor", category.getBackgroundImage());
                        } else {
                            map.put("roomTypeColor", category.getBackgroundColor());
                        }
                    }
                    map.put("status", room.getRoomHeat());
                }

            }

            //个人
            if (type == 3) {
                //根据用户id获取信息 value为用户id
                UserDO user = userService.getUserById(Long.parseLong(value));
                if (user != null) {
                    RtcRoomListItem userStayRoom = rtcRoomService.getUserStayRoom(user.getUserId());
                    String stayRoomId = userStayRoom == null ? "" : userStayRoom.getRoomId() + "";

                    map.put("icon", user.getUserAvatar());
                    map.put("id", user.getUserId());
                    map.put("title", user.getUserName());
                    map.put("subTitle", user.getUserAge());
                    map.put("sex", user.getUserSex());
                    map.put("isonline", user.getIsOnline());
                    map.put("status", stayRoomId);
                    map.put("value", user.getUserId());
                    map.put("userNo", user.getUserNo());
                }
            }

            if (type == 4) {
                JSONObject obj = JSON.parseObject(value);
                map.put("title", obj.getString("title"));
                map.put("value", value);
                map.put("link", obj.getString("value"));
            }

            content.add(map);
        }

        if (content.size() > 0) {
            result.put("content", content);
        }

        return new Result(result);
    }


    /**
     * 获取首页资讯
     *
     * @return
     */
    @GET
    @Path("getFeedActivityInfoList")
    @Override
    @PermitAll
    public Result getFeedActivityInfoList() {
        List<FeedActivityInfoVO> res = feedActivityInfoService.getMainPageFeedActivityInfoList();
        return Result.ofList(res);
    }

    /**
     * 随机获取一个三天内发表过状态的关注的用户
     *
     * @return
     */
    @Override
    @Path("getUserFollowName")
    @GET
    public Result getUserFollowName() {
        Long userId = this.getToken().getUserId();
        if (userId == null || userId == 0) {
            return Result.ofSuccess();
        }
        String userName = feedService.getUserFollowName(userId);
        Map<String, String> res = new HashMap<>();
        res.put("userName", userName);
        return Result.ofSuccess(res);
    }
}
