package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.app.config.GameDevConfig;
import com.starbuds.server.api.app.pojo.*;
import com.starbuds.server.api.app.provider.GameDevProvider;
import com.starbuds.server.api.app.v1.facade.GameDevFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.GameService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.game.sdkdev.param.*;
import com.starbuds.server.game.sdkdev.request.*;
import com.starbuds.server.game.sdkdev.response.GetGamesResponse;
import com.starbuds.server.game.sdkdev.response.GetOrderTotalResponse;
import com.starbuds.server.game.sdkdev.response.data.GetGamesData;
import com.starbuds.server.game.sdkdev.response.data.GetOrderTotalData;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * 游戏Dev接口实现
 *
 * @author aoe
 * @date 2020/9/23
 */
@Path("/v1/gamedev")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class GameDevFacadeImpl extends BaseFacade implements GameDevFacade {

    @Resource
    private GameDevConfig gameDevConfig;

    @Reference(version = "1.0.0")
    private GameService gameService;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource
    private GameDevProvider gameDevProvider;

    /**
     * 初始化用户
     *
     * @return
     */
    @POST
    @Path("initUser")
    @Override
    public Result initUser() {
        String userId = getUserIdStr();
        InitUserParam param = InitUserParam.builder()
                .userid(userId)
                .playPwd(userId)
                .build();
        InitUserRequest request = new InitUserRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 获取游戏列表
     *
     * @return
     */
    @GET
    @Path("getGameList")
    @Override
    public Result getGameList() {
        return gameDevProvider.getResult(getGameListResponse());
    }

    /**
     * 获取游戏
     *
     * @param gameId 游戏id
     * @return
     */
    @GET
    @Path("getGame")
    @Override
    public Result getGame(@QueryParam("gameId") String gameId) {
        if (StringUtils.isBlank(gameId)) {
            return new Result(GlobalApiError.BadRequest, "gameId不能为空");
        }
        GetGamesResponse res = getGameListResponse();
        if (res == null || res.getData() == null) {
            return new Result(GlobalApiError.OperationFailed, "获取游戏失败");
        }

        for (GetGamesData game : res.getData()) {
            if (game == null) {
                continue;
            }
            if (gameId.equals(game.getGameId())) {
                return new Result(game);
            }
        }
        return new Result(GlobalApiError.BadRequest, "无效的gameId");
    }

    /**
     * 获取玩家余额
     *
     * @return
     */
    @GET
    @Path("getBalance")
    @Override
    public Result getBalance() {
        GetBalanceParam param = GetBalanceParam.builder()
                .userid(getUserIdStr())
                .build();
        GetBalanceRequest request = new GetBalanceRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 获取指定游戏期号信息
     *
     * @return
     */
    @GET
    @Path("getGamePlay")
    @Override
    public Result getGamePlay(@QueryParam("gameId") String gameId) {
        if (StringUtils.isBlank(gameId)) {
            return new Result(GlobalApiError.BadRequest, "gameId必填");
        }

        GetGameplayParam param = GetGameplayParam.builder()
                .userid(getUserIdStr())
                .gameId(gameId)
                .build();
        GetGameplayRequest request = new GetGameplayRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 获取开奖历史分页数据
     *
     * @param gameId
     * @param pageIndex 当前页码 从1开始
     * @param pageSize  分页大小
     * @return
     */
    @GET
    @Path("getWinNosPageList")
    @Override
    public Result getWinNosPageList(@QueryParam("gameId") String gameId,
                                    @QueryParam("pageIndex") Integer pageIndex,
                                    @QueryParam("pageSize") Integer pageSize) {
        if (StringUtils.isBlank(gameId)) {
            return new Result(GlobalApiError.BadRequest, "gameId必填");
        }
        if (pageIndex == null) {
            return new Result(GlobalApiError.BadRequest, "pageIndex必填");
        }
        if (pageSize == null) {
            return new Result(GlobalApiError.BadRequest, "pageSize必填");
        }

        GetWinnosParam param = GetWinnosParam.builder()
                .userid(getUserIdStr())
                .gameId(gameId)
                .page("" + pageIndex)
                .size("" + pageSize)
                .build();
        GetWinnosRequest request = new GetWinnosRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 开奖
     *
     * @return
     */
    @GET
    @Path("getNewWinNo")
    @Override
    public Result getNewWinNo() {
        GetNewWinnoParam param = GetNewWinnoParam.builder()
                .userid(getUserIdStr())
                .build();
        GetNewWinnoRequest request = new GetNewWinnoRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 获取指定游戏玩法
     *
     * @param gameId
     * @return
     */
    @GET
    @Path("getGamePlayInfo")
    @Override
    public Result getGamePlayInfo(@QueryParam("gameId") String gameId) {
        if (StringUtils.isBlank(gameId)) {
            return new Result(GlobalApiError.BadRequest, "gameId必填");
        }

        GetPlayinfoParam param = GetPlayinfoParam.builder()
                .userid(getUserIdStr())
                .gameId(gameId)
                .build();
        GetPlayinfoRequest request = new GetPlayinfoRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 进入下一期
     *
     * @param gameId
     * @return
     */
    @GET
    @Path("getNextPeriod")
    @Override
    public Result getNextPeriod(@QueryParam("gameId") String gameId) {
        if (StringUtils.isBlank(gameId)) {
            return new Result(GlobalApiError.BadRequest, "gameId必填");
        }

        GetNextPeriodParam param = GetNextPeriodParam.builder()
                .userid(getUserIdStr())
                .gameId(gameId)
                .build();
        GetNextPeriodRequest request = new GetNextPeriodRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 添加游戏币订单(划转资产，转入/转出)
     *
     * @param operate 操作（10:转入;20:转出）
     * @param coin    游戏币数量(单位：转入：游戏币；转出：人民币)
     * @return true:成功
     */
    @POST
    @Path("addGameCoinOrder")
    @Override
    public Result addGameCoinOrder(@FormParam("operate") Integer operate,
                                   @FormParam("coin") Long coin) {
        if (coin == null) {
            return new Result(GlobalApiError.BadRequest, "游戏币数量不能为空");
        }
        if (coin < 1) {
            return new Result(GlobalApiError.BadRequest, "游戏币数量必须大于0");
        }
        GameCoinOrderOperateEnum operateEnum = GameCoinOrderOperateEnum.toEnum(operate);
        if (operateEnum == null) {
            return new Result(GlobalApiError.BadRequest, "无效的操作代码");
        }

        Long userId = getToken().getUserId();
        GameDevResult orderResult = gameDevProvider.addGameCoinOrder(operateEnum, coin, userId);
        if (!orderResult.getIsSuccess()) {
            return new Result(GlobalApiError.OperationFailed, orderResult.getErrMsg());
        }

        return new Result(orderResult.getOrderId());
    }

    /**
     * 游戏Dev下单
     *
     * @param form
     * @return
     */
    @POST
    @Path("addOrder")
    @Override
    public Result addOrder(@BeanParam AddGameDevOrderForm form) {
        List<AddGameDevOrderDetailForm> details = JSONObject.parseArray(form.getDetails(), AddGameDevOrderDetailForm.class);
        Integer bet = form.getBet();
        Long userId = getToken().getUserId();
        AddOrderParam param = AddOrderParam.builder()
                .userid("" + userId)
                .gameId(form.getGameId())
                // 下单期号
                .byt1CurPno(form.getCurPeriod())
                // 倍数
                .byt1TtBet("" + bet)
                // 总额 = 单价 * 倍数
                .byt1TtMoney("" + getOrderTotal(details, bet))
                // 明细
                .btDtls(getOrderDetailStr(details, bet))
                .build();
        AddOrderRequest request = new AddOrderRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 获取订单列表
     *
     * @param gameId
     * @param status    状态 0全部 1未开奖 2中奖
     * @param pageIndex 当前页码 从1开始
     * @param pageSize  分页大小
     * @return
     */
    @GET
    @Path("getOrderPageList")
    @Override
    public Result getOrderPageList(@QueryParam("gameId") String gameId,
                                   @QueryParam("status") Integer status,
                                   @QueryParam("pageIndex") Integer pageIndex,
                                   @QueryParam("pageSize") Integer pageSize) {
        GetOrderListParam param = GetOrderListParam.builder()
                .userid(getToken().getUserId() + "")
                .gameId(gameId)
                .status("" + status)
                .page("" + pageIndex)
                .size("" + pageSize)
                .build();
        GetOrderListRequest request = new GetOrderListRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 游戏订单详情
     *
     * @param orderNo 订单号
     * @return
     */
    @GET
    @Path("getOrderDetail")
    @Override
    public Result getOrderDetail(@QueryParam("orderNo") String orderNo) {
        GetOrderDetailParam param = GetOrderDetailParam.builder()
                .userid(getToken().getUserId() + "")
                .orderno(orderNo)
                .build();
        GetOrderDetailRequest request = new GetOrderDetailRequest();
        request.setParam(param);
        return gameDevProvider.getResult(gameDevConfig.client().execute(request));
    }

    /**
     * 订单统计数据（自主游戏、第三方游戏总和）
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     */
    @GET
    @Path("getOrderStatistic")
    @Override
    public Result getOrderStatistic(@QueryParam("startTime") Long startTime, @QueryParam("endTime") Long endTime) {
        if (startTime == null) {
            return new Result(GlobalApiError.BadRequest, "startTime不能为空");
        }
        if (endTime == null) {
            return new Result(GlobalApiError.BadRequest, "endTime不能为空");
        }

        GameStatisticReq statisticReq = new GameStatisticReq();
        List<GameStatisticItem> list = new ArrayList<>();
        statisticReq.setList(list);

        GetOrderTotalResponse devRes = getGameDevOrderStatistic(getToken().getUserId() + "",
                startTime, endTime);
        if (devRes != null && devRes.getData() != null) {
            GetOrderTotalData data = devRes.getData();
            statisticReq.setBettingTotal(data.getBettingAmount());
            statisticReq.setWinningTotal(data.getWinningAmount());

            GameStatisticItem item = new GameStatisticItem();
            // 类型 1：自主游戏(Game Dev)
            item.setType(1);
            item.setName("直播游戏");
            item.setBettingAmount(data.getBettingAmount());
            item.setWinningAmount(data.getWinningAmount());
            item.setBettingCount(NumberUtil.isNumber(data.getBettingCount())
                    ? Integer.parseInt(data.getBettingCount())
                    : 0);
            list.add(item);
        }

        // 统计数据
        if (list.isEmpty()) {
            statisticReq.setBettingTotal("0.00");
            statisticReq.setWinningTotal("0.00");
        } else {
            BigDecimal bettingTotal = BigDecimal.ZERO;
            BigDecimal winningTotal = BigDecimal.ZERO;

            for (GameStatisticItem item : list) {
                String bettingAmount = item.getBettingAmount();
                String winningAmount = item.getWinningAmount();

                if (NumberUtil.isNumber(bettingAmount)) {
                    bettingTotal = bettingTotal.add(new BigDecimal(bettingAmount));
                }
                if (NumberUtil.isNumber(winningAmount)) {
                    winningTotal = winningTotal.add(new BigDecimal(winningAmount));
                }
            }

            DecimalFormat df = new DecimalFormat("0.00");
            statisticReq.setBettingTotal(df.format(bettingTotal));
            statisticReq.setWinningTotal(df.format(winningTotal));
        }

        return new Result(statisticReq);
    }


    private GetOrderTotalResponse getGameDevOrderStatistic(String userId, Long startTime, Long endTime) {
        GetOrderTotalParam param = GetOrderTotalParam.builder()
                .userid(userId)
                .startTime("" + startTime)
                .endTime("" + endTime)
                .build();
        GetOrderTotalRequest request = new GetOrderTotalRequest();
        request.setParam(param);
        return gameDevConfig.client().execute(request);
    }

    private String getUserIdStr() {
        return getToken().getUserId() + "";
    }

    /**
     * 计算订单总额
     *
     * @param details
     * @param bet
     * @return
     */
    private Integer getOrderTotal(List<AddGameDevOrderDetailForm> details, Integer bet) {
        Integer total = 0;
        for (AddGameDevOrderDetailForm detail : details) {
            total += detail.getPrice() * bet;
        }
        return total;
    }

    /**
     * 下单明细字符串
     *
     * @param details
     * @param bet
     * @return
     */
    private String getOrderDetailStr(List<AddGameDevOrderDetailForm> details, Integer bet) {
        String split = "|";
        StringBuilder result = new StringBuilder();
        for (AddGameDevOrderDetailForm detail : details) {
            // 玩法id^玩法值^投注模式(默认1)^总倍投数^返点序列值^单价^名称
            // 多注 以 | 隔开
            String str = StrUtil.format("{}^{}^1^{}^{}^{}^{}",
                    detail.getPlayId(), detail.getDetailOrder(), bet, detail.getSeries(),
                    detail.getPrice(), detail.getPlayName());
            result.append(str).append(split);
        }
        return result.substring(0, result.lastIndexOf(split) - 1);
    }

    /**
     * 获取游戏列表 Response
     *
     * @return
     */
    private GetGamesResponse getGameListResponse() {
        GetGamesParam param = GetGamesParam.builder()
                .userid(getUserIdStr())
                .build();
        GetGamesRequest request = new GetGamesRequest();
        request.setParam(param);
        return gameDevConfig.client().execute(request);
    }
}
