package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.ImmutableSet;
import com.starbuds.server.api.app.config.UpgradeConfig;
import com.starbuds.server.api.app.pojo.gift.SendRewardGiftForm;
import com.starbuds.server.api.app.pojo.gift.vo.GiftRewardHisVO;
import com.starbuds.server.api.app.provider.RewardGiftProvider;
import com.starbuds.server.api.app.provider.SendGiftProvider;
import com.starbuds.server.api.app.v1.facade.GiftRewardFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.SendGiftRewardFeedSnapshotDO;
import com.starbuds.server.common.pojo.finance.gift.GiftReceiver;
import com.starbuds.server.common.pojo.finance.gift.SendGiftRewardFeedBO;
import com.starbuds.server.common.pojo.finance.qo.SendGiftRewardFeedSnapshotPageListQO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.feed.FeedService;
import com.starbuds.server.common.service.finance.SendGiftRewardFeedSnapshotService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.ws.rs.*;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 打赏动态送礼物
 *
 * @author aoe
 * @date 2021/11/15
 */
@Path("/v1/gift")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
@Validated
public class GiftRewardFacadeImpl extends BaseFacade implements GiftRewardFacade {

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private FeedService feedService;

    @Resource
    private UpgradeConfig upgradeConfig;

    @Resource
    private SendGiftProvider sendGiftProvider;

    @Resource
    private RewardGiftProvider rewardGiftProvider;

    @Reference(version = "1.0.0")
    private SendGiftRewardFeedSnapshotService sendGiftRewardFeedSnapshotService;

    @Reference(version = "1.0.0")
    private UserService userService;

    /**
     * (动态)打赏礼物列表
     *
     * @return
     */
    @GET
    @Path("getRewardGifts")
    @Override
    public Result getRewardGifts() {
        return Result.ofList(rewardGiftProvider.getRewardGifts());
    }

    /**
     * 赠送(动态)打赏礼物
     *
     * @param form
     * @return
     */
    @POST
    @Path("sendRewardGift")
    @Override
    public Result sendRewardGift(@BeanParam @Valid SendRewardGiftForm form) {
        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }
        // 判断礼物是否可以赠送动态
        Long giftId = form.getGiftId();
        if (!giftService.existRewardGift(giftId)) {
            return Result.ofBadRequest("无效的礼物");
        }
        log.info("赠送(动态)打赏礼物 {}", JSON.toJSONString(form, true));

        Long feedId = form.getFeedId();
        Long receiverUserId = feedService.getUserId(feedId);
        if (receiverUserId == null) {
            return Result.ofBadRequest("动态用户有误");
        }

        form.setFeedId(feedId);
        form.setUserIds(receiverUserId + "");
        form.setIsFromBag(false);

        SendGiftRewardFeedBO sendGift = new SendGiftRewardFeedBO();
        sendGift.setFeedId(feedId);
        GiftReceiver receiver = new GiftReceiver();
        sendGift.setReceiverUsers(ImmutableSet.of(receiver));
        sendGift.setAppVer(getHeader().getAppVersion());

        // 避免数据库主从同步的不确定性，赠送礼物之前读取历史数据
        long rewardValue = 0;
        GiftItemVO gift = giftService.getGift(giftId);
        if (gift != null && gift.getGiftPrice() != null) {
            rewardValue = gift.getGiftPrice();
        }
        String totalPrice = feedService.newestTotalReward(feedId, rewardValue);

        Result result = sendGiftProvider.sendGift2Result(form, sendGift, () -> null, getToken().getUserId(), getHeader().getChannelId());
        Object data = result.getData();
        if (data == null || !(data instanceof MutableResult)) {
            return result;
        }

        MutableResult mResult = (MutableResult) data;
        mResult.put("totalPrice", totalPrice);

        return mResult.build();
    }

    /**
     * (动态)打赏礼物记录列表
     *
     * @param feedId    动态id
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getRewardGiftHisList")
    @Override
    public Result getRewardGiftHisList(@QueryParam("feedId") Long feedId,
                                       @QueryParam("pageIndex") Integer pageIndex,
                                       @QueryParam("pageSize") Integer pageSize) {
        if (feedId == null || pageIndex == null || pageSize == null) {
            return Result.ofBadRequest("参数错误");
        }
        SendGiftRewardFeedSnapshotPageListQO qo = new SendGiftRewardFeedSnapshotPageListQO();
        qo.setFeedId(feedId);
        qo.setPageIndex(pageIndex);
        qo.setPageSize(pageSize);
        PageList<SendGiftRewardFeedSnapshotDO> pageList = sendGiftRewardFeedSnapshotService.getPageList(qo);

        PageList<GiftRewardHisVO> result = new PageList();
        BeanUtils.copyProperties(pageList, result);
        List<SendGiftRewardFeedSnapshotDO> list = pageList.getList();
        if (CollectionUtils.isEmpty(list)) {
            return Result.ofSuccess(result);
        }

        Set<Long> userIds = list.stream().filter(Objects::nonNull).map(po -> po.getSenderUserId()).collect(Collectors.toSet());
        Map<Long, UserBasicInfo> userMap = userService.getBasicInfoMap(userIds);
        List<Long> giftIds = list.stream().filter(Objects::nonNull).map(po -> po.getGiftId()).distinct().collect(Collectors.toList());
        Map<Long, GiftDO> giftMap = giftService.getGiftMapByGiftIds(giftIds);

        List<GiftRewardHisVO> voList = list.stream()
                .filter(Objects::nonNull)
                .map(po -> {
                    GiftRewardHisVO vo = new GiftRewardHisVO();
                    BeanUtils.copyProperties(po, vo);
                    Long userId = po.getSenderUserId();
                    vo.setUserId(userId);
                    vo.setUserName(po.getSenderUserName());
                    UserBasicInfo user = userMap.get(userId);
                    if (user != null) {
                        vo.setUserAvatar(user.getUserAvatar());
                    }
                    GiftDO gift = giftMap.get(po.getGiftId());
                    if (gift != null) {
                        vo.setGiftIcon(gift.getGiftIcon());
                    }
                    return vo;
                })
                .collect(Collectors.toList());
        result.setList(voList);

        return Result.ofSuccess(result);
    }
}
