package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.LiveFacade;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.media.StreamStatusEnum;
import com.starbuds.server.common.constant.user.LiveUserRole;
import com.starbuds.server.common.constant.user.NobleLevelEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.pojo.data.ShareParam;
import com.starbuds.server.common.pojo.db.AnchorDisableHisDO;
import com.starbuds.server.common.pojo.media.*;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.media.RankService;
import com.starbuds.server.common.service.user.*;
import com.starbuds.server.game.sdkdev.response.data.GetGamesData;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * 直播接口实现
 *
 * @author Lay
 * @date 2019/12/23
 */
@Path("/v1/live")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class LiveFacadeImpl extends BaseFacade implements LiveFacade {

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    RankService rankService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Value("${share.url}")
    private String shareUrl;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    /**
     * 开始直播
     *
     * @param liveTitle
     * @param liveCover
     * @param categoryId
     * @param areaId
     * @param productIds
     * @param gameId     游戏id
     * @return
     */
    @POST
    @Path("startLive")
    @Override
    public Result startLive(@FormParam("liveTitle") String liveTitle,
                            @FormParam("liveCover") String liveCover,
                            @FormParam("categoryId") Long categoryId,
                            @FormParam("areaId") int areaId,
                            @FormParam("productIds") String productIds,
                            @FormParam("gameId") String gameId) {
        Long userId = this.getToken().getUserId();

        // 判断用户是否主播
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "您未开通直播权限");
        }

        // 判断是否禁播
        Long enableTime = Optional.ofNullable(anchorService.getEnableTime(userId)).orElse(0L);
        if (enableTime != 0 && (System.currentTimeMillis() < enableTime || enableTime == -1L)) {
            MutableResult mResult = new MutableResult();
            mResult.put("enableTime", enableTime);
            return mResult.build();
        }

        // 判断是否直播中
        LiveStatusEnum liveStatus = anchorService.getLiveStatus(userId);
        if (LiveStatusEnum.Living == liveStatus) {
            return new Result(GlobalApiError.OperationFailed, "直播尚未结束，无法重复开播");
        }

        // 判断直播封面是否设置
        liveCover = anchorService.getLiveCover(userId);
        if (StringUtils.isBlank(liveCover)) {
            return new Result(GlobalApiError.OperationFailed, "请先设置一个直播封面哦");
        }

        // 判断区域是否存在
        if (areaId > 0 && !areaService.isAreaExist(areaId)) {
            return new Result(GlobalApiError.OperationFailed, String.format("无效的城市[%d]", areaId));
        }

        // 电商直播
        if (StringUtils.isNotBlank(productIds)) {
            // 判断类目是否允许电商直播
            if (!liveService.isCategorySellEnabled(categoryId)) {
                return new Result(GlobalApiError.OperationFailed, "该类目不允许商品直播");
            }

            // 判断主播是否允许电商直播
            if (!anchorService.isSellEnabled(userId)) {
                return new Result(GlobalApiError.OperationFailed, "您没有商品直播权限");
            }
        }

        // 游戏
        if (StringUtils.isNotBlank(gameId)) {
            log.info("设置直播游戏 userId={} gameId={}", userId, gameId);
            redisService.hashPut(RedisLiveKeys.LIVE_GAME, String.valueOf(userId), gameId);
        }

        // 开播
        LiveStartVO liveStartVO = liveService.startLive(userId, liveTitle, liveCover, categoryId, areaId > 0 ? areaId : null, Convert.toDouble(this.getHeader().getLongitude()), Convert.toDouble(this.getHeader().getLatitude()), this.getHeader().getChannelId(), productIds);
        if (liveStartVO != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("pushUrl", liveStartVO.getPushUrl());
            mResult.put("liveHisId", liveStartVO.getLiveHisId());
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed, "开播失败");
    }

    /**
     * 下播接口
     *
     * @param liveHisId
     * @return
     */
    @POST
    @Path("stopLive")
    @Override
    public Result stopLive(@FormParam("liveHisId") Long liveHisId) {
        Long userId = this.getToken().getUserId();

        // 判断直播记录是否存在
        if (liveHisId == null || !liveService.isLiveHisExists(liveHisId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到您的直播记录");
        }

        // 判断是否主播
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "您未开通直播权限");
        }

        // 判断是否开播
        LiveStatusEnum liveStatus = anchorService.getLiveStatus(userId);
        if (LiveStatusEnum.Closed == liveStatus) {
            return new Result();
        }

        // 下播
        LiveDataVO liveDataVO = liveService.stopLive(liveHisId, false);
        if (liveDataVO != null) {
            return new Result(liveDataVO);
        }

        return new Result(GlobalApiError.OperationFailed, "下播失败");
    }

    /**
     * 继续直播
     *
     * @param liveHisId
     * @return
     */
    @POST
    @Path("resumeLive")
    @Override
    public Result resumeLive(@FormParam("liveHisId") Long liveHisId) {
        Long userId = this.getToken().getUserId();

        // 判断直播记录是否存在
        if (liveHisId == null || !liveService.isLiveHisExists(liveHisId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到您的直播记录");
        }

        MutableResult mResult = new MutableResult();

        // 判断禁播
        if (anchorService.isDisable(userId)) {
            AnchorDisableHisDO anchorDisableHis = anchorService.getLatestDisableHis(userId);
            if (anchorDisableHis != null && System.currentTimeMillis() < anchorDisableHis.getEnableTime()) {
                mResult.put("disabled", BoolIntVal.True);
                mResult.put("enableTime", anchorDisableHis.getEnableTime());
                mResult.put("disableRemarks", anchorDisableHis.getRemark());
            }
        } else {
            mResult.put("disabled", BoolIntVal.False);
        }

        // 判断是否开播
        LiveStatusEnum liveStatus = anchorService.getLiveStatus(userId);
        mResult.put("liveStatus", liveStatus.getValue());
        if (LiveStatusEnum.Living == liveStatus) {
            mResult.put("pushUrl", liveService.getStreamPushUrl(userId, liveHisId));
        }

        return mResult.build();
    }

    /**
     * 获取置顶推荐直播
     *
     * @return
     */
    @GET
    @Path("getTopSuggest")
    @PermitGuest
    @Override
    public Result getTopSuggest() {
        Long userId = Optional.ofNullable(this.getToken()).map(TokenBO::getUserId).orElse(0L);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveService.getTopSuggestList(TopSuggestDTO.builder().userId(userId).channelId(this.getHeader().getChannelId()).build()));
        return mResult.build();
    }

    /**
     * 获取热门列表
     *
     * @param lastId
     * @return
     */
    @GET
    @Path("getHotList")
    @Override
    public Result getHotList(@QueryParam("lastId") int lastId) {
        if (lastId < 0) {
            lastId = 0;
        }
        return new Result(liveService.getHeatList(lastId, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取类目直播列表
     *
     * @param categoryId
     * @return
     */
    @GET
    @Path("getListByCategory")
    @Override
    public Result getListByCategory(@QueryParam("categoryId") Long categoryId,
                                    @QueryParam("pageIndex") int pageIndex) {
        // 判断类目是否存在
        if (!channelService.isLiveCategoryExists(this.getHeader().getChannelId(), categoryId)) {
            return new Result(GlobalApiError.OperationFailed, "类目无效");
        }

        // 获取列表
        return new Result(liveService.getList(pageIndex, GlobalConfig.PAGE_SIZE, categoryId, LiveStatusEnum.Living));
    }

    /**
     * 获取关注直播列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getFollowList")
    @Override
    public Result getFollowList(@QueryParam("pageIndex") int pageIndex) {
        return new Result(liveService.getFollowList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId(), LiveStatusEnum.Living));
    }

    /**
     * 获取附近直播列表
     *
     * @param pageIndex
     * @param areaId
     * @return
     */
    @GET
    @Path("getNearByList")
    @Override
    public Result getNearByList(@QueryParam("pageIndex") int pageIndex,
                                @QueryParam("areaId") Integer areaId) {
        Double latitude = Convert.toDouble(this.getHeader().getLatitude());
        Double longitude = Convert.toDouble(this.getHeader().getLongitude());
        Double distance = 1000000d;

        if (latitude == null || longitude == null) {
            IpLocation ipLocation = areaService.getIpLocation(this.getIpAddress());
            if (ipLocation != null) {
                latitude = Optional.ofNullable(ipLocation.getCenterLatitude()).orElse(ipLocation.getLbLatitude());
                longitude = Optional.ofNullable(ipLocation.getCenterLongitude()).orElse(ipLocation.getLbLongitude());
            }
            if (latitude == null && longitude == null) {
                return new Result(GlobalApiError.OperationFailed, "未能获取您的位置，请先开启定位后重试！");
            }
        }
        return new Result(liveService.getNearbyList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId(), LiveStatusEnum.Living, areaId, latitude, longitude, distance, null));
    }

    /**
     * 获取直播详情
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getLiveInfo")
    @Override
    public Result getLiveInfo(@QueryParam("userId") Long userId) {
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户未开通直播权限");
        }
        Long myUserId = this.getToken().getUserId();

        // 小黑屋
        if (anchorService.isBlack(userId, myUserId)) {
            return new Result(GlobalApiError.OperationFailed, "您已被主播关进小黑屋");
        }

        // 主播信息
        UserCardVO userCardVO = userService.getCard(userId);
        if (userCardVO == null) {
            return new Result(GlobalApiError.OperationFailed, "主播不存在");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("anchorCard", userCardVO);

        // 主播详情
        AnchorInfoVO anchorInfoVO = anchorService.getInfo(userId);
        mResult.put("anchorInfo", anchorInfoVO);

        // 是否关注主播
        boolean isFollow = socialRelationService.isFollower(myUserId, userId);
        mResult.put("isFollow", isFollow);

        // 是否粉丝团
        boolean isFans = anchorService.isFans(userId, myUserId);
        mResult.put("isFans", isFans);

        // 直播间角色
        int liveRole = LiveUserRole.AUDIENCE;
        boolean hasForbidden = false;
        boolean hasBlack = false;
        if (userId.equals(myUserId)) {
            liveRole = LiveUserRole.ANCHOR;
            hasForbidden = true;
            hasBlack = true;
        } else if (userService.isSuper(myUserId)) {
            liveRole = LiveUserRole.SUPER;
            hasForbidden = true;
            hasBlack = true;
        } else if (anchorService.isMaster(userId, myUserId)) {
            liveRole = LiveUserRole.MASTER;
            hasForbidden = true;
        }

        // 贵族特权
        UserNobleVO userNoble = nobleService.getUserNoble(myUserId);
        int nobleLevel = Optional.ofNullable(userNoble).map(UserNobleVO::getNobleLevel).orElse(0);
        if (nobleLevel == NobleLevelEnum.DIHUANG.getCode()) {
            hasBlack = true;
        }
        if (nobleLevel > NobleLevelEnum.HOUJUE.getCode()) {
            hasForbidden = true;
        }

        mResult.put("liveRole", liveRole);
        mResult.put("hasForbidden", hasForbidden);
        mResult.put("hasBlack", hasBlack);

        // 拉流地址
        if (StreamStatusEnum.Pushing.getValue() == anchorInfoVO.getStreamStatus()) {
            mResult.put("pullUrl", liveService.getStreamPullUrl(userId));
        } else {
            mResult.put("pullUrl", null);
        }

        // 分享参数
        ShareParam shareParam = new ShareParam();
        shareParam.setShareUrl(String.format("%s?a=%d&c=%d&t=%s", shareUrl, userId, this.getHeader().getChannelId(), anchorInfoVO.getLiveTitle()));
        shareParam.setShareTitle(anchorInfoVO.getLiveTitle());
        shareParam.setShareSubTitle(String.format("%s(ID:%d)正在直播%s，快来围观！", userCardVO.getUserName(), userCardVO.getNiceNo() == null ? userCardVO.getUserNo() : userCardVO.getNiceNo(), Optional.ofNullable(anchorInfoVO.getCategoryName()).orElse("")));
        shareParam.setSharePic(anchorInfoVO.getLiveCover());
        mResult.put("shareParam", shareParam);

        // 当前直播热度
        mResult.put("liveHeat", liveService.getLiveHeat(userId));
        // 当前主播魅力值
        mResult.put("charmValue", scoreService.getScoreValue(userId, ScoreTypeEnum.CHARM));

        // 直播展示商品
        mResult.put("liveSell", liveService.getLiveSell(userId));

        // 获取直播游戏
        mResult.put("liveGame", (String) redisService.hashGet(RedisLiveKeys.LIVE_GAME, String.valueOf(userId)));
        return mResult.build();
    }

    /**
     * 获取直播间详情（游客）
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getLiveInfoLite")
    @PermitGuest
    @Override
    public Result getLiveInfoLite(@QueryParam("userId") Long userId) {
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户未开通直播权限");
        }

        // 主播信息
        UserCardVO userCardVO = userService.getCard(userId);
        if (userCardVO == null) {
            return new Result(GlobalApiError.OperationFailed, "主播不存在");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("anchorCard", userCardVO);

        // 主播详情
        AnchorInfoVO anchorInfoVO = anchorService.getInfo(userId);
        mResult.put("anchorInfo", anchorInfoVO);

        // 拉流地址
        if (StreamStatusEnum.Pushing.getValue() == anchorInfoVO.getStreamStatus()) {
            mResult.put("pullUrl", liveService.getStreamPullUrl(userId));
        } else {
            mResult.put("pullUrl", null);
        }

        // 连接地址
        mResult.put("liveMsgUrl", String.format("%s?token=GUEST&rid=%d&cid=%d", channelService.getLiveMsgServer(this.getHeader().getChannelId()), userId, this.getHeader().getChannelId()));

        // 当前直播热度
        mResult.put("liveHeat", liveService.getLiveHeat(userId));

        return mResult.build();
    }

    /**
     * 获取日榜数据
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getRankDay")
    @Override
    public Result getRankDay(@QueryParam("userId") Long userId) {
        // 拉取前 100
        List<LiveRankItem> liveRankItems = rankService.getLiveRankDay(userId, 0, 99);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        Long myUserId = this.getToken().getUserId();
        int myRankNo = liveRankItems.stream().filter(t -> t.getUserId().equals(myUserId)).findFirst().map(LiveRankItem::getRankNo).orElse(-1);
        LiveRankItem liveRankItem = rankService.getDayRankItem(userId, myUserId);
        liveRankItem.setRankNo(myRankNo);
        mResult.put("myRankItem", liveRankItem);
        return mResult.build();
    }

    /**
     * 获取周榜数据
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getRankWeek")
    @Override
    public Result getRankWeek(@QueryParam("userId") Long userId) {
        // 拉取前 100
        List<LiveRankItem> liveRankItems = rankService.getLiveRankWeek(userId, 0, 99);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        Long myUserId = this.getToken().getUserId();
        int myRankNo = liveRankItems.stream().filter(t -> t.getUserId().equals(myUserId)).findFirst().map(LiveRankItem::getRankNo).orElse(-1);
        LiveRankItem liveRankItem = rankService.getWeekRankItem(userId, myUserId);
        liveRankItem.setRankNo(myRankNo);
        mResult.put("myRankItem", liveRankItem);
        return mResult.build();
    }

    /**
     * 获取月榜数据
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getRankMonth")
    @Override
    public Result getRankMonth(@QueryParam("userId") Long userId) {
        // 拉取前 100
        List<LiveRankItem> liveRankItems = rankService.getLiveRankMonth(userId, 0, 99);

        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        Long myUserId = this.getToken().getUserId();
        int myRankNo = liveRankItems.stream().filter(t -> t.getUserId().equals(myUserId)).findFirst().map(LiveRankItem::getRankNo).orElse(-1);
        LiveRankItem liveRankItem = rankService.getMonthRankItem(userId, myUserId);
        liveRankItem.setRankNo(myRankNo);
        mResult.put("myRankItem", liveRankItem);
        return mResult.build();
    }

    /**
     * 获取总榜数据
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getRankTotal")
    @Override
    public Result getRankTotal(@QueryParam("userId") Long userId) {
        // 拉取前 100
        List<LiveRankVO> liveRankItems = rankService.getLiveRankTotal(userId, 0, 99);

        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        return mResult.build();
    }

    /**
     * 获取贵族
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getNobles")
    @Override
    public Result getNobles(@QueryParam("userId") Long userId) {
        // 获取所有贵族
        List<LiveUserProfile> liveUserProfiles = userService.getLiveNobleList(userId, 0, -1);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveUserProfiles);
        return mResult.build();
    }

    /**
     * 获取粉丝团
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getFans")
    @Override
    public Result getFans(@QueryParam("userId") Long userId) {
        // 获取前50名粉丝
        List<LiveUserProfile> liveUserProfiles = userService.getLiveFansList(userId, 0, 49);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveUserProfiles);
        return mResult.build();
    }

    /**
     * 获取直播记录
     *
     * @param pageIndex
     * @param yearMonth
     * @return
     */
    @GET
    @Path("getLiveHisList")
    @Override
    public Result getLiveHisList(@QueryParam("pageIndex") int pageIndex,
                                 @QueryParam("yearMonth") Integer yearMonth) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        Long userId = this.getToken().getUserId();
        return new Result(liveService.getLiveHisList(pageIndex, GlobalConfig.PAGE_SIZE, userId, yearMonth));
    }

    /**
     * 分享直播间
     *
     * @param userId
     * @return
     */
    @POST
    @Path("shareLive")
    @Override
    public Result shareLive(@FormParam("userId") Long userId) {
        liveService.shareLive(userId, this.getToken().getUserId(), this.getHeader().getChannelId());
        return new Result();
    }

    /**
     * 获取在线用户
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getUsers")
    @Override
    public Result getUsers(@QueryParam("userId") Long userId) {
        // 获取前 50 个在线用户
        List<LiveUserProfile> liveUserProfiles = userService.getLiveUserList(userId, 0, 49);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveUserProfiles);
        return mResult.build();
    }

    /**
     * 设置展示商品
     *
     * @param productId
     * @return
     */
    @POST
    @Path("setShowProduct")
    @Override
    public Result setShowProduct(@FormParam("productId") Long productId) {
        Long userId = this.getToken().getUserId();

        productId = Optional.ofNullable(productId).orElse(0L);
        if (productId < 0) {
            productId = 0L;
        }

        // 获取当前展示商品信息
        LiveSellVO liveSell = liveService.getLiveSell(userId);
        if (productId.equals(liveSell.getShowProductId())) {
            return new Result();
        }

        // 判断商品 id 有效性
        String productIds = Optional.ofNullable(liveSell.getProductIds()).orElse("");
        List<String> productIdList = Arrays.asList(productIds.split(","));
        if (CollectionUtil.isEmpty(productIdList)) {
            return new Result(GlobalApiError.OperationFailed, "没有可以展示的商品");
        }

        // 判断直播商品是否包含该 id
        if (productId != 0 && !productIdList.contains(String.valueOf(productId))) {
            return new Result(GlobalApiError.OperationFailed, "直播商品中不包含此商品");
        }

        if (liveService.setShowProductId(userId, productId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

}
