package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.v1.facade.MusicFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.MusicStatusEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.RecordList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.MusicVO;
import com.starbuds.server.common.pojo.db.MusicDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.MusicService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 音乐接口实现
 *
 * @author jiangxiaowei
 * @date 2020/8/3 15:30
 */
@Path("/v1/music")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class MusicFacadeImpl extends BaseFacade implements MusicFacade {

    @Reference(version = "1.0.0")
    MusicService musicService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    /**
     * 添加音乐
     *
     * @param musicName
     * @param musicSinger
     * @param musicUrl
     * @param original
     * @return
     */
    @POST
    @Path("addMusic")
    @Override
    public Result addMusic(@FormParam("musicName") String musicName,
                           @FormParam("musicSinger") String musicSinger,
                           @FormParam("musicUrl") String musicUrl,
                           @FormParam("original") Integer original) {

        long curTime = System.currentTimeMillis();
        Long userId = this.getToken().getUserId();

        MusicDO musicDO = new MusicDO();
        musicDO.setMusicId(identityService.getId());
        musicDO.setMusicName(musicName);
        musicDO.setMusicSinger(musicSinger);
        musicDO.setMusicUrl(musicUrl);
        musicDO.setOriginal(original);
        musicDO.setCreateTime(curTime);
        musicDO.setUpdateTime(curTime);
        musicDO.setUserId(userId);
        musicDO.setShelfStatus(BoolIntVal.True);
        musicDO.setStatus(MusicStatusEnum.USER_UPLOAD.getValue());
        musicDO.setDownloadCount(0);

        if (musicService.addMusic(musicDO)) {
            return new Result();
        }

        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 删除音乐
     *
     * @param musicIds
     * @return
     */
    @POST
    @Path("delMusic")
    @Override
    public Result delMusic(@FormParam("musicIds") String musicIds) {
        if (StringUtils.isEmpty(musicIds)) {
            return new Result(GlobalApiError.ServerError, "音乐id不能为空");
        }
        List<Long> musicIdList = Arrays.stream(musicIds.split(",")).map(Convert::toLong).collect(Collectors.toList());
        List<Long> userIds = musicService.getUserIds(musicIdList);
        if (CollectionUtil.isEmpty(userIds) || userIds.size() != 1) {
            return new Result(GlobalApiError.ServerError, "没有权限");
        }
        if (musicService.delMusic(musicIdList)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取上传列表
     *
     * @param lastId
     * @param sceneType
     * @return
     */
    @GET
    @Path("getUploadList")
    @Override
    public Result getUploadList(@QueryParam("lastId") Long lastId, @QueryParam("sceneType") Integer sceneType) {
        Long userId = this.getToken().getUserId();
        if (sceneType == null) {
            sceneType = BoolIntVal.False;
        }
        RecordList<MusicVO, Long> recordList = musicService.getUploadList(userId, lastId, sceneType, GlobalConfig.PAGE_SIZE);
        MutableResult mResult = new MutableResult();
        mResult.put("record", recordList);
        mResult.put("uploadCount", musicService.getUploadCount(userId));
        return mResult.build();
    }

    /**
     * 获取下载列表
     *
     * @param lastId
     * @return
     */
    @GET
    @Path("getDownloadList")
    @Override
    public Result getDownloadList(@QueryParam("lastId") Long lastId) {
        Long userId = this.getToken().getUserId();
        return new Result(musicService.getDownloadList(userId, lastId, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 下载音乐
     *
     * @param musicId
     * @return
     */
    @POST
    @Path("downloadMusic")
    @Override
    public Result downloadMusic(@FormParam("musicId") Long musicId) {
        if (!musicService.isMusicDownloadable(musicId)) {
            return new Result(GlobalApiError.ServerError, "音乐暂不可下载，试试其他的吧");
        }
        Long userId = this.getToken().getUserId();

        musicService.downloadMusic(musicId, userId);

        MutableResult mResult = new MutableResult();
        mResult.put("musicUrl", musicService.getMusicUrl(musicId));

        return mResult.build();
    }
}
