package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.NobleFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.user.NobleLevelEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.NoblePriceConfigDO;
import com.starbuds.server.common.pojo.user.NobleUpgradeAmount;
import com.starbuds.server.common.pojo.user.UserNobleVO;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.user.NobleService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;

/**
 * 贵族接口实现
 *
 * @author Lay
 * @date 2020/3/4
 */
@Path("/v1/noble")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class NobleFacadeImpl extends BaseFacade implements NobleFacade {

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    private String BUY_NOBLE_ERR_MSG = "余额不足";

    /**
     * 获取贵族价格
     *
     * @return
     */
    @GET
    @Path("getNoblePrice")
    @PermitGuest
    @Override
    public Result getNoblePrice() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", nobleService.getPriceConfigList());
        return mResult.build();
    }

    /**
     * 新开贵族
     *
     * @param nobleLevel
     * @return
     */
    @POST
    @Path("newNoble")
    @Override
    public Result newNoble(@FormParam("nobleLevel") int nobleLevel) {
        Long userId = this.getToken().getUserId();

        // 获取用户当前的贵族
        UserNobleVO userNoble = nobleService.getUserNoble(userId);

        // 判断是否已开通贵族
        if (userNoble != null) {
            return new Result(GlobalApiError.OperationFailed, String.format("您已经是%s贵族了哦", NobleLevelEnum.toEnum(userNoble.getNobleLevel()).getDesc()));
        }

        NoblePriceConfigDO noblePriceConfig = nobleService.getPriceConfigByLevel(nobleLevel);
        if (noblePriceConfig == null) {
            return new Result(GlobalApiError.OperationFailed, "该贵族等级暂未开放");
        }

        // 判断钱够不够
        MutableResult mResult = new MutableResult();
        WalletTypeEnum priceType = WalletTypeEnum.toEnum(noblePriceConfig.getNoblePriceType());
        Long balance = walletService.getBalance(userId, priceType);
        if (balance < noblePriceConfig.getNoblePrice()) {
            mResult.put("balance", balance);
            Result result = mResult.build();
            result.setCode(GlobalApiError.BadRequest.getCode());
            result.setMsg(BUY_NOBLE_ERR_MSG);
            return result;
        }

        balance = nobleService.newNoble(userId, nobleLevel, this.getHeader().getChannelId());
        if (balance != null) {
            mResult.put("balance", balance);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 续费贵族
     *
     * @return
     */
    @POST
    @Path("renewNoble")
    @Override
    public Result renewNoble(@FormParam("times") int times) {
        Long userId = this.getToken().getUserId();
        if (times < 1) {
            times = 1;
        }

        // 获取用户当前的贵族
        UserNobleVO userNoble = nobleService.getUserNoble(userId);
        if (userNoble == null) {
            return new Result(GlobalApiError.OperationFailed, "您还未开通贵族哦，赶快加入吧");
        }

        NoblePriceConfigDO noblePriceConfig = nobleService.getPriceConfigByLevel(userNoble.getNobleLevel());
        if (noblePriceConfig == null) {
            return new Result(GlobalApiError.OperationFailed, "该贵族等级暂未开放");
        }

        // 判断钱够不够
        MutableResult mResult = new MutableResult();
        WalletTypeEnum priceType = WalletTypeEnum.toEnum(noblePriceConfig.getNoblePriceType());
        Long balance = walletService.getBalance(userId, priceType);
        if (balance < noblePriceConfig.getRenewalPrice() * times) {
            mResult.put("balance", balance);
            Result result = mResult.build();
            result.setCode(GlobalApiError.BadRequest.getCode());
            result.setMsg(BUY_NOBLE_ERR_MSG);
            return result;
        }

        balance = nobleService.renewNoble(userId, false, times, this.getHeader().getChannelId());
        if (balance != null) {
            mResult.put("balance", balance);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 升级贵族
     *
     * @param nobleLevel
     * @return
     */
    @POST
    @Path("upgradeNoble")
    @Override
    public Result upgradeNoble(@FormParam("nobleLevel") int nobleLevel) {
        Long userId = this.getToken().getUserId();

        // 获取用户当前的贵族
        UserNobleVO userNoble = nobleService.getUserNoble(userId);
        if (userNoble == null) {
            return new Result(GlobalApiError.OperationFailed, "您还未开通贵族哦，赶快加入吧");
        }

        NoblePriceConfigDO noblePriceConfig = nobleService.getPriceConfigByLevel(nobleLevel);
        if (noblePriceConfig == null) {
            return new Result(GlobalApiError.OperationFailed, "该贵族等级暂未开放");
        }

        // 判断新等级是否高于当前等级
        if (nobleLevel <= userNoble.getNobleLevel()) {
            return new Result(GlobalApiError.OperationFailed, "当前等级无需升级哦");
        }

        Long balance = nobleService.upgradeNoble(userId, nobleLevel, this.getHeader().getChannelId());
        if (balance != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("balance", balance);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取升级差价
     *
     * @param nobleLevel
     * @return
     */
    @GET
    @Path("getUpgradeAmount")
    @Override
    public Result getUpgradeAmount(@QueryParam("nobleLevel") int nobleLevel) {
        Long userId = this.getToken().getUserId();
        NobleUpgradeAmount nobleUpgradeAmount = nobleService.getUpgradeAmount(userId, nobleLevel);
        if (nobleUpgradeAmount == null) {
            nobleUpgradeAmount = new NobleUpgradeAmount();
            nobleUpgradeAmount.setDateDiff(0L);
            nobleUpgradeAmount.setAmount(0L);
        }
        return new Result(nobleUpgradeAmount);
    }

    /**
     * 设置自动续费
     *
     * @param autoRenewal
     * @return
     */
    @POST
    @Path("setAutoRenewal")
    @Override
    public Result setAutoRenewal(@FormParam("autoRenewal") int autoRenewal) {
        Long userId = this.getToken().getUserId();
        // 获取用户当前的贵族
        UserNobleVO userNoble = nobleService.getUserNoble(userId);
        if (userNoble == null) {
            return new Result(GlobalApiError.OperationFailed, "您还未开通贵族哦，赶快加入吧");
        }

        if (nobleService.setAutoRenewal(userId, autoRenewal == 1 ? BoolIntVal.True : BoolIntVal.False)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

}
