package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.manage.ManageService;
import com.starbuds.server.api.app.pojo.KeyValueVO;
import com.starbuds.server.api.app.pojo.official.form.ForbiddenForm;
import com.starbuds.server.api.app.v1.facade.OfficialFacade;
import com.starbuds.server.common.constant.common.KeyNameTypeVO;
import com.starbuds.server.common.constant.user.ForbiddenDurationTypeEnum;
import com.starbuds.server.common.constant.user.ForbiddenWayTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.ws.rs.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 官方管理员操作
 *
 * @author aoe
 * @date 2022/1/7
 */
@Path("/v1/official")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
@Validated
public class OfficialFacadeImpl extends BaseFacade implements OfficialFacade {

    @Resource
    private ManageService manageService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private ImService imService;

    private final String SUCCESS_CODE = "OK";

    /**
     * 封禁方式列表
     *
     * @return
     */
    @GET
    @Path("getForbiddenWayTypes")
    @Override
    public Result getForbiddenWayTypes() {
        List<KeyValueVO> wayTypes = new ArrayList<>();
        for (ForbiddenWayTypeEnum wayType : ForbiddenWayTypeEnum.values()) {
            KeyValueVO kv = new KeyValueVO();
            kv.setKey(wayType.getCode());
            kv.setValue(wayType.getDesc());
            wayTypes.add(kv);
        }
        return Result.ofList(wayTypes);
    }

    /**
     * 封禁时长列表
     *
     * @return
     */
    @GET
    @Path("getForbiddenDurationTypes")
    @Override
    public Result getForbiddenDurationTypes() {
        List<KeyValueVO> durationTypes = new ArrayList<>();
        for (ForbiddenDurationTypeEnum wayType : ForbiddenDurationTypeEnum.values()) {
            KeyValueVO kv = new KeyValueVO();
            kv.setKey(wayType.getCode());
            kv.setValue(wayType.getDesc());
            durationTypes.add(kv);
        }
        return Result.ofList(durationTypes);
    }

    /**
     * 封禁(禁言)
     *
     * @param form
     * @return
     */
    @POST
    @Path("forbidden")
    @Override
    public Result forbidden(@BeanParam @Valid ForbiddenForm form) throws IOException {
        return manageService.forbidden(form, getPhone());
    }

    /**
     * 重置昵称
     *
     * @param userId
     * @return
     * @throws IOException
     */
    @POST
    @Path("resetUserName")
    @Override
    public Result resetUserName(@FormParam("userId") Long userId) throws IOException {
        if (userId == null) {
            return Result.ofBadRequest("请选择用户");
        }
        Result result = manageService.resetUserName(userId, getPhone());
        if (SUCCESS_CODE.equals(result.getCode())) {
            imService.sendNoActionSystemMsg(userId, "内容涉嫌违规",
                    "因不符合用户资料使用规范，官方人员已将您的昵称更改为默认昵称，您还可以自行修改~");
        }
        return result;
    }

    /**
     * 重置用户签名
     *
     * @param userId
     * @return
     * @throws IOException
     */
    @POST
    @Path("resetUserSign")
    @Override
    public Result resetUserSign(@FormParam("userId") Long userId) throws IOException {
        if (userId == null) {
            return Result.ofBadRequest("请选择用户");
        }
        Result result = manageService.resetUserSign(userId, getPhone());
        if (SUCCESS_CODE.equals(result.getCode())) {
            imService.sendNoActionSystemMsg(userId, "内容涉嫌违规",
                    "因不符合用户资料使用规范，官方人员已将您的个性签名更改为默认签名，您还可以自行修改~");
        }
        return result;
    }

    /**
     * 重置用户头像
     *
     * @param userId
     * @return
     * @throws IOException
     */
    @POST
    @Path("resetUserAvatar")
    @Override
    public Result resetUserAvatar(@FormParam("userId") Long userId) throws IOException {
        if (userId == null) {
            return Result.ofBadRequest("请选择用户");
        }
        Result result = manageService.resetUserAvatar(userId, getPhone());
        if (SUCCESS_CODE.equals(result.getCode())) {
            imService.sendNoActionSystemMsg(userId, "内容涉嫌违规",
                    "因不符合用户资料使用规范，官方人员已将您的头像更改为默认头像，您还可以自行修改~");
        }
        return result;
    }

    /**
     * 删除照片墙
     *
     * @param feedPhotoWallId
     * @param userId
     * @return
     */
    @POST
    @Path("deletePhotoWall")
    @Override
    public Result deletePhotoWall(@FormParam("feedPhotoWallId") Long feedPhotoWallId,
                                  @FormParam("userId") Long userId) throws IOException {
        if (feedPhotoWallId == null) {
            return Result.ofBadRequest("请选择要删除的照片");
        }
        if (userId == null) {
            return Result.ofBadRequest("请选择用户");
        }
        return manageService.deletePhotoWall(feedPhotoWallId, userId, getPhone());
    }

    /**
     * 删除动态
     *
     * @param feedId
     * @return
     */
    @POST
    @Path("deleteFeed")
    @Override
    public Result deleteFeed(@FormParam("feedId") Long feedId) throws IOException {
        if (feedId == null) {
            return Result.ofBadRequest("请选择要删除的动态");
        }
        return manageService.deleteFeed(feedId, "内容违规", getPhone());
    }

    private String getPhone() {
        Long userId = this.getUserId();
        if (userId == null) {
            return null;
        }
        return userService.getPhone(userId);
    }


}
