package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.PayFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.PayWayEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.AppVersionDO;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.utils.SafeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.Optional;

/**
 * 支付接口实现
 *
 * @author Lay
 * @date 2019/12/18
 */
@Path("/v1/pay")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class PayFacadeImpl extends BaseFacade implements PayFacade {

    @Reference(version = "1.0.0")
    PayService payService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    DataService dataService;

    /**
     * 创建支付订单
     *
     * @param orderId
     * @return
     */
    @POST
    @Path("addPayOrder")
    @Override
    public Result addPayOrder(@FormParam("orderId") Long orderId) {
        // 查询订单是否存在
        if (!payService.isPayOrderExists(orderId)) {
            return new Result(GlobalApiError.OperationFailed, "订单不存在");
        }

        // 判断是否已创建支付订单
        String payParams = payService.getPayParams(orderId);

        if (StringUtils.isBlank(payParams)) {
            // 创建支付订单
            payParams = payService.addPayOrder(orderId);
            if (StringUtils.isBlank(payParams)) {
                return new Result(GlobalApiError.OperationFailed, "支付订单创建失败");
            }
        }

        MutableResult mResult = new MutableResult();
        mResult.put("orderId", orderId);
        mResult.put("payParams", payParams);
        return mResult.build();
    }

    /**
     * 创建支付订单
     *
     * @param orderId
     * @param openId
     * @return
     */
    @POST
    @Path("addPayOrderGuest")
    @PermitGuest
    @Override
    public Result addPayOrderGuest(@FormParam("orderId") Long orderId,
                                   @FormParam("openId") String openId) {
        // 查询订单是否存在
        if (!payService.isPayOrderExists(orderId)) {
            return new Result(GlobalApiError.OperationFailed, "订单不存在");
        }

        // 判断是否已创建支付订单
        String payParams = payService.getPayParams(orderId);

        if (StringUtils.isBlank(payParams)) {
            // 创建支付订单
            payParams = payService.addPayOrder(orderId, openId);
            if (StringUtils.isBlank(payParams)) {
                return new Result(GlobalApiError.OperationFailed, "支付订单创建失败");
            }
        }

        MutableResult mResult = new MutableResult();
        mResult.put("orderId", orderId);
        mResult.put("payParams", payParams);
        return mResult.build();
    }

    /**
     * 创建h5支付订单
     *
     * @param orderId
     * @param openId
     * @return
     */
    @POST
    @Path("addH5PayOrderGuest")
    @PermitGuest
    @Override
    public Result addH5PayOrderGuest(@FormParam("orderId") Long orderId, @FormParam("openId") String openId, @FormParam("userId") String userId, @FormParam("sign") String sign) {
        if (StringUtils.isEmpty(userId) || StringUtils.isEmpty(sign) || !sign.equalsIgnoreCase(SafeUtil.getSign(userId))) {
            return new Result(GlobalApiError.OperationFailed, "登录失效");
        }


        // 查询订单是否存在
        if (!payService.isPayOrderExists(orderId)) {
            return new Result(GlobalApiError.OperationFailed, "订单不存在");
        }

        // 判断是否已创建支付订单
        String payParams = payService.getPayParams(orderId);

        if (StringUtils.isBlank(payParams)) {
            // 创建支付订单
            payParams = payService.addPayOrder(orderId, openId);
            if (StringUtils.isBlank(payParams)) {
                return new Result(GlobalApiError.OperationFailed, "支付订单创建失败");
            }
        }

        MutableResult mResult = new MutableResult();
        mResult.put("orderId", orderId);
        mResult.put("payParams", payParams);
        return mResult.build();
    }

    /**
     * 创建 ApplePay 支付订单
     *
     * @param orderId
     * @param transactionId
     * @param receiptData
     * @return
     */
    @POST
    @Path("addApplePayOrder")
    @Override
    public Result addApplePayOrder(@FormParam("orderId") Long orderId, @FormParam("transactionId") String transactionId, @FormParam("receiptData") String receiptData) {
        // 查询订单是否存在
        if (!payService.isPayOrderExists(orderId)) {
            return new Result(GlobalApiError.OperationFailed, "订单不存在");
        }

        // 判断订单类型是否正确
        Integer payWay = payService.getPayWay(orderId);
        if (PayWayEnum.APPLE.getValue() != payWay) {
            return new Result(GlobalApiError.OperationFailed, "订单支付类型不匹配");
        }

        // 判断交易是否已处理
        if (StringUtils.isNotBlank(payService.getAppleTransactionId(orderId))) {
            return new Result(GlobalApiError.OperationFailed, "该订单已交易完成，请勿重复提交");
        }

        AppVersionDO lastVersion = dataService.getLatestAppVersion(this.getHeader().getClientType(), this.getHeader().getAppType(), 0, this.getHeader().getChannelId());
        boolean inApprove = false;

        if (lastVersion != null && Optional.ofNullable(lastVersion.getMarketStatus()).orElse(0) == 1) {
            // 市场包审核时走沙箱
            String appVersion = this.getHeader().getAppVersion();
            if (appVersion != null && appVersion.equals(lastVersion.getVersionCode())) {
                inApprove = true;
            }
        }

        payService.updateApplePayInfo(orderId, transactionId, receiptData);
        // 处理
        if (payService.addApplePayOrder(orderId, transactionId, receiptData, inApprove)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "交易处理失败");
    }

    /**
     * 获取支付状态
     *
     * @param orderId
     * @return
     */
    @GET
    @Path("getPayStatus")
    @Override
    public Result getPayStatus(@QueryParam("orderId") Long orderId) {

        Integer payStatus = payService.getPayStatus(orderId);

        if (payStatus == null) {
            return new Result(GlobalApiError.OperationFailed, "订单不存在");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("orderId", orderId);
        mResult.put("payStatus", payStatus);
        return mResult.build();
    }

    /**
     * 加入主播粉丝团
     *
     * @param anchorUserId
     * @return
     */
    @POST
    @Path("forJoinFansTeam")
    @Override
    public Result forJoinFansTeam(@FormParam("anchorUserId") Long anchorUserId) {
        // 判断是否主播
        if (!anchorService.isAnchor(anchorUserId)) {
            return new Result(GlobalApiError.OperationFailed, "对方不是主播哦");
        }

        Long userId = this.getToken().getUserId();
        // 判断是否自己
        if (anchorUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "无法加入自己的粉丝团哦");
        }

        // 判断是否设置粉丝牌
        if (StringUtils.isBlank(anchorService.getFansName(anchorUserId))) {
            return new Result(GlobalApiError.OperationFailed, "该主播暂未开通粉丝团");
        }

        // 判断是否已是粉丝团成员
        if (anchorService.isFans(anchorUserId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "您已是该粉丝团成员");
        }

        // 付钱
        Long balance = anchorService.addFans(anchorUserId, userId);
        if (balance == null) {
            return new Result(GlobalApiError.OperationFailed, "加入失败");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", balance);
        return mResult.build();
    }
}
