package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.RechargeFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.RankScore;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.finance.CoinRecharge;
import com.starbuds.server.common.pojo.finance.CoinRechargeItem;
import com.starbuds.server.common.pojo.finance.CoinRechargeOrderAddDTO;
import com.starbuds.server.common.pojo.finance.CoinRechargeOrderVO;
import com.starbuds.server.common.service.finance.RechargeService;
import com.starbuds.server.common.service.i18n.I18nService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.SafeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 充值接口实现
 *
 * @author Lay
 * @date 2019/12/17
 */
@Path("/v1/recharge")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RechargeFacadeImpl extends BaseFacade implements RechargeFacade {

    @Reference(version = "1.0.0")
    RechargeService rechargeService;

    @Reference(version = "1.0.0")
    private I18nService i18nService;

    @Reference(version = "1.0.0")
    UserService userService;

    /**
     * 获取金币充值项目
     *
     * @return
     */
    @GET
    @Path("getCoinRechargeItems")
    @Override
    public Result getCoinRechargeItems() {
        List<CoinRecharge> list = rechargeService.getCoinRechargeByChannelId(this.getHeader().getChannelId());
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(list, lang()));
        return mResult.build();
    }

    /**
     * 获取金币充值项目
     *
     * @return
     */
    @GET
    @Path("getCoinRechargeItemsGuest")
    @PermitGuest
    @Override
    public Result getCoinRechargeItemsGuest() {
        List<CoinRecharge> list = rechargeService.getCoinRechargeByChannelId(this.getHeader().getChannelId());
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(list, lang()));
        return mResult.build();
    }

    /**
     * 创建金币充值订单
     *
     * @param itemId
     * @return
     */
    @POST
    @Path("addCoinRechargeOrder")
    @Override
    public Result addCoinRechargeOrder(@FormParam("itemId") String itemId) {

        Long realItemId = 0L;
        if (StringUtils.isNotBlank(itemId)) {
            if (itemId.contains("_")) {
                realItemId = Convert.toLong(itemId.split("_")[1], 0L);
            } else {
                realItemId = Convert.toLong(itemId, 0L);
            }
        }

        // 判断充值项目是否存在
        CoinRechargeItem coinRechargeItem = rechargeService.getCoinRechargeItem(realItemId);
        if (coinRechargeItem == null) {
            return new Result(GlobalApiError.OperationFailed, "充值项目不存在");
        }

        // 创建充值订单
        CoinRechargeOrderVO coinRechargeOrderVO = rechargeService.addCoinRechargeOrder(CoinRechargeOrderAddDTO.builder()
                .itemId(realItemId)
                .userId(this.getToken().getUserId())
                .channelId(this.getHeader().getChannelId())
                .clientIp(this.getIpAddress()).build());

        if (coinRechargeOrderVO != null) {
            return new Result(coinRechargeOrderVO);
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 创建金币充值订单
     *
     * @param itemId
     * @param userId
     * @return
     */
    @POST
    @Path("addCoinRechargeOrderGuest")
    @PermitGuest
    @Override
    public Result addCoinRechargeOrderGuest(@FormParam("itemId") String itemId,
                                            @FormParam("userId") Long userId) {

        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.ServerError, "用户不存在");
        }

        if (StringUtils.isEmpty(itemId)) {
            return new Result(GlobalApiError.ServerError, "充值项目不能为空");
        }

        Long realItemId = 0L;
        if (itemId.contains("_")) {
            realItemId = Convert.toLong(itemId.split("_")[1], 0L);
        } else {
            realItemId = Convert.toLong(itemId, 0L);
        }

        // 判断充值项目是否存在
        CoinRechargeItem coinRechargeItem = rechargeService.getCoinRechargeItem(realItemId);
        if (coinRechargeItem == null) {
            return new Result(GlobalApiError.OperationFailed, "充值项目不存在");
        }

        // 创建充值订单
        CoinRechargeOrderVO coinRechargeOrderVO = rechargeService.addCoinRechargeOrder(CoinRechargeOrderAddDTO.builder()
                .itemId(realItemId)
                .userId(userId)
                .channelId(this.getHeader().getChannelId())
                .clientIp(this.getIpAddress()).build());

        if (coinRechargeOrderVO != null) {
            return new Result(coinRechargeOrderVO);
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 创建h5金币充值订单
     *
     * @param itemId
     * @param userId
     * @param loginUserId
     * @param sign
     * @return
     */
    @POST
    @Path("addH5CoinRechargeOrderGuest")
    @PermitGuest
    @Override
    public Result addH5CoinRechargeOrderGuest(@FormParam("itemId") String itemId,
                                              @FormParam("userId") Long userId,
                                              @FormParam("loginUserId") String loginUserId,
                                              @FormParam("sign") String sign
    ) {

        if (StringUtils.isEmpty(loginUserId) || StringUtils.isEmpty(sign) || !sign.equalsIgnoreCase(SafeUtil.getSign(loginUserId))) {
            return new Result(GlobalApiError.OperationFailed, "登录失效");
        }

        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.ServerError, "用户不存在");
        }

        if (StringUtils.isEmpty(itemId)) {
            return new Result(GlobalApiError.ServerError, "充值项目不能为空");
        }

        Long realItemId = 0L;
        if (itemId.contains("_")) {
            realItemId = Convert.toLong(itemId.split("_")[1], 0L);
        } else {
            realItemId = Convert.toLong(itemId, 0L);
        }

        // 判断充值项目是否存在
        CoinRechargeItem coinRechargeItem = rechargeService.getCoinRechargeItem(realItemId);
        if (coinRechargeItem == null) {
            return new Result(GlobalApiError.OperationFailed, "充值项目不存在");
        }

        // 创建充值订单
        CoinRechargeOrderVO coinRechargeOrderVO = rechargeService.addCoinRechargeOrder(CoinRechargeOrderAddDTO.builder()
                .itemId(realItemId)
                .userId(userId)
                .channelId(this.getHeader().getChannelId())
                .clientIp(this.getIpAddress()).build());

        if (coinRechargeOrderVO != null) {
            return new Result(coinRechargeOrderVO);
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取今日充值
     *
     * @return
     */
    @GET
    @Path("getTodayRecharge")
    @Override
    public Result getTodayRecharge() {
        RankScore rankScore = rechargeService.getTodayRecharge(this.getToken().getUserId());
        if (rankScore == null) {
            rankScore = new RankScore();
            rankScore.setScore(0d);
            rankScore.setRankNo(-1L);
        }
        MutableResult mResult = new MutableResult();
        mResult.put("amount", rankScore.getScore());
        mResult.put("rankNo", rankScore.getRankNo());
        return mResult.build();
    }
}
