package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.rtc.MainPageRoomConfigVO;
import com.starbuds.server.api.app.pojo.rtc.MainPageRoomListVO;
import com.starbuds.server.api.app.v1.facade.RtcFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.media.BattleStatusEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.*;
import com.starbuds.server.common.constant.user.NoblePrivilegeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.RtcRoomBackground;
import com.starbuds.server.common.pojo.db.RtcBattleRoomCrossDO;
import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.db.RtcRoomLiveHisDO;
import com.starbuds.server.common.pojo.db.RtcRoomRoleDO;
import com.starbuds.server.common.pojo.media.LiveRankItem;
import com.starbuds.server.common.pojo.message.im.rtc.RtcLiveStatusMsg;
import com.starbuds.server.common.pojo.rtc.*;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomVisitHisVO;
import com.starbuds.server.common.pojo.user.TokenBO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotNull;
import javax.ws.rs.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * RTC 房间接口实现
 *
 * @author Lay
 * @date 2020/7/6
 */
@Path("/v1/rtc")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RtcFacadeImpl extends RtcBaseFacade implements RtcFacade {

    /**
     * 召唤粉丝：每次间隔时间
     */
    @Value("${xy.rtc.convene.fans.interval}")
    private Integer CONVENE_FANS_INTERVAL;

    /**
     * 召唤粉丝：每天最大使用次数
     */
    @Value("${xy.rtc.convene.fans.max}")
    private Integer CONVENE_FANS_MAX;

    private final Random random = new Random();

    /**
     * 创建房间
     *
     * @param roomType
     * @param categoryId
     * @param roomName
     * @param roomCover
     * @param roomHorizontalCover
     * @return
     */
    @POST
    @Path("createRoom")
    @Override
    public Result createRoom(@FormParam("roomType") Integer roomType,
                             @FormParam("categoryId") Long categoryId,
                             @FormParam("roomName") String roomName,
                             @FormParam("roomCover") String roomCover,
                             @FormParam("roomHorizontalCover") String roomHorizontalCover) {
        Long userId = this.getToken().getUserId();
        Long channelId = this.getHeader().getChannelId();

        // 判断房间类型
        if (RoomTypeEnum.toEnum(roomType) == null) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "不支持该房间类型");
        }

        // 判断类目是否存在
        if (!dataService.isRtcRoomCategoryExists(categoryId)) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "无法设置该标签");
        }


        // 判断房间名称
        if (StringUtils.isBlank(roomName)) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "请填写房间名称");
        }

        if (roomName.length() > 45) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "房间名称太长");
        }


        // 授权语聊配置
        AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(userId, channelId);
        if (authRtcConfig == null) {
            return buildFailResult(false, GlobalApiError.ServerError, "该功能逐步开放中，敬请期待");

        }

        // 用户开通房间数
        int roomCount = rtcRoomService.getRoomCountByUserId(userId);
        if (roomCount >= authRtcConfig.getRoomNum()) {
            return buildFailResult(false, GlobalApiError.ServerError, "您的房间数量超上限");
        }

        // 房间分类
        String authRoomCategory = authRtcConfig.getRoomCategory();
        if (!authRoomCategory.contains(Convert.toStr(categoryId))) {
            return buildFailResult(false, GlobalApiError.ServerError, "您未获得该类目授权");
        }

        // 房间类型
        String authRoomType = authRtcConfig.getRoomType();
        if (!authRoomType.contains(Convert.toStr(roomType))) {
            return buildFailResult(false, GlobalApiError.ServerError, "您未获得该类型授权");
        }
        // 判断是否实名认证
        if (!userService.isAuthExist(userId)) {
            return buildFailResult(true, GlobalApiError.OperationFailed, "您尚未实名认证");
        }

        RtcRoomAddDTO rtcRoomAddDTO = new RtcRoomAddDTO();
        rtcRoomAddDTO.setUserId(userId);
        rtcRoomAddDTO.setCategoryId(categoryId);
        rtcRoomAddDTO.setRoomCover(roomCover);
        rtcRoomAddDTO.setRoomHorizontalCover(roomHorizontalCover);
        rtcRoomAddDTO.setRoomName(roomName);
        rtcRoomAddDTO.setRoomType(roomType);
        rtcRoomAddDTO.setChannelId(channelId);

        Long roomId = rtcRoomService.addRoom(rtcRoomAddDTO);
        if (roomId == null) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "房间创建失败");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("roomId", roomId);
        mResult.put("goAuth", "0");
        return mResult.build();
    }

    private Result buildFailResult(boolean needGoAuth, GlobalApiError errCode, String msg) {
        MutableResult failBuild = new MutableResult();
        if (needGoAuth) {
            failBuild.put("goAuth", "1");
        } else {
            failBuild.put("goAuth", "0");
        }
        Result failResult = failBuild.build();
        failResult.setSuccess(false);
        failResult.setCode(errCode.getCode());
        failResult.setMsg(msg);
        return failResult;
    }

    /**
     * 进入语音房（客户端2.0.0以上版本开始调用）
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("enterRoom")
    @Override
    public Result enterRoom(@FormParam("roomId") @NotNull(message = "roomId is required") Long roomId) {
        Long userId = getToken().getUserId();
        rtcRoomService.sendUserJoinInRoomMsg(System.currentTimeMillis(), userId, roomId);
        // 排查Bug：用户在房间内，在线列表中看不到
        log.info("融云-rong-userJoin-{}-{}-{}-sendMsg", userId, roomId, getHeader().getAppVersion());
        return Result.ofSuccess();
    }

    /**
     * 获取我的房间列表
     *
     * @return
     */
    @GET
    @Path("getMyRooms")
    @Override
    public Result getMyRooms() {
        Long userId = this.getToken().getUserId();

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRoomService.getUserRoomList(userId));
        return mResult.build();
    }

    /**
     * 获取房间详情
     *
     * @param roomId
     * @param pass
     * @return
     */
    @GET
    @Path("getRoomInfo")
    @Override
    public Result getRoomInfo(@QueryParam("roomId") Long roomId,
                              @QueryParam("pass") String pass) {

        // 如果进入的不是自己的房间，并且自己房间在开播状态，那就不让进入
        Long userId = this.getToken().getUserId();
        Long familyId = familyService.getFamilyId(userId);
        if (familyId != null) {
            RtcRoomDO myRoom = rtcRoomService.getSingleRtcRoom(userId, familyId);
            if (myRoom != null) {
                if (myRoom.getStatus() == CommonStatus.Normal && !myRoom.getRoomId().equals(roomId)) {
                    return buildFailResult(false, GlobalApiError.OperationFailed, "您正在开播状态，请前往房间关播！");
                }
            }
        }


        MutableResult data = (MutableResult) getRoomInfoBase(roomId, pass).getData();
        RtcRoom rtcRoom = JSON.parseObject(JSON.toJSONString(data.get("room")), RtcRoom.class);

        if (rtcRoom != null) {

            if (rtcRoom.getBusinessType().equals(RtcBusinessTypeEnum.RTC_SINGLE.getValue())) {

                data.put("singleRtcHisId", Optional.ofNullable(rtcRoomLiveHisService.getCurrentRound(roomId)).orElse(new RtcRoomLiveHisDO()).getHisId());
                data.put("isSingleRtcAnchorOnline", rtcRoomService.isSingleRtcRoomUserOnline(rtcRoom.getUserId()) ? BoolIntVal.True : BoolIntVal.False);
            }
        } else {
            data.put("singleRtcHisId", null);
            data.put("isSingleRtcAnchorOnline", BoolIntVal.False);
        }
        return data.build();
    }

    /**
     * 刷新房间token
     *
     * @param roomID
     * @return
     */
    @POST
    @Path("getRefreshToken")
    @Override
    public Result getRefreshToken(@FormParam("roomID") Long roomID) {
        Long userId = this.getToken().getUserId();

        MutableResult mResult = new MutableResult();

        mResult.put("authInfo", rtcRoomService.getAgoraAuthInfo(roomID, userId, getHeader().getAppVersion()));

        return mResult.build();
    }

    /**
     * 设置房间锁密码
     *
     * @param roomId
     * @param pass
     * @return
     */
    @POST
    @Path("setRoomLockPass")
    @Override
    public Result setRoomLockPass(@FormParam("roomId") Long roomId,
                                  @FormParam("pass") String pass) {
        Long userId = this.getToken().getUserId();

        if (StringUtils.isBlank(pass)) {
            pass = null;
        }

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 设置密码
        if (rtcRoomService.setRoomLockPass(roomId, pass)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 编辑房间
     *
     * @param roomId
     * @param roomCover
     * @param roomHorizontalCover
     * @param roomName
     * @param roomWelcome
     * @param roomRule
     * @param categoryId
     * @return
     */
    @POST
    @Path("editRoom")
    @Override
    public Result editRoom(@FormParam("roomId") Long roomId,
                           @FormParam("roomCover") String roomCover,
                           @FormParam("roomHorizontalCover") String roomHorizontalCover,
                           @FormParam("roomName") String roomName,
                           @FormParam("roomWelcome") String roomWelcome,
                           @FormParam("roomRule") String roomRule,
                           @FormParam("categoryId") Long categoryId) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断类目是否有效
        if (categoryId != null && !dataService.isRtcRoomCategoryExists(categoryId)) {
            return new Result(GlobalApiError.OperationFailed, "分类无效");
        }

        if (categoryId != null) {
            // 授权语聊配置
            AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(userId, this.getHeader().getChannelId());
            if (authRtcConfig == null) {
                return new Result(GlobalApiError.ServerError, "该功能逐步开放中，敬请期待");
            }

            // 房间分类
            String authRoomCategory = authRtcConfig.getRoomCategory();
            if (!authRoomCategory.contains(Convert.toStr(categoryId))) {
                return new Result(GlobalApiError.ServerError, "您未获得该类目授权");
            }
        }

        RtcRoomEditDTO roomEditDTO = new RtcRoomEditDTO();
        roomEditDTO.setRoomId(roomId);
        roomEditDTO.setRoomCover(roomCover);
        roomEditDTO.setRoomHorizontalCover(roomHorizontalCover);
        roomEditDTO.setRoomName(roomName);
        roomEditDTO.setRoomWelcome(roomWelcome);
        roomEditDTO.setRoomRule(roomRule);
        roomEditDTO.setCategoryId(categoryId);
        roomEditDTO.setUserId(userId);
        if (rtcRoomService.editRoom(roomEditDTO)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取房管列表
     *
     * @param roomId
     * @param roomRole
     * @return
     */
    @GET
    @Path("getRoomMangers")
    @Override
    public Result getRoomMangers(@QueryParam("roomId") Long roomId, @QueryParam("roomRole") Integer roomRole) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限
        int userRoomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (userRoomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRoomService.getRoomMangers(roomId, roomRole));
        return mResult.build();
    }

    /**
     * 添加房间主持人
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("addRoomHost")
    @Override
    public Result addRoomHost(@FormParam("roomId") Long roomId,
                              @FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限
        int myRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (myRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断目标权限
        int targetUserRole = rtcRoomService.getRoomRole(roomId, userId);
        if (myRole < targetUserRole) {
            return new Result(GlobalApiError.OperationFailed, "你没有对方权限高");
        }

        // 判断是否自己
        if (myUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "不能对自己操作");
        }

        // 判断用户是否存在
        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.OperationFailed, "目标用户不存在");
        }

        RtcRoomRoleDO rtcRoomRole = new RtcRoomRoleDO();
        rtcRoomRole.setRoomId(roomId);
        rtcRoomRole.setUserId(userId);
        rtcRoomRole.setRoomRole(RtcRoomRole.HOST);
        if (rtcRoomService.addRoomRole(rtcRoomRole)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 移除房间主持人
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("removeRoomHost")
    @Override
    public Result removeRoomHost(@FormParam("roomId") Long roomId,
                                 @FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限
        int myRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (myRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断目标权限
        int targetUserRole = rtcRoomService.getRoomRole(roomId, userId);
        if (myRole < targetUserRole) {
            return new Result(GlobalApiError.OperationFailed, "你没有对方权限高");
        }

        // 判断是否自己
        if (myUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "不能对自己操作");
        }

        if (rtcRoomService.removeRoomRole(roomId, userId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 添加房管
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("addRoomMaster")
    @Override
    public Result addRoomMaster(@FormParam("roomId") Long roomId,
                                @FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.MASTER) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断是否自己
        if (myUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "不能对自己操作");
        }

        // 判断对象是否是管理或房主
        if (rtcRoomService.getRoomRole(roomId, userId) > RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "该用户已拥有房管权限");
        }

        // 判断用户是否存在
        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.OperationFailed, "目标用户不存在");
        }

        RtcRoomRoleDO rtcRoomRole = new RtcRoomRoleDO();
        rtcRoomRole.setRoomId(roomId);
        rtcRoomRole.setUserId(userId);
        rtcRoomRole.setRoomRole(RtcRoomRole.MASTER);
        if (rtcRoomService.addRoomRole(rtcRoomRole)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 移除房管
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("removeRoomMaster")
    @Override
    public Result removeRoomMaster(@FormParam("roomId") Long roomId,
                                   @FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.MASTER) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断对象是否是房管
        if (rtcRoomService.getRoomRole(roomId, userId) != RtcRoomRole.MASTER) {
            return new Result(GlobalApiError.OperationFailed, "该用户不是房管或已被移除");
        }

        // 判断是否自己
        if (myUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "不能对自己操作");
        }

        if (rtcRoomService.removeRoomRole(roomId, userId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 申请上麦
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("sitSeat")
    @Override
    public Result sitSeat(@FormParam("roomId") Long roomId,
                          @FormParam("seatNo") Integer seatNo) {
        Long userId = this.getToken().getUserId();
        return rtcUtil.sitSeat(userId, roomId, seatNo, getHeader().getAppVersion());
    }

    /**
     * 下麦
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("leaveSeat")
    @Override
    public Result leaveSeat(@FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        int seatNo = rtcSeatService.leaveSeat(roomId, userId, getHeader().getAppVersion());
        if (seatNo > -1) {
            MutableResult mResult = new MutableResult();
            mResult.put("seatNo", seatNo);
            mResult.put("authInfo", rtcRoomService.getAgoraAuthInfo(roomId, userId, getHeader().getAppVersion()));
            return mResult.build();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 离开麦序队列
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("leaveQueue")
    @Override
    public Result leaveQueue(@FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        if (rtcQueueService.removeQueueUser(roomId, userId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取麦序用户列表
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getQueueUsers")
    @Override
    public Result getQueueUsers(@QueryParam("roomId") Long roomId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcQueueService.getUsers(roomId));
        return mResult.build();
    }

    /**
     * 获取麦序用户数量
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getQueueUsersCount")
    @Override
    public Result getQueueUsersCount(@QueryParam("roomId") Long roomId) {
        MutableResult mResult = new MutableResult();
        mResult.put("count", rtcQueueService.getUsersCount(roomId));
        return mResult.build();
    }

    /**
     * 获取麦位信息
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getSeats")
    @Override
    public Result getSeats(@QueryParam("roomId") Long roomId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcSeatService.getSeats(roomId));
        return mResult.build();
    }

    /**
     * 从麦序移除用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("removeQueueUser")
    @Override
    public Result removeQueueUser(@FormParam("roomId") Long roomId,
                                  @FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        rtcUtil.doRemoveQueueUserHandler(userId, roomId);
        return new Result();
    }

    /**
     * 抱用户上麦
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("setUserToSeat")
    @Override
    public Result setUserToSeat(@FormParam("roomId") Long roomId,
                                @FormParam("userId") Long userId,
                                @FormParam("seatNo") Integer seatNo) {
        Long myUserId = this.getToken().getUserId();

        // 相亲房
        if (RoomTypeEnum.LOVE.getValue() == rtcRoomService.getRoomType(roomId)) {
            if (rtcLoveService.getStage(roomId) > RtcLoveStage.QUESTION) {
                return new Result(GlobalApiError.OperationFailed, "当前阶段无法上麦");
            }
        }

        Result checkResult = rtcUtil.checkSetUserToSet(roomId, myUserId, userId, seatNo);
        if (!checkResult.isSuccess()) {
            return checkResult;
        }

        // 跳锁上麦
        int seat = rtcSeatService.sitSeat(roomId, userId, seatNo, true, getHeader().getAppVersion());
        if (seat > 0) {
            MutableResult mResult = new MutableResult();
            mResult.put("seatNo", seat);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 移除麦位用户
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("removeSeatUser")
    @Override
    public Result removeSeatUser(@FormParam("roomId") Long roomId,
                                 @FormParam("seatNo") int seatNo) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (seatNo < 0) {
            return new Result(GlobalApiError.OperationFailed, "麦位错误");
        }

        Long userId = rtcSeatService.getSeatUserId(roomId, seatNo);
        if (userId == null) {
            return new Result();
        }

        // 强制下麦
        if (rtcSeatService.leaveSeat(roomId, seatNo, true, getHeader().getAppVersion()) > -1) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 快速上麦
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("setUserToSeatInQueue")
    @Override
    public Result setUserToSeatInQueue(@FormParam("roomId") Long roomId) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }
        // 自动上麦
        rtcSeatService.sitSeatInQueue(roomId, getHeader().getAppVersion());

        return new Result();
    }

    /**
     * 锁定麦位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("lockSeat")
    @Override
    public Result lockSeat(@FormParam("roomId") Long roomId,
                           @FormParam("seatNo") Integer seatNo) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (seatNo == null) {
            rtcSeatService.lockSeatAll(roomId);
            return new Result();
        } else {

            // 判断是不是个播房
            if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoomService.getRoomBusinessType(roomId)) {
                if (seatNo > 4) {
                    return new Result(GlobalApiError.OperationFailed, "个播房间只有四个麦位可用哦~");
                }
            }

            if (rtcSeatService.lockSeat(roomId, seatNo)) {
                return new Result();
            }
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 解锁麦位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("unlockSeat")
    @Override
    public Result unlockSeat(@FormParam("roomId") Long roomId,
                             @FormParam("seatNo") Integer seatNo) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (seatNo == null) {
            rtcSeatService.unlockSeatAll(roomId);
            return new Result();
        } else {
            if (rtcSeatService.unlockSeat(roomId, seatNo)) {
                return new Result();
            }
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 麦位禁麦
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("setMicDisabled")
    @Override
    public Result setMicDisabled(@FormParam("roomId") Long roomId,
                                 @FormParam("seatNo") int seatNo) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (seatNo > 4) {
            // 判断是不是个播房
            if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoomService.getRoomBusinessType(roomId)) {
                return new Result(GlobalApiError.OperationFailed, "个播房间只有四个麦位可用哦~");
            }
        }

        if (rtcSeatService.setMicDisabled(roomId, seatNo, true)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 麦位开麦
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("setMicEnabled")
    @Override
    public Result setMicEnabled(@FormParam("roomId") Long roomId,
                                @FormParam("seatNo") int seatNo) {
        Long myUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (seatNo > 4) {
            // 判断是不是个播房
            if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoomService.getRoomBusinessType(roomId)) {
                return new Result(GlobalApiError.OperationFailed, "个播房间只有四个麦位可用哦~");
            }
        }

        if (rtcSeatService.setMicDisabled(roomId, seatNo, false)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 随机获取相亲房
     *
     * @return
     */
    //随机获取相亲房
    @GET
    @Path("getRandomLoveRoom")
    @PermitGuest
    @Override
    public Result getRandomLoveRoom() {
        return getResult(RoomTypeEnum.LOVE);
    }

    /**
     * 随机获取拍卖房
     *
     * @return
     */
    @Override
    @GET
    @Path("getRandomAuctionRoom")
    @PermitGuest
    public Result getRandomAuctionRoom() {
        return getResult(RoomTypeEnum.AUCTION);
    }

    private Result getResult(RoomTypeEnum input) {
        List<RtcRoomListItem> roomListItems = rtcRoomService.getRoomListByType(1, GlobalConfig.PAGE_SIZE, input.getValue());
        if (CollectionUtils.isEmpty(roomListItems)) {
            return new Result("暂无房间开播");
        }
        int randomIndex = random.nextInt(roomListItems.size());
        return new Result(roomListItems.get(randomIndex));
    }

    /**
     * 获取房间列表
     *
     * @param pageIndex
     * @param categoryId
     * @return
     */
    @GET
    @Path("getRoomList")
    @PermitGuest
    @Override
    public Result getRoomList(@QueryParam("pageIndex") int pageIndex,
                              @QueryParam("categoryId") Long categoryId,
                              @QueryParam("businessType") Integer businessType) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }

        // 兼容老版本，不传默认为0：语聊房
        // 1为个播房
        if (businessType == null) {
            businessType = 0;
        }

        PageList<RtcRoomListItem> list = rtcRoomService.getRoomList(pageIndex, GlobalConfig.PAGE_SIZE, categoryId, businessType);
        if (isDev()) {
            log.info("获取房间列表:{}", JSON.toJSONString(list));
        }

        return new Result(list);
    }


    /**
     * 获取首页房间
     *
     * @return
     */
    @GET
    @Path("getMainPageRoomList")
    @PermitGuest
    @Override
    public Result getMainPageRoomList() {

        String config = releaseConfigService.getCachReleaseConfig("mainPageCategory");
        List<MainPageRoomConfigVO> configVo = JSON.parseArray(config, MainPageRoomConfigVO.class);

        List<MainPageRoomListVO> res = new ArrayList<>();
        configVo.forEach(i -> {
            MainPageRoomListVO tmp = new MainPageRoomListVO();
            tmp.setLayout(i.getLayout());
            tmp.setCategoryName(i.getCategoryName());
            tmp.setCategoryId(i.getCategoryId());
            tmp.setTitle(i.getTitle());
            List<RtcRoomListItem> roomList = rtcRoomService.getRoomListByType(1, i.getPageSize(), i.getRoomType());
            roomList.forEach(j -> j.setCategoryName(i.getCategoryName()));
            tmp.setRtcRoomList(roomList);
            res.add(tmp);
        });

        return Result.ofList(res);
    }

    /**
     * 获取交友房间列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getMakeFriendRoomList")
    @PermitGuest
    @Override
    public Result getMakeFriendRoomList(@QueryParam("pageIndex") int pageIndex) {
        return new Result(rtcRoomService.getMakeFriendRoomList(pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 收藏房间
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("addRoomFav")
    @Override
    public Result addRoomFav(@FormParam("roomId") Long roomId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        if (rtcRoomService.addRoomFav(roomId, this.getToken().getUserId())) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 取消收藏房间
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("removeRoomFav")
    @Override
    public Result removeRoomFav(@FormParam("roomId") Long roomId) {
        if (rtcRoomService.removeRoomFav(roomId, this.getToken().getUserId())) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取收藏的房间列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getFavRoomList")
    @Override
    public Result getFavRoomList(@QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        Long userId = this.getToken().getUserId();
        return new Result(rtcRoomService.getFavRoomList(pageIndex, GlobalConfig.PAGE_SIZE, userId));
    }

    /**
     * 设置上麦模式
     *
     * @param roomId
     * @param micFree
     * @return
     */
    @POST
    @Path("setMicFree")
    @Override
    public Result setMicFree(@FormParam("roomId") Long roomId,
                             @FormParam("micFree") int micFree) {
        Long userId = this.getToken().getUserId();

        micFree = BoolIntVal.True == micFree ? micFree : BoolIntVal.False;

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断是否相亲房
        if (RoomTypeEnum.LOVE.getValue() == rtcRoomService.getRoomType(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "相亲房无法设置自由上麦");
        }

        if (rtcRoomService.setMicFree(roomId, micFree)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置心动值开关
     *
     * @param roomId
     * @param scoreEnabled
     * @return
     */
    @POST
    @Path("setScoreEnabled")
    @Override
    public Result setScoreEnabled(@FormParam("roomId") Long roomId,
                                  @FormParam("scoreEnabled") int scoreEnabled) {
        Long userId = this.getToken().getUserId();

        scoreEnabled = BoolIntVal.True == scoreEnabled ? scoreEnabled : BoolIntVal.False;

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断是否相亲房
        if (RoomTypeEnum.LOVE.getValue() == rtcRoomService.getRoomType(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "相亲房无法设置心动值开关");
        }

        if (rtcRoomService.setScoreEnabled(roomId, scoreEnabled)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置公屏开关
     *
     * @param roomId
     * @param chatEnabled
     * @return
     */
    @POST
    @Path("setChatEnabled")
    @Override
    public Result setChatEnabled(@FormParam("roomId") Long roomId,
                                 @FormParam("chatEnabled") int chatEnabled) {
        Long userId = this.getToken().getUserId();

        chatEnabled = BoolIntVal.True == chatEnabled ? chatEnabled : BoolIntVal.False;

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (rtcRoomService.setChatEnabled(roomId, chatEnabled)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置大冒险开关
     *
     * @param roomId
     * @param bigAdventureEnabled
     * @return
     */
    @POST
    @Path("setBigAdventureEnabled")
    @Override
    public Result setBigAdventureEnabled(@FormParam("roomId") Long roomId, @FormParam("bigAdventureEnabled") int bigAdventureEnabled) {
        Long userId = this.getToken().getUserId();

        bigAdventureEnabled = BoolIntVal.True == bigAdventureEnabled ? bigAdventureEnabled : BoolIntVal.False;

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断配置是否齐全
        if (!rtcRoomService.isTurntableConfig(roomId)) {
            return new Result(GlobalApiError.IncompleteConfiguration);
        }

        if (rtcRoomService.setBigAdventureEnabled(roomId, bigAdventureEnabled)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置麦克风静音状态
     *
     * @param roomId
     * @param micSilenced
     * @return
     */
    @POST
    @Path("setMicSilenced")
    @Override
    public Result setMicSilenced(@FormParam("roomId") Long roomId,
                                 @FormParam("micSilenced") int micSilenced) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断是否在麦位上
        int seatNo = rtcSeatService.getUserSeatNo(roomId, userId);
        if (seatNo == -1) {
            return new Result(GlobalApiError.OperationFailed, "您未在麦位上");
        }

        // 设置麦位静音状态
        if (rtcSeatService.setMicSilenced(roomId, seatNo, micSilenced == 1)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置房间背景
     *
     * @param roomId
     * @param backgroundId
     * @return
     */
    @POST
    @Path("setRoomBackground")
    @Override
    public Result setRoomBackground(@FormParam("roomId") Long roomId,
                                    @FormParam("backgroundId") Integer backgroundId) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断背景是否有效
        if (backgroundId != null && !dataService.isRtcRoomBackgroundExists(backgroundId)) {
            return new Result(GlobalApiError.OperationFailed, "无效的房间背景");
        }

        RtcRoomBackground background = dataService.getRtcRoomBackground(backgroundId);
        if (background.getScope() == 1) {
            if (StringUtils.isBlank(background.getScopeValue())) {
                return new Result(GlobalApiError.OperationFailed, "没有权限获取房间背景");
            }
            if (System.currentTimeMillis() > background.getEndTimestamp()) {
                return new Result(GlobalApiError.OperationFailed, "房间背景不再有效期内");
            }

            boolean isMatch = Arrays.stream(background.getScopeValue().split(",")).anyMatch(m -> m.contentEquals(roomId + ""));
            if (!isMatch) {
                return new Result(GlobalApiError.OperationFailed, "没有权限获取房间背景");
            }
        }

        if (rtcRoomService.setRoomBackground(roomId, backgroundId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取星钻日榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDiamondRankDay")
    @Override
    public Result getDiamondRankDay(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDiamondRankDay(roomId, 0, 29, false);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDiamondDayRankItem(roomId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取星钻周榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDiamondRankWeek")
    @PermitGuest
    @Override
    public Result getDiamondRankWeek(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDiamondRankWeek(roomId, 0, 29, false);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        TokenBO token = this.getToken();
        mResult.put("myRankItem", token == null ? null : rtcRankService.getDiamondWeekRankItem(roomId, token.getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取星钻月榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDiamondRankMonth")
    @Override
    public Result getDiamondRankMonth(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDiamondRankMonth(roomId, 0, 29, false);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDiamondMonthRankItem(roomId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取星尘日榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDustRankDay")
    @Override
    public Result getDustRankDay(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDustRankDay(roomId, 0, 29, false);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDustDayRankItem(roomId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取星尘周榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDustRankWeek")
    @Override
    public Result getDustRankWeek(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDustRankWeek(roomId, 0, 29, false);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDustWeekRankItem(roomId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取星尘月榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDustRankMonth")
    @Override
    public Result getDustRankMonth(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDustRankMonth(roomId, 0, 29, false);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDustMonthRankItem(roomId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取星钻往期日榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDiamondRankDayPrevious")
    @Override
    public Result getDiamondRankDayPrevious(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDiamondRankDay(roomId, 0, 29, true);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDiamondDayRankItem(roomId, this.getToken().getUserId(), true));
        return mResult.build();
    }

    /**
     * 获取星钻往期周榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDiamondRankWeekPrevious")
    @PermitGuest
    @Override
    public Result getDiamondRankWeekPrevious(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDiamondRankWeek(roomId, 0, 29, true);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        TokenBO token = this.getToken();
        // 我的排名信息
        mResult.put("myRankItem", token == null ? null : rtcRankService.getDiamondWeekRankItem(roomId, token.getUserId(), true));
        return mResult.build();
    }

    /**
     * 获取星钻往期月榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDiamondRankMonthPrevious")
    @Override
    public Result getDiamondRankMonthPrevious(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDiamondRankMonth(roomId, 0, 29, true);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDiamondMonthRankItem(roomId, this.getToken().getUserId(), true));
        return mResult.build();
    }

    /**
     * 获取星尘往期日榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDustRankDayPrevious")
    @Override
    public Result getDustRankDayPrevious(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDustRankDay(roomId, 0, 29, true);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDustDayRankItem(roomId, this.getToken().getUserId(), true));
        return mResult.build();
    }

    /**
     * 获取星尘往期周榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDustRankWeekPrevious")
    @Override
    public Result getDustRankWeekPrevious(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDustRankWeek(roomId, 0, 29, true);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDustWeekRankItem(roomId, this.getToken().getUserId(), true));
        return mResult.build();
    }

    /**
     * 获取星尘往期月榜
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getDustRankMonthPrevious")
    @Override
    public Result getDustRankMonthPrevious(@QueryParam("roomId") Long roomId) {
        // 拉取前 30
        List<LiveRankItem> liveRankItems = rtcRankService.getDustRankMonth(roomId, 0, 29, true);
        MutableResult mResult = new MutableResult();
        mResult.put("list", liveRankItems);
        // 我的排名信息
        mResult.put("myRankItem", rtcRankService.getDustMonthRankItem(roomId, this.getToken().getUserId(), true));
        return mResult.build();
    }

    /**
     * 设置禁言
     *
     * @param roomId
     * @param userId
     * @param duration
     * @param remarks
     * @return
     */
    @POST
    @Path("setForbidden")
    @Override
    public Result setForbidden(@FormParam("roomId") Long roomId, @FormParam("userId") Long userId, @FormParam("duration") Long duration, @FormParam("remarks") String remarks) {
        Long curUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        // 判断权限
        int myRole = rtcRoomService.getRoomRole(roomId, curUserId);
        int destRole = rtcRoomService.getRoomRole(roomId, userId);
        if (myRole < RtcRoomRole.HOST || myRole <= destRole) {
            return new Result(GlobalApiError.OperationFailed, "权限不足");
        }

        if (noblePrivilegeService.hasPrivilege(userId, NoblePrivilegeEnum.PREVENT_SILENCE)) {
            return new Result(GlobalApiError.OperationFailed, "贵族特权：防止禁言");
        }

        if (imService.forbiddenChat(roomId, userId, Convert.toInt(duration / 1000 / 60))) {
            rtcRoomService.setForbidden(curUserId, roomId, userId, duration, remarks, CommonStatus.Normal);
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 移除禁言
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("removeForbidden")
    @Override
    public Result removeForbidden(@FormParam("roomId") Long roomId, @FormParam("userId") Long userId) {
        Long curUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        int myRole = rtcRoomService.getRoomRole(roomId, curUserId);
        int destRole = rtcRoomService.getRoomRole(roomId, userId);
        if (myRole < RtcRoomRole.HOST || myRole <= destRole) {
            return new Result(GlobalApiError.OperationFailed, "权限不足");
        }

        if (!rtcRoomService.isForbiddenListUser(roomId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "用户未被禁言");
        }

        if (imService.enableChat(roomId, userId)) {
            rtcRoomService.setForbidden(curUserId, roomId, userId, null, "", CommonStatus.Deleted);
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置黑名单
     *
     * @param roomId
     * @param userId
     * @param duration
     * @param blackRemarks
     * @return
     */
    @POST
    @Path("setBlack")
    @Override
    public Result setBlack(@FormParam("roomId") Long roomId, @FormParam("userId") Long userId, @FormParam("duration") Long duration, @FormParam("blackRemarks") String blackRemarks) {
        Long curUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        int myRole = rtcRoomService.getRoomRole(roomId, curUserId);
        int destRole = rtcRoomService.getRoomRole(roomId, userId);
        if (myRole < RtcRoomRole.HOST || myRole <= destRole) {
            return new Result(GlobalApiError.OperationFailed, "权限不足");
        }
        if (noblePrivilegeService.hasPrivilege(userId, NoblePrivilegeEnum.PREVENT_KICK)) {
            return new Result(GlobalApiError.OperationFailed, "贵族特权：防止被踢");
        }

        if (rtcRoomService.setBlack(roomId, userId, CommonStatus.Normal, duration, blackRemarks, this.getHeader().getChannelId(), curUserId, getHeader().getAppVersion())) {
            imService.blockRoomUser(roomId, userId, Convert.toInt(duration / 1000 / 60));
            MutableResult mResult = new MutableResult();
            mResult.put("userId", userId);
            return mResult.build();
        }

        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 移除黑名单
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("removeBlack")
    @Override
    public Result removeBlack(@FormParam("roomId") Long roomId, @FormParam("userId") Long userId) {
        Long curUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        int myRole = rtcRoomService.getRoomRole(roomId, curUserId);
        int destRole = rtcRoomService.getRoomRole(roomId, userId);
        if (myRole < RtcRoomRole.HOST || myRole <= destRole) {
            return new Result(GlobalApiError.OperationFailed, "权限不足");
        }

        if (!rtcRoomService.isBlackListUser(roomId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "用户未被拉黑");
        }

        if (imService.unblockRoomUser(roomId, userId)) {
            rtcRoomService.setBlack(roomId, userId, CommonStatus.Deleted, null, "", this.getHeader().getChannelId(), getHeader().getAppVersion());
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取禁言列表
     *
     * @param pageIndex
     * @param roomId
     * @return
     */
    @GET
    @Path("getForbiddenList")
    @Override
    public Result getForbiddenList(@QueryParam("pageIndex") int pageIndex, @QueryParam("roomId") Long roomId) {
        return new Result(rtcRoomService.getForbiddenList(pageIndex, GlobalConfig.PAGE_SIZE, roomId));
    }

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @param roomId
     * @param userNo
     * @return
     */
    @GET
    @Path("getBlackList")
    @Override
    public Result getBlackList(@QueryParam("pageIndex") int pageIndex, @QueryParam("roomId") Long roomId, @QueryParam("userNo") Long userNo) {
        return new Result(rtcRoomService.getBlackList(pageIndex, GlobalConfig.PAGE_SIZE, roomId, userNo));
    }

    /**
     * 获取管理数量
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getMasterNum")
    @Override
    public Result getMasterNum(@QueryParam("roomId") Long roomId) {

        MutableResult mResult = new MutableResult();
        mResult.put("masterNum", rtcRoomService.getMasterNum(roomId));
        mResult.put("hostNum", rtcRoomService.getHostNum(roomId));
        mResult.put("forbiddenNum", rtcRoomService.getForbiddenNum(roomId));
        mResult.put("blackNum", rtcRoomService.getBlackNum(roomId));

        return mResult.build();
    }

    /**
     * 获取在线用户
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getOnlineUser")
    @Override
    public Result getOnlineUser(@QueryParam("roomId") Long roomId) {
        MutableResult mResult = new MutableResult();
        List<RtcLiveUserProfile> list = rtcUserService.getUserProfiles(roomId);
        List<RtcLiveUserProfile> nobleList = getNobleList(list);

        mResult.put("list", list);
        mResult.put("onlineCount", list.size());
        mResult.put("onlineNobleCount", nobleList.size());
        return mResult.build();
    }

    /**
     * 获取在线用户-贵族
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getOnlineUserNoble")
    @Override
    public Result getOnlineUserNoble(@QueryParam("roomId") Long roomId) {
        MutableResult mResult = new MutableResult();
        List<RtcLiveUserProfile> list = rtcUserService.getUserProfiles(roomId);
        List<RtcLiveUserProfile> nobleList = getNobleList(list);

        mResult.put("list", nobleList);
        mResult.put("onlineCount", list.size());
        mResult.put("onlineNobleCount", nobleList.size());
        return mResult.build();
    }

    /**
     * 获得在线贵族用户
     *
     * @param list
     * @return
     */
    private List<RtcLiveUserProfile> getNobleList(List<RtcLiveUserProfile> list) {
        List<RtcLiveUserProfile> nobleList;
        if (CollectionUtils.isEmpty(list)) {
            nobleList = Collections.EMPTY_LIST;
        } else {
            nobleList = new ArrayList<>();
            for (RtcLiveUserProfile user : list) {
                if (user.getIsNoble()) {
                    nobleList.add(user);
                }
            }
        }
        return nobleList;
    }

    /**
     * 清空心动值
     *
     * @param roomId
     * @param userId
     * @return
     */
    @POST
    @Path("clearScore")
    @Override
    public Result clearScore(@FormParam("roomId") Long roomId,
                             @FormParam("userId") Long userId) {
        Long curUserId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, curUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 清空指定用户
        if (userId != null) {
            if (rtcRoomService.clearScore(roomId, userId)) {
                return new Result();
            }
        } else {
            // 清空所有
            if (rtcRoomService.clearAllScore(roomId)) {
                return new Result();
            }
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置老板位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("setBossSeat")
    @Override
    public Result setBossSeat(@FormParam("roomId") Long roomId,
                              @FormParam("seatNo") Integer seatNo) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (seatNo != null && seatNo > 4) {
            // 判断是不是个播房
            if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoomService.getRoomBusinessType(roomId)) {
                return new Result(GlobalApiError.OperationFailed, "个播房间只有四个麦位可用哦~");
            }
        }

        if (rtcSeatService.setBossSeat(roomId, seatNo)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 移除老板位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @POST
    @Path("removeBossSeat")
    @Override
    public Result removeBossSeat(@FormParam("roomId") Long roomId,
                                 @FormParam("seatNo") Integer seatNo) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (rtcSeatService.removeBossSeat(roomId, seatNo)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置倒计时
     *
     * @param roomId
     * @param seatNo
     * @param seconds
     * @return
     */
    @POST
    @Path("setCountDown")
    @Override
    public Result setCountDown(@FormParam("roomId") Long roomId,
                               @FormParam("seatNo") Integer seatNo,
                               @FormParam("seconds") Integer seconds) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断时间
        if (seconds == null || seatNo < 0) {
            seconds = 0;
        }

        if (seconds > 5 * 60) {
            return new Result(GlobalApiError.OperationFailed, "倒计时上限5分钟");
        }

        if (rtcSeatService.setCountDown(roomId, seatNo, seconds)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取心动值列表
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getSeatScoreList")
    @Override
    public Result getSeatScoreList(@QueryParam("roomId") Long roomId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcSeatService.getScoreList(roomId));
        return mResult.build();
    }

    /**
     * 获取用户心动值列表
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getUserScoreList")
    @Override
    public Result getUserScoreList(@QueryParam("roomId") Long roomId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }
        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcSeatService.getUserScoreList(roomId));
        return mResult.build();
    }

    /**
     * 获取授权语聊配置
     *
     * @return
     */
    @GET
    @Path("getAuthRtcConfig")
    @Override
    public Result getAuthRtcConfig() {
        return new Result(rtcUserService.getAuthRtcConfig(this.getToken().getUserId(), this.getHeader().getChannelId()));
    }

    /**
     * 获取授权信息
     *
     * @return
     */
    @GET
    @Path("getRtcAuthInfo")
    @Override
    public Result getRtcAuthInfo(@QueryParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        return new Result(rtcRoomService.getAgoraAuthInfo(roomId, userId, getHeader().getAppVersion()));
    }

    /**
     * 相亲模式下一阶段
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("nextLoveStage")
    @Override
    public Result nextLoveStage(@FormParam("roomId") Long roomId) {

        Integer roomType = rtcRoomService.getRoomType(roomId);
        if (roomType == null || RoomTypeEnum.LOVE.getValue() != roomType) {
            return new Result(GlobalApiError.OperationFailed, "仅相亲模式下可操作");
        }

        // 判断权限
        Long userId = this.getToken().getUserId();
        if (!rtcSeatService.isCurrentHost(roomId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        int stage = rtcLoveService.nextStage(roomId, getHeader().getAppVersion());
        MutableResult mResult = new MutableResult();
        mResult.put("stage", stage);
        return mResult.build();
    }

    /**
     * 相亲模式阶段重置
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("resetLoveStage")
    @Override
    public Result resetLoveStage(@FormParam("roomId") Long roomId) {
        Integer roomType = rtcRoomService.getRoomType(roomId);
        if (roomType == null || RoomTypeEnum.LOVE.getValue() != roomType) {
            return new Result(GlobalApiError.OperationFailed, "仅相亲模式下可操作");
        }

        // 判断权限
        Long userId = this.getToken().getUserId();
        if (!rtcSeatService.isCurrentHost(roomId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        if (rtcLoveService.resetStage(roomId, getHeader().getAppVersion())) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 心动选人
     *
     * @param roomId
     * @param targetSeatNo
     * @return
     */
    @POST
    @Path("chooseLove")
    @Override
    public Result chooseLove(@FormParam("roomId") Long roomId,
                             @FormParam("targetSeatNo") int targetSeatNo) {
        Long userId = this.getToken().getUserId();

        Integer roomType = rtcRoomService.getRoomType(roomId);
        if (roomType == null || RoomTypeEnum.LOVE.getValue() != roomType) {
            return new Result(GlobalApiError.OperationFailed, "仅相亲模式下可操作");
        }

        // 判断是否选人阶段
        if (RtcLoveStage.CHOOSE != rtcLoveService.getStage(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "只有选人阶段可以选人哦");
        }

        int seatNo = rtcSeatService.getUserSeatNo(roomId, userId);
        if (seatNo < 1) {
            return new Result(GlobalApiError.OperationFailed, "参与者才可以选人哦");
        }

        if (seatNo == targetSeatNo) {
            return new Result(GlobalApiError.OperationFailed, "不能选择自己哦");
        }

        // 判断目标麦位是否有人
        Long targetUserId = null;
        if (targetSeatNo > 0) {
            targetUserId = rtcSeatService.getSeatUserId(roomId, targetSeatNo);
            if (targetUserId == null) {
                return new Result(GlobalApiError.OperationFailed, "心动对象已下麦");
            }
        }

        // 判断是否主持人
        if (targetSeatNo == 0) {
            return new Result(GlobalApiError.OperationFailed, "不能选择主持人哦");
        }

        // 判断是否已选人
        if (rtcLoveService.isLoveChose(roomId, seatNo)) {
            return new Result(GlobalApiError.OperationFailed, "您已经选择过心动对象了");
        }

        if (rtcLoveService.chooseLove(roomId, userId, seatNo, targetUserId, targetSeatNo)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取热门房间列表
     *
     * @return
     */
    @GET
    @Path("getHotRoomList")
    @PermitGuest
    @Override
    public Result getHotRoomList(@QueryParam("businessType") Integer businessType) {

        if (businessType == null) {
            businessType = RtcBusinessTypeEnum.RTC_NORMAL.getValue();
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRoomService.getHotRoomList(50, businessType));
        return mResult.build();
    }

    /**
     * 发送表情
     *
     * @param roomId
     * @param expId
     * @return
     */
    @POST
    @Path("sendFacialExpression")
    @Override
    public Result sendFacialExpression(@FormParam("roomId") Long roomId, @FormParam("expId") Integer expId) {
        Long userId = this.getToken().getUserId();

        // 判断用户是否麦上
        int seatNo = rtcSeatService.getUserSeatNo(roomId, userId);

        // 判断表情是否有效
        if (!rtcRoomService.isFacialExpressionExists(roomId, expId)) {
            return new Result(GlobalApiError.OperationFailed, "该表情暂时无法使用");
        }

        if (rtcSeatService.sendFacialExpression(roomId, userId, seatNo, expId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "表情发送失败");
    }

    /**
     * 获取表情列表
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getFacialExpressionList")
    @Override
    public Result getFacialExpressionList(@QueryParam("roomId") Long roomId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRoomService.getFacialExpressionList(roomId));
        return mResult.build();
    }

    /**
     * 获取房间日流水
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @return
     */
    @GET
    @Path("getRtcRoomDayStream")
    @Override
    public Result getRtcRoomDayStream(@QueryParam("roomId") Long roomId,
                                      @QueryParam("startTime") Long startTime,
                                      @QueryParam("endTime") Long endTime) {
        if (roomId == null || startTime == null || endTime == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.ServerError, "房间不存在");
        }
        long diffTime = endTime - startTime;
        // 90天
        long ninetyDays = 90 * 24 * 3600 * 1000L;
        if (diffTime < 0 || diffTime > ninetyDays) {
            return new Result(GlobalApiError.ServerError, "时间跨度最长90天");
        }

        Long userId = this.getToken().getUserId();
        if (rtcRoomService.getRoomRole(roomId, userId) <= RtcRoomRole.AUDIENCE) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限");
        }

        return new Result(rtcRoomService.getRtcRoomDayStream(roomId, startTime, endTime));
    }

    /**
     * 获取房间小时流水
     *
     * @param roomId
     * @param date
     * @return
     */
    @GET
    @Path("getRtcRoomHourStream")
    @Override
    public Result getRtcRoomHourStream(@QueryParam("roomId") Long roomId, @QueryParam("date") Long date) {
        Long userId = this.getToken().getUserId();
        if (rtcRoomService.getRoomRole(roomId, userId) <= RtcRoomRole.AUDIENCE) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限");
        }

        return new Result(rtcRoomService.getRtcRoomHourStream(roomId, date));
    }

    /**
     * 获取房间周流水
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getRtcRoomWeekStream")
    @Override
    public Result getRtcRoomWeekStream(@QueryParam("roomId") Long roomId) {
        if (roomId == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.ServerError, "房间不存在");
        }

        Long userId = this.getToken().getUserId();
        if (rtcRoomService.getRoomRole(roomId, userId) <= RtcRoomRole.AUDIENCE) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限");
        }

        return new Result(rtcRoomService.getRtcRoomWeekStream(roomId));
    }

    /**
     * 获取房间月流水
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getRtcRoomMonthStream")
    @Override
    public Result getRtcRoomMonthStream(@QueryParam("roomId") Long roomId) {
        if (roomId == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.ServerError, "房间不存在");
        }

        Long userId = this.getToken().getUserId();
        if (rtcRoomService.getRoomRole(roomId, userId) <= RtcRoomRole.AUDIENCE) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限");
        }

        return new Result(rtcRoomService.getRtcRoomMonthStream(roomId));
    }

    /**
     * 获取心动值助攻用户列表
     *
     * @param roomId
     * @param userId
     * @return
     */
    @GET
    @Path("getRtcScoreAssisterList")
    @Override
    public Result getRtcScoreAssisterList(@QueryParam("roomId") Long roomId,
                                          @QueryParam("userId") Long userId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到房间");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcLoveService.getAssisterList(roomId, userId));
        return mResult.build();
    }

    /**
     * 召集粉丝
     * 1、在语音房麦位上，给所有粉丝一键发私信，召集粉丝来语音房；
     * 2、每次至少间隔1小时，每天三次；
     * 3、私信内容：我正在xxx（语音房名称）房间，快来和我一起玩耍吧！！！
     * 4、点击私信链接，可以直接跳转到语音房。
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("conveneFans")
    @Override
    public Result conveneFans(@FormParam("roomId") Long roomId) {
        Long userId = getToken().getUserId();

        // 是否在麦上
        boolean isSeat = false;

        // 麦位信息
        List<RtcSeat> seats = rtcSeatService.getSeats(roomId);
        Integer seatNo = null;
        for (RtcSeat seat : seats) {
            if (seat == null) {
                continue;
            }
            RtcRoomUser user = seat.getUser();
            if (user == null) {
                continue;
            }
            if (userId.equals(user.getUserId())) {
                seatNo = seat.getSeatNo();
                isSeat = true;
                break;
            }
        }
        if (!isSeat) {
            return new Result(GlobalApiError.BadRequest, "必须在麦位上");
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        DateTime today = new DateTime();

        // 每次间隔
        String intervalKey = StrUtil.format("rtc:convene:fans:interval:{}:{}", sdf.format(today), userId);
        String interval = redisService.get(intervalKey);
        if (StringUtils.isNotBlank(interval)) {
            return new Result(GlobalApiError.BadRequest, "每次至少间隔1小时");
        }
        // 每天最大次数
        String maxKey = StrUtil.format("rtc:convene:fans:max:{}:{}", sdf.format(today), userId);
        String maxStr = redisService.get(maxKey);
        Integer max = StringUtils.isBlank(maxStr) ? 0 : Integer.parseInt(maxStr);
        if (max >= CONVENE_FANS_MAX) {
            return new Result(GlobalApiError.BadRequest, "已达今日使用上限");
        }
        redisService.set(intervalKey, "ok", CONVENE_FANS_INTERVAL, TimeUnit.SECONDS);
        redisService.set(maxKey, ++max + "", 25, TimeUnit.HOURS);

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.BadRequest, "未找到该房间");
        }


        RtcRoom room = rtcRoomService.getRoom(roomId);
        String roomName = "";
        if (room != null) {
            roomName = room.getRoomName();
        }
        // 判断房间是否上锁
        String lockPass = rtcRoomService.getRoomLockPass(roomId);
        rtcRoomService.conveneFans(roomId, roomName, room.getRoomType(), StringUtils.isNotBlank(lockPass), seatNo, userId);
        return new Result("ok");
    }

    /**
     * 获取房间昨日钻石MVP
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getYesterdayDiamondRankMVP")
    @Override
    public Result getYesterdayDiamondRankMVP(@QueryParam("roomId") Long roomId) {
        return new Result(rtcRankService.getYesterdayDiamondRankMVP(roomId));
    }

    /**
     * 设置转盘配置
     *
     * @param roomId
     * @param turntableConfig
     * @return
     */
    @POST
    @Path("setTurntableConfig")
    @Override
    public Result setTurntableConfig(@FormParam("roomId") Long roomId, @FormParam("turntableConfig") String turntableConfig) {
        if (roomId == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.ServerError, "房间不存在");
        }

        Long userId = this.getToken().getUserId();
        if (rtcRoomService.getRoomRole(roomId, userId) <= RtcRoomRole.AUDIENCE) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限");
        }
        if (rtcRoomService.setTurntableConfig(roomId, turntableConfig)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取转盘配置
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getTurntableConfig")
    @Override
    public Result getTurntableConfig(@QueryParam("roomId") Long roomId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRoomService.getTurntableConfig(roomId));
        return mResult.build();
    }

    /**
     * 转盘抽奖
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("turntableLottery")
    @Override
    public Result turntableLottery(@FormParam("roomId") Long roomId) {
        // 判断开关是否开启
        if (!rtcRoomService.isBigAdventureEnabled(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "请先开启大冒险");
        }
        // 判断配置是否齐全
        if (!rtcRoomService.isTurntableConfig(roomId)) {
            return new Result(GlobalApiError.IncompleteConfiguration);
        }
        MutableResult mResult = new MutableResult();
        mResult.put("animationDuration", rtcRoomService.getTurntableAnimationDuration());
        mResult.put("lotteryResult", rtcRoomService.turntableLottery(roomId));
        return mResult.build();
    }

    /**
     * 开启房间内 PK
     *
     * @param roomId
     * @param topic
     * @param duration
     * @return
     */
    @POST
    @Path("startBattleRoomIn")
    @Override
    public Result startBattleRoomIn(@FormParam("roomId") Long roomId, @FormParam("topic") String topic, @FormParam("duration") long duration) {
        Long userId = this.getToken().getUserId();

        // 权限判定
        if (!rtcBattleService.isRoomBattleEnabled(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "当前房间类目未启用 PK 功能");
        }

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以开启 PK");
        }

        if (duration < 1) {
            return new Result(GlobalApiError.OperationFailed, "PK 时长不合法");
        }

        // 判断状态
        int battleStatus = rtcBattleService.getRoomInBattleStatus(roomId);
        if (BattleStatusEnum.Battling.getValue() == battleStatus) {
            return new Result(GlobalApiError.OperationFailed, "当前房间正在PK中，请先结束后再开启下一轮");
        }
        if (BattleStatusEnum.Ringing.getValue() == battleStatus) {
            return new Result(GlobalApiError.OperationFailed, "当前房间正在呼叫跨房PK，请稍后再试");
        }

        if (rtcBattleService.startRoomInBattle(roomId, topic, duration)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提前结束 PK
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("finishBattleRoomIn")
    @Override
    public Result finishBattleRoomIn(@FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        // 权限判定
        if (!rtcBattleService.isRoomBattleEnabled(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "当前房间类目未启用 PK 功能");
        }

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以结束 PK");
        }

        // 判断状态
        RtcBattleStatus battleStatus = rtcBattleService.getBattleStatus(roomId);
        if (battleStatus == null || BattleStatusEnum.Battling != battleStatus.getBattleStatus()) {
            return new Result(GlobalApiError.OperationFailed, "当前房间未在PK中");
        }

        if (RtcRoomBattleType.ROOM_IN != battleStatus.getBattleType()) {
            return new Result(GlobalApiError.OperationFailed, "当前未处于PK状态");
        }

        if (rtcBattleService.stopRoomInBattle(roomId, battleStatus.getBattleId())) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取 PK 记录
     *
     * @param roomId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getBattleRoomInHis")
    @Override
    public Result getBattleRoomInHis(@QueryParam("roomId") Long roomId,
                                     @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(rtcBattleService.getBattleRoomInHisList(roomId, pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取 跨房PK 房间列表
     *
     * @param pageIndex
     * @param pageSize
     * @param keyword
     * @return
     */
    @GET
    @Path("getBattleRoomCrossList")
    @Override
    public Result getBattleRoomCrossList(@QueryParam("pageIndex") Integer pageIndex,
                                         @QueryParam("pageSize") Integer pageSize,
                                         @QueryParam("keyword") String keyword,
                                         @QueryParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        if (roomId == null) {
            roomId = rtcUserService.getUserStayRoomId(userId);
        }

        // 判断是否在房间
        if (roomId == null) {
            if (isDev()) {
                log.info("获取pk房间列表失败,用户不在房间, userId: {}", userId);
            }
            return new Result(GlobalApiError.OperationFailed, "您目前不在房间");
        }

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以获取PK信息");
        }

        if (pageIndex == null || pageIndex < 1) {
            pageIndex = 1;
        }

        if (pageSize == null || pageSize < 1) {
            pageSize = GlobalConfig.PAGE_SIZE;
        }
        return new Result(rtcBattleService.getBattleRoomCrossList(roomId, keyword, pageIndex, pageSize));
    }

    /**
     * 获取 跨房PK 房间状态
     *
     * @return
     */
    @GET
    @Path("getBattleRoomCrossStatus")
    @Override
    public Result getBattleRoomCrossStatus(@QueryParam("roomId") Long roomId) {

        if (roomId == null) {
            return new Result(GlobalApiError.BadRequest, "roomId缺失");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("battleStatus", rtcBattleService.getBattleRoomCrossStatus(roomId));
        return mResult.build();
    }

    /**
     * 设置跨房PK 房间是否收听状态
     *
     * @return
     */
    @POST
    @Path("setBattleRoomCrossListenStatus")
    @Override
    public Result setBattleRoomCrossListenStatus(@FormParam("roomId") Long roomId,
                                                 @FormParam("enableListen") Integer enableListen) {

        if (roomId == null) {
            return new Result(GlobalApiError.BadRequest, "roomId缺失");
        }

        if (enableListen == null) {
            enableListen = 1;
        }

        if (isDev()) {
            log.info("setBattleRoomCrossListenStatus:roomId{},enableListen:{} ", roomId, enableListen);
        }

        rtcBattleService.setBattleRoomCrossListenStatus(roomId, enableListen);
        return Result.ofSuccess();
    }

    /**
     * 开启咵房 PK
     *
     * @param roomId
     * @param targetRoomId
     * @param topic
     * @param duration
     * @return
     */
    @POST
    @Path("startBattleRoomCross")
    @Override
    public Result startBattleRoomCross(@FormParam("roomId") Long roomId,
                                       @FormParam("targetRoomId") Long targetRoomId,
                                       @FormParam("topic") String topic,
                                       @FormParam("duration") long duration) {
        Long userId = this.getToken().getUserId();

        // 权限判定
        if (!rtcBattleService.isRoomBattleEnabled(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "当前房间类目未启用 PK 功能");
        }

        boolean isRandom = targetRoomId == null;

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以开启 PK");
        }

        if (duration < 1) {
            return new Result(GlobalApiError.OperationFailed, "PK 时长不合法");
        }

        // 判断状态
        int battleStatus = rtcBattleService.getRoomInBattleStatus(roomId);
        if (BattleStatusEnum.Battling.getValue() == battleStatus) {
            return new Result(GlobalApiError.OperationFailed, "当前房间正在PK中，请先结束后再开启下一轮");
        }
        if (BattleStatusEnum.Ringing.getValue() == battleStatus) {
            return new Result(GlobalApiError.OperationFailed, "当前房间正在呼叫跨房PK，请稍后再试");
        }

        if (!isRandom) {
            // 目标房间判定
            if (!rtcBattleService.isRoomBattleEnabled(targetRoomId)) {
                return new Result(GlobalApiError.OperationFailed, "目标房间所在类目未启用 PK 功能");
            }

            // 判断目标房间是不是允许pk
            RtcRoom targetRoom = rtcRoomService.getRoom(targetRoomId);
            if (Optional.ofNullable(targetRoom.getBattleMatchEnabled()).orElse(BoolIntVal.False).equals(BoolIntVal.False)) {
                return new Result(GlobalApiError.OperationFailed, "当前房间未启用 PK 功能");
            }

            int targetBattleStatus = rtcBattleService.getRoomInBattleStatus(targetRoomId);
            if (BattleStatusEnum.Battling.getValue() == targetBattleStatus) {
                return new Result(GlobalApiError.OperationFailed, "目标房间正在PK中，请先等对方结束后再试");
            }
            if (BattleStatusEnum.Ringing.getValue() == targetBattleStatus) {
                return new Result(GlobalApiError.OperationFailed, "目标房间正忙，请稍后再试");
            }
        } else {
            // 随机匹配对手
            targetRoomId = rtcBattleService.getRandomBattleRoomId(roomId);
            if (targetRoomId == null) {
                return new Result(GlobalApiError.OperationFailed, "当前没有空闲的对手，请稍后再试");
            }
        }

        if (rtcBattleService.startRoomCrossBattle(roomId, targetRoomId, topic, duration)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 取消跨房 PK
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("cancelBattleRoomCross")
    @Override
    public Result cancelBattleRoomCross(@FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以取消 PK");
        }

        // 判断状态
        int battleStatus = rtcBattleService.getRoomInBattleStatus(roomId);
        if (BattleStatusEnum.Ringing.getValue() != battleStatus) {
            return new Result(GlobalApiError.OperationFailed, "PK取消失败");
        }

        if (rtcBattleService.cancelRoomCrossBattle(roomId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 决定跨房 PK
     *
     * @param roomId
     * @param battleId
     * @param decision
     * @return
     */
    @POST
    @Path("decideBattleRoomCross")
    @Override
    public Result decideBattleRoomCross(@FormParam("roomId") Long roomId,
                                        @FormParam("battleId") Long battleId,
                                        @FormParam("decision") int decision) {
        Long userId = this.getToken().getUserId();

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以操作");
        }

        RtcBattleRoomCrossDO rtcBattleRoomCross = rtcBattleService.getBattleRoomCross(battleId);
        if (rtcBattleRoomCross == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到 PK 记录");
        }

        // 判断房间是否当前房间
        if (!roomId.equals(rtcBattleRoomCross.getTargetRoomId())) {
            return new Result(GlobalApiError.OperationFailed, "仅跨房PK请求接收方可以操作");
        }

        // 判断状态
        if (BattleStatusEnum.Ringing.getValue() != rtcBattleRoomCross.getBattleStatus()) {
            return new Result(GlobalApiError.OperationFailed, "当前非呼叫状态，无法操作");
        }

        if (rtcBattleService.decideRoomCrossBattle(battleId, decision)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提前结束 PK
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("finishBattleRoomCross")
    @Override
    public Result finishBattleRoomCross(@FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以结束 PK");
        }

        // 判断状态
        RtcBattleStatus battleStatus = rtcBattleService.getBattleStatus(roomId);
        if (battleStatus == null || BattleStatusEnum.Battling != battleStatus.getBattleStatus()) {
            return new Result(GlobalApiError.OperationFailed, "当前房间未在PK中");
        }

        if (RtcRoomBattleType.ROOM_CROSS != battleStatus.getBattleType()) {
            return new Result(GlobalApiError.OperationFailed, "当前未处于跨房PK状态");
        }

        if (rtcBattleService.stopRoomCrossBattle(roomId, battleStatus.getBattleId())) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取 PK 记录
     *
     * @param roomId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getBattleRoomCrossHis")
    @Override
    public Result getBattleRoomCrossHis(@QueryParam("roomId") Long roomId,
                                        @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(rtcBattleService.getBattleRoomCrossHisList(roomId, pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 设置跨房PK随机匹配开关
     *
     * @param roomId
     * @param enabled
     * @return
     */
    @POST
    @Path("setBattleCrossMatchEnabled")
    @Override
    public Result setBattleCrossMatchEnabled(@FormParam("roomId") Long roomId,
                                             @FormParam("enabled") int enabled) {
        Long userId = this.getToken().getUserId();

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(roomId, userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "仅主持人可以操作");
        }

        if (rtcBattleService.setBattleRoomCrossMatchEnabled(roomId, BoolIntVal.True == enabled)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 开播
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("startLive")
    @Override
    public Result startLive(@FormParam("roomId") Long roomId) {
        if (roomId == null) {
            return Result.ofBadRequest("roomId缺失");
        }
        RtcRoom room = rtcRoomService.getRoom(roomId);
        if (room == null) {
            return Result.ofOperationFailed("无此房间");
        }

        Long userId = this.getToken().getUserId();
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole == RtcRoomRole.AUDIENCE || roomRole == RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "开播需要管理员权限");
        }

        //更新room的状态es
        RtcRoomEditDTO dto = new RtcRoomEditDTO();
        dto.setRoomId(roomId);
        dto.setStatus(RtcRoomStatusEnum.OPEN.getValue());
        rtcRoomService.editRoom(dto);

        RtcLiveStatusMsg statusMsg = new RtcLiveStatusMsg();
        statusMsg.setStatus(BoolIntVal.True);
        statusMsg.setRoomId(roomId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, statusMsg);

        return new Result();
    }

    /**
     * 关播
     */
    @POST
    @Path("stopLive")
    @Override
    public Result stopLive(@FormParam("roomId") Long roomId) {
        if (roomId == null) {
            return Result.ofBadRequest("roomId缺失");
        }

        RtcRoom room = rtcRoomService.getRoom(roomId);
        if (room == null) {
            return Result.ofOperationFailed("无此房间");
        }
        Long userId = this.getToken().getUserId();
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole == RtcRoomRole.AUDIENCE || roomRole == RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "关播需要管理员权限");
        }

        // 将rtc房间状态改为关播； 同步es搜索信息
        RtcRoomEditDTO editDTO = new RtcRoomEditDTO();
        editDTO.setStatus(RtcRoomStatusEnum.CLOSE.getValue());
        editDTO.setRoomId(roomId);
        rtcRoomService.editRoom(editDTO);

        RtcLiveStatusMsg statusMsg = new RtcLiveStatusMsg();
        statusMsg.setStatus(BoolIntVal.False);
        statusMsg.setRoomId(roomId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, statusMsg);

        return new Result();
    }

    /**
     * 获取用户房间历史
     *
     * @return
     */
    @Override
    @GET
    @Path("getRtcRoomHisList")
    public Result getRtcRoomHisList() {
        Long userId = this.getToken().getUserId();
        List<RtcRoomVisitHisVO> voList = rtcRoomService.getRtcRoomHisList(userId);
        return Result.ofList(voList);
    }

    /**
     * 获取用户房间历史分页
     *
     * @return
     */
    @Override
    @GET
    @Path("getRtcRoomHisPageList")
    public Result getRtcRoomHisPageList(@QueryParam("pageIndex") Integer pageIndex,
                                        @QueryParam("pageSize") Integer pageSize) {
        Long userId = this.getToken().getUserId();

        if (pageIndex == null) {
            pageIndex = 1;
        }

        if (pageSize == null) {
            pageSize = GlobalConfig.PAGE_SIZE;
        }

        return Result.ofSuccess(rtcRoomService.getRtcRoomHisPageList(userId, pageIndex, pageSize));
    }

    /**
     * 房间底部菜单入口
     *
     * @return
     */
    @GET
    @Path("getBottomMenuEntry")
    @Override
    public Result getBottomMenuEntry() {
        String rtcRecharge = releaseConfigService.getCachReleaseConfig("rtc_recharge");
        if (StringUtils.isBlank(rtcRecharge)) {
            return new Result(new ArrayList<>());
        }
        //后续接口输出可扩展（加tag），排序问题后续有需要再处理
        boolean hasRecharged = userService.hasRecharged(this.getToken().getUserId());
        String configKey = hasRecharged ? "daily" : "first";
        JSONObject rechargeConfig = JSONObject.parseObject(rtcRecharge);
        if (rechargeConfig.containsKey(configKey)) {
            JSONObject node = rechargeConfig.getJSONObject(configKey);
            MutableResult mr = new MutableResult();
            mr.put("order", 1);
            mr.put("path", node.getString("path"));
            mr.put("img", node.getString("img"));
            mr.put("action", node.getIntValue("action"));

            return new Result(Arrays.asList(mr));
        }

        return new Result(new ArrayList<>());
    }
}
