package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.app.config.UpgradeConfig;
import com.starbuds.server.api.app.util.RtcUtil;
import com.starbuds.server.api.app.v1.facade.RtcRoomAuctionFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcAuctionSeatNo;
import com.starbuds.server.common.constant.rtc.RtcAuctionSeatType;
import com.starbuds.server.common.constant.rtc.RtcRoomRole;
import com.starbuds.server.common.exception.rtc.RtcAuctionRoomSetSitException;
import com.starbuds.server.common.exception.rtc.RtcEndAuctionException;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.WatchConfigDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.rtc.RtcQueueUser;
import com.starbuds.server.common.pojo.rtc.RtcRoomUser;
import com.starbuds.server.common.pojo.rtc.RtcSeat;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.rtc.RtcQueueService;
import com.starbuds.server.common.service.rtc.RtcRoomAuctionService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserWatchRelationService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * RTC 房间拍卖接口实现
 *
 * @author aoe
 * @date 2021/7/27
 */
@Path("/v1/rtc")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RtcRoomAuctionFacadeImpl extends BaseFacade implements RtcRoomAuctionFacade {

    @Reference(version = "1.0.0")
    private RtcRoomAuctionService rtcRoomAuctionService;

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    private RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private UserWatchRelationService userWatchRelationService;

    @Reference(version = "1.0.0")
    RtcQueueService rtcQueueService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Resource
    private RtcUtil rtcUtil;

    @Resource
    private UpgradeConfig upgradeConfig;

    /**
     * 获取拍卖房起拍设置的相关数据
     *
     * @return
     */
    @GET
    @Path("getStartAuctionSettingData")
    @Override
    public Result getStartAuctionSettingData() {
        return new Result(rtcRoomAuctionService.getStartAuctionSettingData());
    }

    /**
     * 设置拍卖信息(设置成功后竞拍开始)
     *
     * @param roomId        拍卖房id
     * @param watchConfigId 拍卖物品（守护）id
     * @param watchDuration 守护时长
     * @param startGiftId   起拍礼物id
     * @return
     */
    @POST
    @Path("setAuction")
    @Override
    public Result setAuction(@FormParam("roomId") Long roomId,
                             @FormParam("watchConfigId") Long watchConfigId,
                             @FormParam("watchDuration") Integer watchDuration,
                             @FormParam("startGiftId") Long startGiftId) {
        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        // 竞拍未结束，无法更改拍卖对象
        if (rtcRoomAuctionService.isBidding(roomId)) {
            return new Result(GlobalApiError.ServerError, "竞拍未结束，无法更改拍卖对象");
        }

        // 获取守护类型信息
        WatchConfigDO watchConfig = userWatchRelationService.getWatchConfig(watchConfigId);
        if (watchConfig == null) {
            return new Result(GlobalApiError.BadRequest, "守护类型不存在");
        }
        String watchName = watchConfig.getName();
        if (StringUtils.isBlank(watchName)) {
            return new Result(GlobalApiError.BadRequest, "守护类型名称不存在");
        }

        // 获取礼物信息
        GiftItemVO gift = giftService.getGift(startGiftId);
        if (gift == null) {
            return new Result(GlobalApiError.BadRequest, "起拍礼物不存在");
        }
        String giftName = gift.getGiftName();
        if (StringUtils.isBlank(giftName)) {
            return new Result(GlobalApiError.BadRequest, "起拍礼物名称不存在");
        }
        Long giftPrice = gift.getGiftPrice();
        if (giftPrice == null) {
            return new Result(GlobalApiError.BadRequest, "无法获取起拍礼物价格");
        }

        Long userId = this.getToken().getUserId();
        try {
            rtcRoomAuctionService.setAuction(userId, roomId,
                    watchConfigId, watchDuration, watchName,
                    startGiftId, giftName, giftPrice, gift.getGiftIcon());
        } catch (RtcAuctionRoomSetSitException e) {
            log.error(String.format("设置拍卖信息失败 userId = %s roomId = %s watchConfigId = %s startGiftId = %s",
                    userId, roomId, watchConfigId, watchDuration, startGiftId), e);
            return new Result(GlobalApiError.ServerError, e.getMessage());
        }
        rtcRoomAuctionService.removeApplyAuctionSeatUsers(roomId, userId);
        return new Result();
    }

    /**
     * 结束竞拍
     *
     * @param roomId
     * @return
     */
    @POST
    @Path("endAuction")
    @Override
    public Result endAuction(@FormParam("roomId") Long roomId) {
        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }

        Long userId = this.getToken().getUserId();
        // 判断角色是否主持
        if (rtcRoomService.getRoomRole(roomId, userId) < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "您没有主持权限");
        }
        long auctionUserId = rtcRoomAuctionService.getAuctionUserId(roomId);
        if (auctionUserId < 1) {
            return new Result(GlobalApiError.OperationFailed, "竞拍对象不存在，无法结束竞拍");
        }
        try {
            rtcRoomAuctionService.endAuction(roomId, getHeader().getAppVersion());
        } catch (RtcEndAuctionException e) {
            log.error(String.format("结束竞拍 操作人userId = %s roomId = %s",
                    userId, roomId), e);
            return new Result(GlobalApiError.ServerError, e.getMessage());
        }
        return new Result();
    }

    /**
     * 申请上麦-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    @POST
    @Path("sitSeatInAuction")
    @Override
    public Result sitSeatInAuction(@FormParam("roomId") Long roomId, @FormParam("type") Integer type) {
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        check = checkSeatType(type);
        if (check != null) {
            return check;
        }

        List<RtcSeat> seats = rtcSeatService.getSeats(roomId);
        if (CollectionUtils.isEmpty(seats)) {
            return new Result(GlobalApiError.OperationFailed, "获取麦位列表信息失败");
        }
        Long userId = getToken().getUserId();
        int seatNo;
        boolean isBidSeat = type == RtcAuctionSeatType.BID;
        if (isDev()) {
            log.info("申请上麦-拍卖房 isBidSeat = {} userId = {} roomId = {} type = {}",
                    isBidSeat, userId, roomId, type);
        }
        if (isBidSeat) {
            // 申请竞拍麦位
            /*seatNo = getCanUseSeatNo(seats, RtcAuctionSeatNo.AUCTION_USER);
            if (rtcRoomAuctionService.isBidding(roomId)) {
                // 竞拍中
                if (userId - rtcRoomAuctionService.getAuctionUserId(roomId) != 0) {
                    // 自己不是被竞拍对象
                    return new Result(GlobalApiError.ServerError, "竞拍未结束，无法更改拍卖对象");
                }
            }*/
            seatNo = RtcAuctionSeatNo.AUCTION_USER;
        } else {
            // 申请普通麦位
//            seatNo = getCanUseSeatNo(seats, RtcAuctionSeatNo.NORMAL);
            seatNo = RtcAuctionSeatNo.NORMAL[0];
        }
        /*if (seatNo < RtcAuctionSeatNo.AUCTION_USER) {
            return new Result(GlobalApiError.OperationFailed, "没有空余的唛位");
        }*/

        if (isDev()) {
            log.info("申请上麦-拍卖房 seatNo = {} userId = {} roomId = {} type = {}",
                    seatNo, userId, roomId, type);
        }

        Result result = rtcUtil.sitSeat(userId, roomId, seatNo, getHeader().getAppVersion());
        if (!result.isSuccess()) {
            return result;
        }
        MutableResult mResult = (MutableResult) result.getData();
        // 加入麦序成功
        if (BooleanUtils.toBoolean(mResult.get("queue") + "")) {
            if (isBidSeat) {
                if (isDev()) {
                    log.info("申请上麦-拍卖房 添加申请上竞拍麦的用户 seatNo = {} userId = {} roomId = {} type = {}",
                            seatNo, userId, roomId, type);
                }
                rtcRoomAuctionService.saveApplyAuctionSeatUser(roomId, userId);
            }
        } /*else {
            // 上麦成功
        }*/
        return result;
    }

    /**
     * 获取麦序用户列表-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    @GET
    @Path("getQueueUsersInAuction")
    @Override
    public Result getQueueUsersInAuction(@QueryParam("roomId") Long roomId, @QueryParam("type") Integer type) {
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        check = checkSeatType(type);
        if (check != null) {
            return check;
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRoomAuctionService.getQueueUsersInAuction(roomId, type));
        return mResult.build();
    }

    /**
     * 麦序移除-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @param userId 用户ID为空, 移除所有
     */
    @POST
    @Path("removeQueueUserInAuction")
    @Override
    public Result removeQueueUserInAuction(@FormParam("roomId") Long roomId, @FormParam("type") Integer type, @FormParam("userId") Long userId) {
        Long myUserId = this.getToken().getUserId();
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        check = checkSeatType(type);
        if (check != null) {
            return check;
        }
        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }
        if (userId == null) {
            // 取出普通、竞拍麦位的用户，如果是全部删除，遍历取用户id，调用老方法
            List<RtcQueueUser> rtcQueueUserList = rtcRoomAuctionService.getQueueUsersInAuction(roomId, type);
            if (CollectionUtil.isNotEmpty(rtcQueueUserList)) {
                rtcQueueUserList.forEach(i -> {
                    rtcUtil.doRemoveQueueUserHandler(i.getUserId(), roomId);
                    rtcRoomAuctionService.removeApplyAuctionSeatUsers(roomId, userId);
                });
            }
        } else {
            rtcUtil.doRemoveQueueUserHandler(userId, roomId);
            if (type == RtcAuctionSeatType.BID) {
                rtcRoomAuctionService.removeApplyAuctionSeatUsers(roomId, userId);
            }
        }
        return new Result();
    }


    /**
     * 抱用户上麦-拍卖房
     *
     * @param roomId
     * @param userId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    @POST
    @Path("setUserToSeatInAuction")
    @Override
    public Result setUserToSeatInAuction(@FormParam("roomId") Long roomId,
                                         @FormParam("userId") Long userId,
                                         @FormParam("type") Integer type) {
        Long myUserId = this.getToken().getUserId();
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        check = checkSeatType(type);
        if (check != null) {
            return check;
        }

        List<RtcSeat> seats = rtcSeatService.getSeats(roomId);
        Integer wantSeat;
        // 竞拍麦位
        if (type == RtcAuctionSeatType.BID) {
            // 竞拍中不能更换麦位的用户
            Result checkUseBidMai = checkUseBidMai(roomId, userId);
            if (checkUseBidMai != null) {
                return checkUseBidMai;
            }
            wantSeat = getCanUseSeatNo(seats, RtcAuctionSeatNo.AUCTION_USER);
        } else if (type == RtcAuctionSeatType.NORMAL) {
            wantSeat = getCanUseSeatNo(seats, RtcAuctionSeatNo.NORMAL);
        } else {
            wantSeat = getCanUseSeatNo(seats, RtcAuctionSeatNo.GUEST);
        }

        Result checkResult = rtcUtil.checkSetUserToSet(roomId, myUserId, userId, wantSeat);
        if (!checkResult.isSuccess()) {
            return checkResult;
        }
        if (wantSeat != -1) {
            // 跳锁上麦
            int seat = rtcSeatService.sitSeat(roomId, userId, wantSeat, true, getHeader().getAppVersion());
            if (seat > 0) {
                rtcRoomAuctionService.removeApplyAuctionSeatUsers(roomId, userId);
                MutableResult mResult = new MutableResult();
                mResult.put("seatNo", seat);
                return mResult.build();
            }
        }
        return new Result(GlobalApiError.OperationFailed, "当前没有空位无法上麦");
    }

    /**
     * 离开麦序队列-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    @POST
    @Path("leaveQueueInAuction")
    @Override
    public Result leaveQueueInAuction(@FormParam("roomId") Long roomId, @FormParam("type") Integer type) {
        Long userId = this.getToken().getUserId();
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        check = checkSeatType(type);
        if (check != null) {
            return check;
        }

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }
        if (rtcQueueService.isUserInQueue(roomId, userId)) {
            rtcQueueService.removeQueueUser(roomId, userId);
            rtcRoomAuctionService.removeApplyAuctionSeatUsers(roomId, userId);
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 快速上麦-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    @POST
    @Path("setUserToSeatInQueueInAuction")
    @Override
    public Result setUserToSeatInQueueInAuction(@FormParam("roomId") Long roomId, @FormParam("type") Integer type) {
        Long myUserId = this.getToken().getUserId();
        Result check = checkRoom(roomId);
        if (check != null) {
            return check;
        }
        check = checkSeatType(type);
        if (check != null) {
            return check;
        }

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }
        // 判断用户是否在房间
        if (!rtcUserService.isUserInRoom(roomId, myUserId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户不在房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 自动上麦
        List<RtcQueueUser> queueUsers = rtcRoomAuctionService.getQueueUsersInAuction(roomId, type);
        if (CollectionUtils.isEmpty(queueUsers)) {
            if (isDev()) {
                log.info("拍卖房自动上麦失败，队列中没有用户");
            }
            return new Result();
        }

        queueUsers.stream()
                .filter(Objects::nonNull)
                .map(user -> user.getUserId())
                .filter(Objects::nonNull)
                .forEach(userId -> {
                    List<RtcSeat> seats = rtcSeatService.getSeats(roomId);
                    int seatNo = -1;
                    if (RtcAuctionSeatType.BID == type) {
                        // 竞拍中不能更换麦位的用户
                        Result checkUseBidMai = checkUseBidMai(roomId, userId);
                        if (checkUseBidMai != null) {
                            return;
                        }
                        seatNo = getCanUseSeatNo(seats, RtcAuctionSeatNo.AUCTION_USER);
                    } else if (RtcAuctionSeatType.NORMAL == type) {
                        seatNo = getCanUseSeatNo(seats, RtcAuctionSeatNo.NORMAL);
                    } else {
                        return;
                    }
                    // 没有空余麦位
                    if (seatNo == -1) {
                        return;
                    }
                    if (isDev()) {
                        log.info("自动上麦成功 roomId = {} type = {} seatNo = {} userId = {}", roomId, type, seatNo, userId);
                    }
                    int seat = rtcSeatService.sitSeat(roomId, userId, seatNo, true, getHeader().getAppVersion());
                    if (seat > 0) {
                        rtcRoomAuctionService.removeApplyAuctionSeatUsers(roomId, userId);
                    }
                });

        return new Result();
    }

    /**
     * 可以使用的麦位序号
     *
     * @param seats   麦位信息
     * @param seatNos 期望使用的麦位
     * @return 不是-1就是正常
     */
    private int getCanUseSeatNo(List<RtcSeat> seats, int... seatNos) {
        return Arrays.stream(seatNos)
                .filter(seatNo -> {
                    for (int i = 0; i < seats.size(); i++) {
                        RtcSeat rtcSeat = seats.get(i);
                        RtcRoomUser user = null;
                        if (rtcSeat != null) {
                            user = rtcSeat.getUser();
                        }
                        if (user != null) {
                            continue;
                        }
                        if (seatNo - i == 0) {
                            return true;
                        }
                    }
                    return false;
                })
                .findAny().orElse(-1);
    }

    /**
     * 检查房间
     *
     * @param roomId
     * @return null 正常；错误提示
     */
    private Result checkRoom(Long roomId) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }
        if (RoomTypeEnum.AUCTION.getValue() - rtcRoomService.getRoomType(roomId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "房间类型不是拍卖房，无法操作");
        }
        return null;
    }

    /**
     * 检查麦位类型
     *
     * @param type
     * @return null 正常；错误提示
     */
    private Result checkSeatType(Integer type) {
        if (type == null) {
            return new Result(GlobalApiError.OperationFailed, "麦位类不能为空");
        }
        if (type != RtcAuctionSeatType.BID && type != RtcAuctionSeatType.NORMAL
                && type != RtcAuctionSeatType.GUEST) {
            return new Result(GlobalApiError.OperationFailed, "麦位类型有误");
        }
        return null;
    }

    /**
     * 检查是否可以上竞拍麦
     *
     * @param roomId
     * @param userId
     * @return
     */
    private Result checkUseBidMai(Long roomId, Long userId) {
        // 竞拍中不能更换麦位的用户
        if (rtcRoomAuctionService.isBidding(roomId)
                && rtcRoomAuctionService.getAuctionUserId(roomId) - userId != 0) {
            return new Result(GlobalApiError.ServerError, "竞拍未结束，无法更改拍卖对象");
        }
        return null;
    }
}
