package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.app.pojo.rtc.RtcSkillOrderHallVO;
import com.starbuds.server.api.app.v1.facade.RtcSkillOrderFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcSkillOrderEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.SkillVO;
import com.starbuds.server.common.pojo.db.RtcSkillOrderDO;
import com.starbuds.server.common.pojo.db.SkillDO;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.rtc.qo.RtcSkillOrderPageListQO;
import com.starbuds.server.common.pojo.rtc.vo.RtcSkillOrderVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.rtc.RtcSkillOrderService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import javax.ws.rs.*;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 派单实现
 *
 * @author laoliu
 * @date 2021/8/17 15:12
 * @description
 */
@Path("/v1/rtc/paidan/")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RtcSkillOrderFacadeImpl extends BaseFacade implements RtcSkillOrderFacade {

    /**
     * 连击间隔时间(毫秒)
     */
    private static final long DOUBLE_CLICK_INTERVAL_MILLS = 1500;

    /**
     * 连击提示信息
     */
    private static final String DOUBLE_CLICK_MSG = "请勿重复提交";


    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    RtcSkillOrderService rtcSkillOrderService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    UserService userService;

    /**
     * 获取服务技能
     *
     * @return
     */
    @GET
    @Path("getSkills")
    @Override
    public Result getSkills() {
        List<SkillVO> skills = skillService.getSkills()
                .stream()
                .filter(Objects::nonNull)
                .filter(i -> i.getVoiceMatchEnable() == BoolIntVal.False)
                .collect(Collectors.toList());

        MutableResult mResult = new MutableResult();
        mResult.put("list", skills);
        return mResult.build();
    }

    /**
     * 提交派单
     *
     * @return
     */
    @POST
    @Path("addOrder")
    @Override
    public Result addOrder(@FormParam("skillId") Long skillId,
                           @FormParam("roomId") Long roomId,
                           @FormParam("gender") Integer gender,
                           @FormParam("price") Long price,
                           @FormParam("quantity") Integer quantity,
                           @FormParam("desc") String desc,
                           @FormParam("bossUserId") Long bossUserId) {

        if (skillId == null) {
            return new Result(GlobalApiError.OperationFailed, "skillId缺失");
        }

        if (roomId == null) {
            return new Result(GlobalApiError.OperationFailed, "roomId缺失");
        }

        if (gender == null) {
            return new Result(GlobalApiError.OperationFailed, "gender缺失");
        }

        if (price == null) {
            return new Result(GlobalApiError.OperationFailed, "price缺失");
        }

        if (quantity == null) {
            return new Result(GlobalApiError.OperationFailed, "quantity缺失");
        }

        if (desc == null) {
            return new Result(GlobalApiError.OperationFailed, "desc缺失");
        }

        if (bossUserId == null) {
            return new Result(GlobalApiError.OperationFailed, "bossUserId缺失");
        }

        Long userId = this.getToken().getUserId();

        String key = String.format("%s:paidan:addOrder:%d:%d", userId, skillId, roomId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }

        SkillDO skillDO = skillService.getSkillDetail(skillId);
        if (skillDO == null) {
            return new Result(GlobalApiError.OperationFailed, "无此服务技能");
        }

        RtcRoom room = rtcRoomService.getRoom(roomId);

        if (room == null) {
            return new Result(GlobalApiError.OperationFailed, "房间不存在");
        }

        RoomTypeEnum roomTypeEnum = RoomTypeEnum.toEnum(room.getRoomType());
        if (!RoomTypeEnum.ORDER.equals(roomTypeEnum)) {
            return new Result(GlobalApiError.OperationFailed, "只有派单房可以派单哦");
        }

        if (!rtcSkillOrderService.createOrder(skillDO, roomId, gender, price, quantity, desc, bossUserId, userId)) {
            return new Result(GlobalApiError.OperationFailed);
        }

        return new Result();
    }

    /**
     * 接单大厅数量
     *
     * @return
     */
    @GET
    @Path("getOrderListCount")
    @Override
    public Result getOrderListCount() {

        MutableResult mResult = new MutableResult();
        Long userId = this.getToken().getUserId();

        RtcSkillOrderDO lastOrderCreated = rtcSkillOrderService.getLastOrderCreated();

        String bossUserName = userService.getUserName(lastOrderCreated.getBossUserId());

        mResult.put("count", rtcSkillOrderService.getAllOrderCreatedCount(userId));
        mResult.put("desc", bossUserName + " 发布了 " + lastOrderCreated.getSkillName() + " 的派单需求");
        mResult.put("timestamp", System.currentTimeMillis());
        return mResult.build();
    }

    /**
     * 接单大厅列表
     *
     * @return
     */
    @GET
    @Path("getOrderList")
    @Override
    public Result getOrderList(@BeanParam RtcSkillOrderPageListQO pageListQO) {

        Long userId = this.getToken().getUserId();

        // 查询待接单，已完成的订单
        PageList<RtcSkillOrderDO> pageList = rtcSkillOrderService.getListInStatus(pageListQO);
        PageList<RtcSkillOrderHallVO> pageResultList = new PageList<>();
        List<RtcSkillOrderHallVO> list = new ArrayList<>();

        Set<Long> ids = pageList.getList().stream().map(RtcSkillOrderDO::getRoomId).collect(Collectors.toSet());

        Map<Long, RtcRoom> roomMap = rtcRoomService.getRoomMap(ids);
        pageList.getList().forEach(e -> {
            RtcSkillOrderHallVO hallVO = new RtcSkillOrderHallVO();
            BeanUtils.copyProperties(e, hallVO);
            hallVO.setRoomName(Optional.ofNullable(roomMap.get(e.getRoomId())).orElse(new RtcRoom()).getRoomName());
            list.add(hallVO);
        });

        BeanUtils.copyProperties(pageList, pageResultList);
        pageResultList.setList(list);

        // 已读功能添加
        // 筛选出待接单的订单
        List<Long> waitOrderIds = pageList.getList().stream()
                .filter(Objects::nonNull)
                .filter(val -> val.getStatus().equals(RtcSkillOrderEnum.WAIT.getValue()))
                .map(RtcSkillOrderDO::getOrderId)
                .collect(Collectors.toList());

        if (CollectionUtil.isNotEmpty(waitOrderIds)) {
            // 将这些订单添加已读表中
            rtcSkillOrderService.addReadHis(userId, waitOrderIds);
        }

        // 重新发送用户未读消息数，归零
        rtcSkillOrderService.resetUnReadCount(userId);
        return new Result(pageResultList);
    }

    /**
     * 在线派单列表
     *
     * @return
     */
    @GET
    @Path("getOrderListInRoom")
    @Override
    public Result getOrderListInRoom(@BeanParam RtcSkillOrderPageListQO pageListQO) {

        if (pageListQO.getRoomId() == null) {
            return new Result(GlobalApiError.OperationFailed, "roomId缺失");
        }

        return new Result(rtcSkillOrderService.getOrderListInRoom(pageListQO));
    }

    /**
     * 获取接单用户列表
     *
     * @return
     */
    @GET
    @Path("getServiceUserList")
    @Override
    public Result getServiceUserList(@QueryParam("roomId") Long roomId,
                                     @QueryParam("orderId") Long orderId) {

        if (orderId == null) {
            return new Result(GlobalApiError.OperationFailed, "orderId缺失");
        }

        if (roomId == null) {
            return new Result(GlobalApiError.OperationFailed, "roomId缺失");
        }

        RtcSkillOrderDO order = rtcSkillOrderService.getById(orderId);

        if (order == null) {
            return new Result(GlobalApiError.OperationFailed, "无此订单");
        }

        return new Result(rtcSkillOrderService.getServiceUserList(order, roomId));
    }

    /**
     * 结束派单
     *
     * @return
     */
    @POST
    @Path("finishOrder")
    @Override
    public Result finishOrder(@FormParam("serviceUserId") Long serviceUserId,
                              @FormParam("orderId") Long orderId) {

        if (serviceUserId == null) {
            return new Result(GlobalApiError.OperationFailed, "serviceUserId缺失");
        }

        if (orderId == null) {
            return new Result(GlobalApiError.OperationFailed, "orderId缺失");
        }

        String key = String.format("%s:paidan:finishOrder:%d", orderId, serviceUserId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }

        RtcSkillOrderDO order = rtcSkillOrderService.getById(orderId);

        if (order == null) {
            return new Result(GlobalApiError.OperationFailed, "无此订单");
        }

        RtcSkillOrderEnum orderStatusEnum = RtcSkillOrderEnum.toEnum(order.getStatus());
        if (!orderStatusEnum.equals(RtcSkillOrderEnum.WAIT)) {

            if (orderStatusEnum.equals(RtcSkillOrderEnum.CLOSE)) {
                return new Result(GlobalApiError.OperationFailed, "此订单已被关闭啦");
            }

            if (orderStatusEnum.equals(RtcSkillOrderEnum.FINISH)) {
                return new Result(GlobalApiError.OperationFailed, "此订单已被接单啦");
            }

            return new Result(GlobalApiError.OperationFailed, "订单此时不允许改变");
        }

        if (order.getBossUserId().equals(serviceUserId)) {
            return new Result(GlobalApiError.OperationFailed, "接单用户不能与下单用户相同");
        }

        if (!familyService.isSkillAuth(order.getSkillId(), serviceUserId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户无服务技能");
        }

        if (rtcSkillOrderService.finishOrder(serviceUserId, orderId)) {
            return new Result();
        } else {
            return new Result(GlobalApiError.OperationFailed);
        }

    }

    /**
     * 订单详情
     *
     * @return
     */
    @GET
    @Path("getOrderDetails")
    @Override
    public Result getOrderDetails(@QueryParam("orderId") Long orderId) {

        if (orderId == null) {
            return new Result(GlobalApiError.OperationFailed, "orderId缺失");
        }

        RtcSkillOrderDO order = rtcSkillOrderService.getById(orderId);

        if (order == null) {
            return new Result(GlobalApiError.OperationFailed, "无此订单");
        }

        RtcSkillOrderVO orderVO = rtcSkillOrderService.getOrderDetails(order);
        // 需要房间短id
        RtcRoom rtcRoom = rtcRoomService.getRoom(order.getRoomId());

        if (rtcRoom == null) {
            return new Result(GlobalApiError.OperationFailed, "房间不存在");
        }

        orderVO.setRoomNo(rtcRoom.getRoomNo());

        return new Result(orderVO);
    }

    /**
     * 获取房间派单记录
     *
     * @return
     */
    @GET
    @Path("getHistoryListInRoom")
    @Override
    public Result getHistoryListInRoom(@BeanParam RtcSkillOrderPageListQO pageListQO) {

        if (pageListQO.getRoomId() == null) {
            return new Result(GlobalApiError.OperationFailed, "roomId缺失");
        }

        return new Result(rtcSkillOrderService.getHistoryListInRoom(pageListQO));
    }

    /**
     * 获取个人派单记录
     *
     * @return
     */
    @GET
    @Path("getHistoryList")
    @Override
    public Result getHistoryList(@BeanParam RtcSkillOrderPageListQO pageListQO) {
        Long userId = this.getToken().getUserId();

        return new Result(rtcSkillOrderService.getHistoryList(userId, pageListQO));
    }

    /**
     * 关闭派单
     *
     * @return
     */
    @POST
    @Path("closeOrder")
    @Override
    public Result closeOrder(@FormParam("orderId") Long orderId) {

        if (orderId == null) {
            return new Result(GlobalApiError.OperationFailed, "orderId缺失");
        }

        String key = String.format("%s:paidan:closeOrder", orderId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }

        Long userId = this.getToken().getUserId();

        RtcSkillOrderDO order = rtcSkillOrderService.getById(orderId);

        if (order == null) {
            return new Result(GlobalApiError.OperationFailed, "无此订单");
        }

        RtcSkillOrderEnum orderStatusEnum = RtcSkillOrderEnum.toEnum(order.getStatus());
        if (!orderStatusEnum.equals(RtcSkillOrderEnum.WAIT)) {

            if (orderStatusEnum.equals(RtcSkillOrderEnum.CLOSE)) {
                return new Result(GlobalApiError.OperationFailed, "此订单已被关闭啦");
            }

            if (orderStatusEnum.equals(RtcSkillOrderEnum.FINISH)) {
                return new Result(GlobalApiError.OperationFailed, "此订单已被接单啦");
            }

            return new Result(GlobalApiError.OperationFailed, "订单此时不允许改变");
        }

        // 判断是否主持人
        if (rtcSeatService.getUserSeatNo(order.getRoomId(), userId) != 0) {
            return new Result(GlobalApiError.OperationFailed, "您不是主持人，没有修改权限");
        }

        if (rtcSkillOrderService.closeOrder(orderId)) {
            return new Result();
        } else {
            return new Result(GlobalApiError.OperationFailed);
        }

    }

    /**
     * 获取房间内订单数量
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getOrderInRoomCount")
    @Override
    public Result getOrderInRoomCount(@QueryParam("roomId") Long roomId) {

        if (roomId == null) {
            return new Result(GlobalApiError.OperationFailed, "roomId缺失");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("count", rtcSkillOrderService.getOrderInRoomCount(roomId));
        return mResult.build();
    }
}
