package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.provider.PeopleNearbyProvider;
import com.starbuds.server.api.app.util.PinyinUtil;
import com.starbuds.server.api.app.v1.facade.SocialFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.user.*;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.pojo.data.NearUserDTO;
import com.starbuds.server.common.pojo.db.FriendReqDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.ws.rs.*;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * 社交接口实现
 *
 * @author jiangxiaowei
 * @date 2019/12/20 17:42
 */
@Path("/v1/social")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class SocialFacadeImpl extends BaseFacade implements SocialFacade {

    private static final long DOUBLE_CLICK_INTERVAL_MILLS = 1000;
    private static final String DOUBLE_CLICK_MSG = "请勿重复提交";

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    private PeopleNearbyProvider peopleNearbyProvider;

    @Reference(version = "1.0.0")
    private AreaService areaService;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 关注用户
     *
     * @param targetId 用户id
     * @return result对象
     */
    @POST
    @Path("followUser")
    @Override
    public Result followUser(@FormParam("targetId") Long targetId) {

        //是否开启关注功能
        if (!releaseConfigService.getReleaseConfig("enable_allow").equals("true")) {
            return new Result(GlobalApiError.OperationFailed, "功能维护中");
        }

        if (!userService.isUserExist(targetId)) {
            return new Result(GlobalApiError.OperationFailed, "目标用户不存在");
        }

        Long userId = this.getToken().getUserId();
        if (userId.equals(targetId)) {
            return new Result(GlobalApiError.OperationFailed, "不能关注自己哦");
        }

        String key = String.format("%s:followUser:%d", userId, targetId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }

        // 关注限制判定
        if (releaseConfigService.getReleaseConfig("enable_follow_limit").equals("true")) {
            if (socialRelationService.isFollowLimit(userId)) {
                return new Result(GlobalApiError.OperationFailed, "操作过于频繁，请稍后再试");
            }
        }

        if (socialRelationService.followUser(userId, targetId)) {

            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "关注失败");
    }

    /**
     * 取消关注
     *
     * @param targetId 用户id
     * @return result对象
     */
    @POST
    @Path("unFollowUser")
    @Override
    public Result unFollowUser(@FormParam("targetId") Long targetId) {


        if (!releaseConfigService.getReleaseConfig("enable_allow").equals("true")) {
            return new Result(GlobalApiError.OperationFailed, "功能维护中");
        }

        if (!userService.isUserExist(targetId)) {
            return new Result(GlobalApiError.OperationFailed, "用户不存在");
        }
        long userId = this.getToken().getUserId();

        String key = String.format("%s:unFollowUser:%d", userId, targetId);
        if (!redisService.tryLock(key, "", DOUBLE_CLICK_INTERVAL_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, DOUBLE_CLICK_MSG);
        }

        if (socialRelationService.unFollowUser(userId, targetId)) {

            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 发起添加好友请求
     *
     * @param targetId 用户id
     * @param reqMsg   添加好友时的验证消息
     * @return result对象
     */
    @POST
    @Path("addFriendReq")
    @Override
    public Result addFriendReq(@FormParam("targetId") Long targetId,
                               @FormParam("reqMsg") String reqMsg) {
        // 目标用户是否存在
        if (!userService.isUserExist(targetId)) {
            return new Result(GlobalApiError.OperationFailed, "用户不存在");
        }

        Long userId = this.getToken().getUserId();
        if (userId.equals(targetId)) {
            return new Result(GlobalApiError.OperationFailed, "不能加自己为好友哦");
        }

        // 判断好友状态
        Integer friendStatus = socialRelationService.getFriendStatus(userId, targetId);
        FriendStatusEnum friendStatusEnum = FriendStatusEnum.toEnum(friendStatus);

        switch (friendStatusEnum) {
            case ADD_FRIEND:
                break;
            case ALREADY_APPLY:
                return new Result(GlobalApiError.ServerError, "已申请，等待同意");
            case ALREADY_ADD:
                return new Result(GlobalApiError.ServerError, "已是好友");
            default:
                break;
        }

//        // 是否已经是好友
//        if (socialRelationService.isFriend(userId, targetId)) {
//            return new Result(GlobalApiError.OperationFailed, "你们已经是好友了~");
//        }

        if (socialRelationService.addFriendReq(this.getToken().getUserId(), targetId, reqMsg)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 同意好友请求
     *
     * @param reqId  好友请求id
     * @param optMsg 处理意见
     * @return result对象
     */
    @POST
    @Path("agreeFriendReq")
    @Override
    public Result agreeFriendReq(@FormParam("reqId") Long reqId,
                                 @FormParam("optMsg") String optMsg) {
        FriendReqDO reqDO = socialRelationService.getFriendReqById(reqId);

        // 校验记录是否存在
        if (reqDO == null) {
            return new Result(GlobalApiError.OperationFailed, "好友申请记录不存在");
        }

        // 是否等待处理
        if (reqDO.getStatus() != FriendReqStatusEnum.WAIT.getCode()) {
            return new Result(GlobalApiError.OperationFailed, "好友申请状态仅等待时才可操作");
        }

        // 是否本人
        if (!reqDO.getTargetId().equals(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "仅本人才可操作");
        }
        // 同意
        if (socialRelationService.agreeFriendReq(reqId, optMsg)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 拒绝好友请求
     *
     * @param reqId  好友请求id
     * @param optMsg 处理意见
     * @return result对象
     */
    @POST
    @Path("denyFriendReq")
    @Override
    public Result denyFriendReq(@FormParam("reqId") Long reqId,
                                @FormParam("optMsg") String optMsg) {

        // 校验记录是否存在
        FriendReqDO reqDO = socialRelationService.getFriendReqById(reqId);
        if (reqDO == null) {
            return new Result(GlobalApiError.OperationFailed, "好友申请记录不存在");
        }

        // 是否等待处理
        if (reqDO.getStatus() != FriendReqStatusEnum.WAIT.getCode()) {
            return new Result(GlobalApiError.OperationFailed, "好友申请状态仅等待时才可操作");
        }

        // 是否本人同意
        if (!reqDO.getTargetId().equals(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "仅本人才可操作");
        }

        // 拒绝
        if (socialRelationService.denyFriendReq(reqId, optMsg)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 删除好友
     *
     * @param targetId 用户id
     * @return result对象
     */
    @POST
    @Path("delFriend")
    @Override
    public Result delFriend(@FormParam("targetId") Long targetId) {
        if (!socialRelationService.isFriend(this.getToken().getUserId(), targetId)) {
            return new Result(GlobalApiError.OperationFailed, "对方不是你的好友，先加好友试试吧");
        }
        if (socialRelationService.delFriend(this.getToken().getUserId(), targetId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取用户关注列表
     *
     * @param pageIndex 当前页码
     * @param userId
     * @return result对象
     */
    @GET
    @Path("getFollowList")
    @Override
    public Result getFollowList(@QueryParam("pageIndex") int pageIndex,
                                @QueryParam("userId") Long userId) {
        if (userId == null) {
            // 自己
            userId = this.getToken().getUserId();
        } else {
            if (!userService.isUserExist(userId)) {
                return new Result(GlobalApiError.OperationFailed, "无效用户");
            }

            // 判断是否隐藏关注
            int hideFollower = userService.getSwitchStatus(userId, SwitchTypeEnum.HIDE_FOLLOWERS);
            if (BoolIntVal.True == hideFollower) {
                return new Result(GlobalApiError.OperationFailed, "该用户设置了隐藏关注");
            }
        }

        PageList<FollowUser> pageList = socialRelationService.getFollowList(this.getToken().getUserId(), userId, pageIndex, GlobalConfig.PAGE_SIZE);
        if (pageList.getTotal() > 0) {
            // 如果需要翻译
            if (I18nTypeEnum.ZH_CN != I18nTypeEnum.toEnum(this.getHeaderString("lang"))) {
                for (FollowUser followUser : pageList.getList()) {
                    followUser.setAreaName(PinyinUtil.firstCharUpperCase(PinyinUtil.convert(followUser.getAreaName())));
                }
            }
        }

        return new Result(pageList);
    }

    /**
     * 获取用户粉丝列表
     *
     * @param pageIndex 当前页码
     * @param userId
     * @return result对象
     */
    @GET
    @Path("getFansList")
    @Override
    public Result getFansList(@QueryParam("pageIndex") int pageIndex,
                              @QueryParam("userId") Long userId) {

        if (userId == null) {
            userId = this.getToken().getUserId();
        }

        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.OperationFailed, "无效用户");
        }

        if (!userId.equals(this.getToken().getUserId())) {
            if (BoolIntVal.False == userService.getSwitchStatus(userId, SwitchTypeEnum.VIEW_FANS)) {
                return new Result(GlobalApiError.OperationFailed, "该用户设置了隐藏粉丝");
            }
        }

        PageList<FansUser> pageList = socialRelationService.getFansList(this.getToken().getUserId(), userId, pageIndex, GlobalConfig.PAGE_SIZE);
        if (pageList.getTotal() > 0) {
            // 如果需要翻译
            if (I18nTypeEnum.ZH_CN != I18nTypeEnum.toEnum(this.getHeaderString("lang"))) {
                for (FansUser fansUser : pageList.getList()) {
                    if (StringUtils.isNotBlank(fansUser.getAreaName())) {
                        fansUser.setAreaName(PinyinUtil.firstCharUpperCase(PinyinUtil.convert(fansUser.getAreaName())));
                    }
                }
            }
        }

        return new Result(pageList);
    }

    /**
     * 获取用户好友列表
     *
     * @param pageIndex 当前页码
     * @param userId
     * @return result对象
     */
    @GET
    @Path("getFriendList")
    @Override
    public Result getFriendList(@QueryParam("pageIndex") int pageIndex,
                                @QueryParam("userId") Long userId) {
        if (userId == null) {
            userId = this.getToken().getUserId();
        }

        if (!userService.isUserExist(userId)) {
            return new Result(GlobalApiError.OperationFailed, "无效用户");
        }

        return new Result(socialRelationService.getFriendList(userId, pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取好友申请列表
     *
     * @param pageIndex 当前页码
     * @return result对象
     */
    @GET
    @Path("getFriendReqList")
    @Override
    public Result getFriendReqList(@QueryParam("pageIndex") int pageIndex) {
        return new Result(socialRelationService.getFriendReqList(this.getToken().getUserId(), pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取访客列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getVisitorList")
    @Override
    public Result getVisitorList(@QueryParam("pageIndex") int pageIndex) {
        return new Result(userService.getVisitorList(this.getToken().getUserId(), pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取足迹列表
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getIntervieweeList")
    @Override
    public Result getIntervieweeList(@QueryParam("pageIndex") int pageIndex) {
        return new Result(userService.getIntervieweeList(this.getToken().getUserId(), pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 最近看过的用户列表
     *
     * @return
     */
    @GET
    @Path("getRecentlySeenUsers")
    @Override
    public Result getRecentlySeenUsers() {
        Long userId = this.getToken().getUserId();
        return Result.ofList(userService.getRecentlySeenUsers(userId, 5));
    }

    /**
     * 我看过谁
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getVisitorListV2")
    @Override
    public Result getVisitorListV2(@QueryParam("pageIndex") Integer pageIndex,
                                   @QueryParam("pageSize") Integer pageSize) {

        if (pageIndex == null) {
            pageIndex = 1;
        }

        if (pageSize == null) {
            pageSize = GlobalConfig.PAGE_SIZE;
        }

        return new Result(userService.getVisitorListV2(this.getToken().getUserId(), pageIndex, pageSize));
    }

    /**
     * 谁看过我
     *
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getIntervieweeListV2")
    @Override
    public Result getIntervieweeListV2(@QueryParam("pageIndex") Integer pageIndex,
                                       @QueryParam("pageSize") Integer pageSize) {

        if (pageIndex == null) {
            pageIndex = 1;
        }

        if (pageSize == null) {
            pageSize = GlobalConfig.PAGE_SIZE;
        }
        return new Result(userService.getIntervieweeListV2(this.getToken().getUserId(), pageIndex, pageSize));
    }

    /**
     * 谁看过我总数
     *
     * @return
     */
    @GET
    @Path("getIntervieweeTotal")
    @Override
    public Result getIntervieweeTotal() {
        MutableResult result = new MutableResult();
        Long userId = this.getToken().getUserId();
        result.put("total", userService.getIntervieweeTotal(userId));
        result.put("today", userService.getIntervieweeToday(userId));
        return result.build();
    }

    /**
     * 未读访客消息数
     *
     * @return
     */
    @GET
    @Path("getIntervieweeUnReadMsgCount")
    @Override
    public Result getIntervieweeUnReadMsgCount() {
        Long userId = this.getToken().getUserId();
        return Result.ofSuccess(userService.getIntervieweeUnReadMsgCount(userId));
    }

    /**
     * 添加黑名单
     *
     * @param userId
     * @return
     */
    @POST
    @Path("addBlack")
    @Override
    public Result addBlack(@FormParam("userId") Long userId) {
        Long curUserId = this.getToken().getUserId();
        if (socialRelationService.addBlack(curUserId, userId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 删除黑名单
     *
     * @param userId
     * @return
     */
    @POST
    @Path("delBlack")
    @Override
    public Result delBlack(@FormParam("userId") Long userId) {
        Long curUserId = this.getToken().getUserId();
        if (socialRelationService.delBlack(curUserId, userId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @param lastSeq
     * @return
     */
    @GET
    @Path("getBlackList")
    @Override
    public Result getBlackList(@QueryParam("pageIndex") int pageIndex,
                               @QueryParam("lastSequence") Integer lastSeq) {
        if (lastSeq == null) {
            lastSeq = 0;
        }
        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        List<UserBlackDTO> blackList = imService.getBlackList(userId, pageIndex, GlobalConfig.PAGE_SIZE, lastSeq);
        if (CollectionUtil.isEmpty(blackList)) {
            mResult.put("list", new ArrayList<>());
            mResult.put("curSeq", 0);
            return mResult.build();
        }
        Map<Long, Long> userBlackMap = blackList.stream().collect(Collectors.toMap(UserBlackDTO::getUserId, UserBlackDTO::getBlackTime));
        Integer curSeq = blackList.get(0).getCurSeq();
        List<UserCardVO> list = userService.getNameAndAvatarByIds(blackList.stream().map(UserBlackDTO::getUserId).collect(Collectors.toList()));
        List<UserBlackVO> blackVOList = new ArrayList<>();
        for (UserCardVO item : list) {
            UserBlackVO blackVO = new UserBlackVO();
            BeanUtils.copyProperties(item, blackVO);
            blackVO.setBlackTime(userBlackMap.get(item.getUserId()));
            blackVOList.add(blackVO);
        }
        mResult.put("list", blackVOList);
        mResult.put("curSeq", curSeq);
        return mResult.build();
    }

    /**
     * 获取附近的人
     *
     * @return
     */
    @GET
    @Path("getPeopleNearby")
    @Override
    public Result getPeopleNearby() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", getPeopleNearbyList());
        return mResult.build();
    }

    /**
     * 获取附近的人分页数据
     *
     * @param pageIndex  当前页码
     * @param userSex    用户性别 1：男 2：女
     * @param userAgeMin 最小年龄
     * @param userAgeMax 最大年龄
     * @return
     */
    @Override
    @GET
    @Path("getPeopleNearbyPageList")
    public Result getPeopleNearbyPageList(@QueryParam("pageIndex") int pageIndex,
                                          @QueryParam("userSex") Integer userSex,
                                          @QueryParam("userAgeMin") Integer userAgeMin,
                                          @QueryParam("userAgeMax") Integer userAgeMax
    ) {
        Double latitude = Convert.toDouble(this.getHeader().getLatitude());
        Double longitude = Convert.toDouble(this.getHeader().getLongitude());
        if (latitude == null || longitude == null) {
            IpLocation ipLocation = areaService.getIpLocation(this.getIpAddress());
            if (ipLocation != null) {
                latitude = Optional.ofNullable(ipLocation.getCenterLatitude()).orElse(ipLocation.getLbLatitude());
                longitude = Optional.ofNullable(ipLocation.getCenterLongitude()).orElse(ipLocation.getLbLongitude());
            }
            if (latitude == null && longitude == null) {
                return new Result(GlobalApiError.OperationFailed, "未能获取您的位置，请先开启定位后重试！");
            }
        }

        NearUserDTO userDTO = new NearUserDTO();
        userDTO.setPageIndex(pageIndex);
        userDTO.setUserLatitude(latitude);
        userDTO.setUserLongitude(longitude);
        userDTO.setPageSize(20);
        if (userSex != null) {
            userDTO.setUserSex(userSex);
        }
        if (userAgeMin != null) {
            userDTO.setUserAgeMin(userAgeMin);
        }
        if (userAgeMax != null) {
            userDTO.setUserAgeMax(userAgeMax);
        }
        userDTO.setMyselfUserId(this.getToken().getUserId());

        return new Result(peopleNearbyProvider.getPeopleNearbyPageList(userDTO));
    }

    /**
     * 获得搜索默认关键词
     *
     * @return 搜索默认关键词
     */
    @GET
    @Path("getSearchDefaultKeywords")
    @PermitAll
    @Override
    public Result getSearchDefaultKeywords() {
        return new Result(userService.getSearchDefaultKeywords());
    }

    /**
     * 猜你喜欢列表
     *
     * @return
     */
    @GET
    @Path("getMayLike")
    @Override
    public Result getMayLike() {
        MutableResult mResult = new MutableResult();
        List<PeopleNearbyVO> peopleNearbyVOS = getPeopleNearbyList();
        List<MayLikeVO> mayLikeVOS = new ArrayList<>(peopleNearbyVOS.size());
        peopleNearbyVOS.forEach(p -> {
            MayLikeVO mayLikeVO = new MayLikeVO();
            BeanUtils.copyProperties(p, mayLikeVO);
            mayLikeVOS.add(mayLikeVO);
        });
        mResult.put("list", mayLikeVOS);
        return mResult.build();
    }

    /**
     * 附近的人列表
     *
     * @return
     */
    private List<PeopleNearbyVO> getPeopleNearbyList() {
        if (isDev()) {
            log.info("附近的人列表Header {}", LogUtil.obj2JsonObj(getHeader()));
        }
        Double latitude = Convert.toDouble(this.getHeader().getLatitude());
        Double longitude = Convert.toDouble(this.getHeader().getLongitude());
        if (latitude == null || longitude == null) {
            IpLocation ipLocation = areaService.getIpLocation(this.getIpAddress());
            if (isDev()) {
                log.info("附近的人列表ipLocation IP={} {}", this.getIpAddress(), LogUtil.obj2JsonObj(ipLocation));
            }
            if (ipLocation != null) {
                latitude = Optional.ofNullable(ipLocation.getCenterLatitude()).orElse(ipLocation.getLbLatitude());
                longitude = Optional.ofNullable(ipLocation.getCenterLongitude()).orElse(ipLocation.getLbLongitude());
            }
        }

        List<PeopleNearbyVO> list;
        if (latitude == null || longitude == null) {
            Map<String, String> girlMap = new HashMap<>();
            girlMap.put("蓝修洁", "http://user.qn.panygo.com/avatar/272849960960000/1608085709362.png");
            girlMap.put("秋秋秋秋", "http://user.qn.panygo.com/avatar/196211134955520/1607671086896.png");
            girlMap.put("小仙女的棒棒糖", "http://user.qn.panygo.com/avatar/172216444518400/1607670929017.png");
            girlMap.put("星星", "http://user.qn.panygo.com/avatar/145409557135360/1607668329275.png");
            girlMap.put("菜菜", "http://user.qn.panygo.com/avatar/143225885032448/1607668201014.png");
            girlMap.put("文^_^", "http://user.qn.panygo.com/avatar/141075591200768/1608011097623.png");

            Map<String, String> boyMap = new HashMap<>();
            boyMap.put("哇", "http://user.qn.panygo.com/avatar/272849684135936/1608084432133.png");
            boyMap.put("长留上仙", "http://user.qn.panygo.com/avatar/176738868920320/1608085278027.png");

            String[] distances = new String[]{"200m", "350m", "920m", "100m", "700m", "90m", "470m", "703m"
                    , "21.9km", "3.3km", "60.9km", "5.3km", "13.1km", "91.9km", "13.1km", "31.3km", "51.9km"};

            list = new ArrayList<>();
            girlMap.forEach((name, avatar) -> {
                PeopleNearbyVO vo = new PeopleNearbyVO();
                vo.setUserName(name);
                vo.setUserAvatar(avatar);
                vo.setUserSex(SexEnum.FEMALE.getCode());
                vo.setDistance(distances[RandomUtils.nextInt(0, distances.length - 1)]);
                list.add(vo);
            });

            boyMap.forEach((name, avatar) -> {
                PeopleNearbyVO vo = new PeopleNearbyVO();
                vo.setUserName(name);
                vo.setUserAvatar(avatar);
                vo.setUserSex(SexEnum.MALE.getCode());
                vo.setDistance(distances[RandomUtils.nextInt(0, distances.length - 1)]);
                list.add(vo);
            });
        } else {
            NearUserDTO userDTO = new NearUserDTO();
            userDTO.setPageIndex(1);
            userDTO.setPageSize(20);
            userDTO.setUserLatitude(latitude);
            userDTO.setUserLongitude(longitude);
            userDTO.setMyselfUserId(this.getToken().getUserId());
            list = peopleNearbyProvider.getPeopleNearbyPageList(userDTO).getList();
        }
        return list;
    }
}
